; ----------------- assembly instructions ---------------------------- 
;
; this is a subroutine library only
; it must be included in an executable source file
;
; The following resources are called:
;
; Local Zero Page Variables used
;
; Serial         1 byte
; HexDigCnt      1 byte
; Addrptr        2 bytes
; Addrptr_H       
; HexDigits      2 bytes
; HexDigits_H    
;
; Local Non-Zero Page Variables used
;
; ChrInVect      holds Character Input Address (3 bytes - low, high, high EOR A5)
; ScanInVect     holds Character Scan Input address (3 bytes)
; ChrOutVect     holds Character Output Address (3 bytes)
;
; Support Routines Called
;
; Print1byte    prints one byte in character format to Output
; Output        used to send characters to the output port
; Scan_Input    scans the input port for new data, does not wait
; Print_CR      prints CR/LF to the Output port
; Input_chr     gets a character from input port, waits for input
;
;*** I/O Locations *******************************
; define the i/o address of the 6526 chip
;*** 6526 CIA ************************
Cia_PRA        =     $7f00
Cia_PRB        =     $7f01
Cia_DDRA       =     $7f02
Cia_DDRB       =     $7f03
Cia_TALO       =     $7f04
Cia_TAHI       =     $7f05
Cia_TBLO       =     $7f06
Cia_TBHI       =     $7f07
Cia_TODT       =     $7f08
Cia_TODS       =     $7f09
Cia_TODM       =     $7f0a
Cia_TODH       =     $7f0b
Cia_SDR        =     $7f0c
Cia_ICR        =     $7f0d
Cia_CRA        =     $7f0e
Cia_CRB        =     $7f0f
Cia_rxdata     =     $01 
Cia_rxclk      =     $02 
Cia_rxmask     =     $FC
Cia_txdata     =     $04 
Cia_txclk      =     $08
Cia_txmask     =     $F3 
;
;***********************************************************************
; 6526 I/O Support Routines
;
Cia_init       LDX   #<Cia_Input       ; set up RAM vectors for
               LDA   #>Cia_Input       ; Input, Output, and Scan
               TAY                     ; Routines
               EOR   #$A5              ;
               sta   ChrInVect+2       ;
               sty   ChrInVect+1       ;
               stx   ChrInVect         ;
               LDX   #<Cia_Scan        ;
               LDA   #>Cia_Scan        ;
               TAY                     ;
               EOR   #$A5              ;
               sta   ScanInVect+2      ;
               sty   ScanInVect+1      ;
               stx   ScanInVect        ;
               LDX   #<Cia_Output      ;
               LDA   #>Cia_Output      ;
               TAY                     ;
               EOR   #$A5              ;
               sta   ChrOutVect+2      ;
               sty   ChrOutVect+1      ;
               stx   ChrOutVect        ;
               ldx   #$00              ; get data from table
Cia_init1      lda   Cia_idata,x       ; init all 16 regs from 00 to 0F
               sta   Cia_PRA,x         ; 
               inx                     ; 
               cpx   #$10              ; 
               bne   Cia_init1         ;       
               stz   Cia_TODT          ; ensure clock starts running
               lda   Cia_TODT          ; clr read latch
               rts                     ; done
;
Cia_idata      .byte $ff               ; pra  "11111111'
               .byte $fc               ; prb  '11111100'
               .byte $ff               ; ddra set bits 0-7 for output
               .byte $cc               ; ddrb set bits 0-1 as input, 2-7 as output
               .byte $00               ; talo  not used
               .byte $00               ; tahi  not used
               .byte $10               ; tblo  TOD timer (1Mhz/100 toggles=10,000 or 2710h) 
               .byte $27               ; tbhi   "    "   (100 toggles = 50Hz output at PB7)
               .byte $01               ; todt  init clock to 12:00:00.1AM
               .byte $00               ; tods
               .byte $00               ; todm
               .byte $00               ; todh
               .byte $00               ; sdr   
               .byte $7f               ; icr  disable all interrupts
               .byte $80               ; cra  set TOD input to 50Hz
               .byte $07               ; crb  set timer b on, pb7 on, toggle mode, clk2 source 
                                       ;      used for 50hz TOD clock
;
;
; input chr from 6526
Cia_Input      ;sei
               phx                     ; save x reg
               stz   serial            ;
               ldx   #$08              ; init shift bit counter for 8 bits
input_lp1      lda   Cia_prb           ;
               and   #Cia_rxclk        ; port b, bit 1
               bne   input_lp1         ; wait for rxclk to go low
               lda   Cia_prb           ; 
               and   #Cia_txmask       ; mask port b2 txdata to zero
               ora   #Cia_txclk        ;
               sta   Cia_prb           ; set txdata low - mark ready to rx
input_lp2      lda   Cia_prb           ;
               and   #Cia_rxclk        ;
               beq   input_lp2         ; wait for rxclk to go high
               lda   Cia_prb           ;
               and   #Cia_rxdata       ; mask data bit
               lsr                     ; move to carry flag
               rol   serial            ; roll into memory, high bit first
               lda   Cia_prb           ; 
               ora   #Cia_txdata       ; mask port b2 txdata to one
               sta   Cia_prb           ; set txdata high, mark data received
               dex                     ; reduce clk count
               bne   input_lp1         ; if 8 bits, done
               plx                     ; restore x reg
               lda   serial            ;
               ;cli
               RTS                     ;
;
; see if input requested without waiting
Cia_Scan       lda   Cia_prb           ; return 0 if input requested
               and   #Cia_rxclk        ; return non-zero if no input requrested
               rts                     ;
;
; output to OutPut Port
Cia_Output     ;sei
               sta  serial             ; save byte
               pha                     ; save registers
               phx                     ;
               phy                     ;
               ldx   #$08              ; init shift bit counter
output_lp1     lda   Cia_prb           ; 
               and   #Cia_txmask       ; mask port b3 to zero (txclk)
               ora   #Cia_txdata       ;
               sta   Cia_prb           ; set txclk low
output_lp2     lda   Cia_prb           ;
               and   #Cia_rxdata       ; poll rxdata bit (confirms term ready to rcv)
               bne   output_lp2        ; wait while rxdata bit is high
               asl   serial            ; roll character into C flag, high bit first
               bcs   output_lp3        ; leave bit 2 txdata high if carry high
               lda   Cia_prb           ; 
               and   #Cia_txmask       ; else set bit 2 txdata low
               sta   Cia_prb           ; store it to prb
output_lp3     nop                     ; pause
               lda   Cia_prb           ; 
               ora   #Cia_txclk        ; mask port b3 txclk to one
               sta   Cia_prb           ; set txclk high
output_lp4     lda   Cia_prb           ;
               and   #Cia_rxdata       ; poll rxdata bit (signals term got bit)
               beq   output_lp4        ; wait while rxdata bit is low
               lda   Cia_prb           ; set bit 2 high
               ora   #Cia_txdata       ; mask it with prb
               sta   Cia_prb           ; store it to prb
               dex                     ; reduce clk count
               bne   output_lp1        ; if <8 bits sent, send next bit
               ply                     ; restore registers
               plx                     ;
               pla                     ;
               ;cli
               RTS                     ; done
;
;****************************************************************************
; TOD clock routines - disp time & set time 
;
Disp_TClk      lda   hexdigcnt         ; if a number was entered,
               bne   Disp_clock        ; run the continuous loop
Disp_Time      lda   Cia_TODH          ; Print time once
               pha                     ; save hh for am/pm
               and   #$1f              ;
               jsr   Print1byte        ;
               lda   #$3a              ; ':'
               jsr   Output            ;
               lda   Cia_TODM          ;
               jsr   print1byte        ;
               lda   #$3a              ; ':'
               jsr   Output            ;
               lda   Cia_TODS          ; 
               jsr   print1byte        ;
               lda   Cia_TODT          ; release latch
               pla                     ;
               bpl   Disp_Time2        ;
               lda   #$50              ; 'P'
Disp_time1     jsr   Output            ;
               lda   #$4D              ; 'M'
               jsr   Output            ; 
               jmp   Print_CR          ; ? CR and return to calling routine
Disp_time2     lda   #$41              ; 'A'
               bra   Disp_Time1        ;
Disp_clock     jsr   Disp_Time         ; print time every second
               lda   Cia_TODS          ;
Disp_clk1      cmp   Cia_TODS          ;
               beq   Disp_clk1         ;
               jsr   Scan_Input        ; until a key is pressed
               bne   Disp_clock        ;
               jsr   input_chr         ; scrub input buffer
               rts                     ;
;
Set_time       lda   hexdigcnt         ; if full time not entered 
               cmp   #$FC              ; 'hhmm.sstt'
               bne   Disp_Time         ; just display the time
               lda   addrptr_H         ; get hours
               cmp   #$24              ; if hr >23, skip hours 
               bpl   Set_time4         ;
               cmp   #$22              ; if hr 22-23, subtract 12h 
               bpl   Set_time2         ;
               cmp   #$20              ; if hr 20-21, subtract 18h 
               bpl   Set_time1         ;
               cmp   #$13              ; if hr 13-19, subtract 12h 
               bpl   Set_time2         ; if hr=12, time will be 12PM
               cmp   #$00              ; is hr=00, set to 92h for 12AM
               bne   Set_time3         ;  * note, these 12's are backwords
               lda   #$92              ;  * from the rest of the PM's
               bra   Set_time3         ;
Set_time1      sbc   #$06              ;
Set_time2      sbc   #$12              ; 
               ora   #$80              ; set PM   
Set_time3      sta   Cia_TODH          ;
Set_time4      lda   addrptr           ;
               sta   Cia_TODM          ;
               lda   Hexdigits_H       ;
               sta   Cia_TODS          ;
               lda   Hexdigits         ;
               sta   Cia_TODT          ; store tenths and unlatch TOD write
               lda   Cia_TODT          ; read tenths unlatch TOD read
               jsr   Disp_time         ;
               jmp   Print_CR          ;

;end of file
