;
; DiskOS for SBC-3
; Written by Daryl Rictor (c)2009
;
; V1.4  4-14-2010
;
;
; System Error codes
;
;--- from ATM32 ---
;00 = no error
;01 = ATA init error
;02 = missing file name
;03 = Format error
;04 = File not found
;05 = Read Only File
;06 = invalid file type/open error
;07 = unable to erase all files?
;08 = file not open
;09 = file already open in another handle
;0A = write protected
;0B = disk full
;0C = root directory full
;0D = invalid command
;0E = Handle in use
;0F = read past EOF
;10 = File too large > 2^32
;11 = Cluster size too big?
;--- from this code ---
;1B = Xmodem error
;1C = Memory wrap from $7FFFF to $00000
;1D = (REMOVED)
;1E = (REMOVED)
;1F = general syntax


;
; Program init
;
DiskOS
Init_DOS_COLD
		lda	#SYSBANK		;
		PHA				;
		#PLB				; set DBR to $07
		lda	#$01			; high addr of stack
		#XBA				;
		lda	#$FF			; low addr of stack
		#TCS				; clear stac
		lda	#$00			; drive 0
		jsr	MountVol0		; init drive
Init_DOS	lda	#"/"
		sta	CurrPath
		lda	#$00
		sta	CurrPath+1		; Mark root dir as curr path
		jsr	ChaDir0			; go to root of drive 0
		jsr	LisVol			; show mounted volume
		ldx	#$00
Init_DOS2	lda	GreetTxt,x
		beq	Boot
		jsr	output
		inx
		bra	Init_DOS2	 
Boot		lda	#$CF			; Test File
		jsr	SendIDECMD
		ldx	#$ff
Boot0		inx
		lda	BootTxt+3,x
		beq	Boot02
		jsr	SendIDE
		bra	Boot0
Boot02		jsr	SendIDE			; null term
Boot03		jsr	IDEStat			; get status
		cmp	#$00
		bpl	Boot03			; Bit 7=0 - wait for command to be done		
		and	#$1F			; mask error code
		beq	Boot11
		cmp	#$0A			; protected?
		bne	Main_Loop		; file error, skip loading
Boot11		ldx	#$00
Boot1		lda	BootTxt,x
		beq	Boot2
		sta	buffer,x
		inx
		bra	Boot1
Boot2		jsr	LoadF			; load boot file
		jsr	IDEStat
		and	#$1F
		bne	Main_Loop		; error - abort autoexec
		ldy	#$03		
		#LDA_DIIL Addr			; lda [addr],y
		cmp	#"S"
		bne	Main_Loop
		iny		
		#LDA_DIIL Addr			; lda [addr],y
		cmp	#"Y"
		bne	Main_Loop
		iny
		#LDA_DIIL Addr			; lda [addr],y
		cmp	#"S"
		bne	Main_Loop
		jsr	Boot3			; execute autoexec		
		bra	Main_Loop						
Boot3		#JML	Addr
		rts
BootTxt		.byte	"LF AUTOEXECSYS", $00
GreetTxt	.byte	$0d, $0A, "SBC-3 DiskOS V1 Ready", $0d, $0a, $00

_OSReturn					; cc65 label
		lda	#SYSBANK		;
		PHA				;
		#PLB				; set DBR to $07
Main_Loop	lda	#$01			; high addr of stack
		#XBA				;
		lda	#$FF			; low addr of stack
		#TCS				; clear stack
		jsr	input
		lda	#$00
		sta	buffer,y		; null terminate the command line
		lda	Buffer
		cmp	#"?"			; help shortcut
		bne	parseCmd1
		jsr	Help
		bra	Main_Loop	
ParseCmd1	cmp	#"."			; .LX direct
		bne	ParseCmd11	
		lda	buffer,y		; need to shift input buffer over 1 chr
		sta	buffer+1,y		; 
ParseCmd10	lda	buffer-1,y
		sta	buffer,y
		dey		
		bne	ParseCmd10
		jsr	LoadExe			; load and execute
		bra	Main_Loop
ParseCmd11	ldy	#$00
		ldx	#$00
ParseCmd12 	lda	Cmd_TBL, x
		beq	ParseErr
		cmp	Buffer, Y
		beq	ParseCmd3
ParseCmd2	inx
		inx
		bra	ParseCmd12
ParseCmd3	lda	Cmd_TBL+1, x
		cmp	Buffer+1, Y
		bne	ParseCmd2
		jsr	ParseExec
		bra	Main_Loop		

ParseExec	jmp	(JMP_TBL,x)
		bra	Main_Loop		; safety

ParseErr	ldx	#$00
ParseErr1	lda	ErrTxt, x
		beq	Main_Loop
		jsr	output
		inx
		bra	ParseErr1
ErrTxt		.byte 	"Not a Valid Command", $0d, $0a, $00

Cmd_TBL		.byte 	"LD"			; List current Directory
		.byte 	"MD"			; Make sub directory in current directory
		.byte 	"CD"			; change directory
		.byte 	"LV"			; List Volume
		.byte 	"RN"			; rename
		.byte 	"DE"			; Delete File/Dir
		.byte 	"PR"			; protect
		.byte 	"UP"			; unprotect
		.byte 	"DT"			; show date and time
		.byte 	"FS"			; show free space
		.byte 	"LF"			; load file
		.byte 	"SF"			; save file
		.byte 	"RM"			; Read memory
		.byte 	"WM"			; write memory
		.byte 	"EX"			; execute memory
		.byte 	"FV"			; Format Volume
		.byte 	"SC"			; Set Clock
		.byte 	"MV"			; Mount Volume
		.byte 	"DI"			; disassemble
		.byte 	"DX"			; disassemblew/16 bit immediate values
		.byte 	"DL"			; xmodem download from PC	
		.byte 	"UL"			; xmodem upload to PC	
		.byte 	"RS"			; Read Sector (raw)
		.byte 	"LX"			; load and execute
_expcmd		.byte 	"~~"			; for patching in new commands
		.byte 	"~~"			; ""
		.byte 	"~~"			; ""
		.byte 	"~~"			; ""
		.byte 	$00, $00

Jmp_TBL		.word  	ListDir
		.word  	MakDir
		.word  	ChaDir
		.word  	LisVol
		.word  	Rename
		.word  	Delete
		.word  	Prot
		.word  	UnPro
		.word  	ShoDT
		.word  	FreeSp
		.word  	LoadF
		.word  	SaveF
		.word  	ReadM
		.word  	WriteM
		.word  	Exec
		.word  	FormatVol
		.word  	SetClock
		.word  	MountVol
		.word  	List_cmd
		.word  	List_cmdL
		.word  	Download
		.word  	Upload		
		.word  	ReadSec			; 
		.word  	LoadExe			; 
_expjmp		.word  	Main_Loop		; for patching in new commands
		.word  	Main_Loop		; ""
		.word  	Main_Loop		; ""
		.word  	Main_Loop		; ""

;
; List Directory
;
ListDir		lda	#$F0
		jsr	SendIDECmd		; Vol Label
		jsr	PrintResult
		jsr	Print_CR
		jsr	PrintCurr		; curr directory
		jsr	Print_CR
		lda	#$C1			; List Dir
		jsr	SendIDECmd
		jsr	PrintResult
		jsr	Print_CR
		jmp	FreeSP

;
; make Directory
;
MakDir		lda	#$C2
		jmp	MultiUse

;
; Change directories
;
ChaDir		jsr	ParsePath
ChaDir0		ldx	#$00
		lda	#$C3			; Change Directory
		jsr	SendIDECmd
		lda	#"/"
		jsr	SendIDE
		bra	ChaDir3			; jump into loop
ChaDir1		lda	CurrPath,x
		beq	ChaDir5			; done
		lda	#$C3			; Change Directory
		jsr	SendIDECmd
ChaDir2		lda	CurrPath,x
		cmp	#"/"
		beq	ChaDir3
		jsr	SendIDE			; if not /, send it
		inx				; next char	
		bra	ChaDir2			; do again
ChaDir3		inx				; end of dir 
		lda	#$00
		jsr	SendIDE			; terminate and execute
ChaDir4		jsr	IDEStat			; get status
		cmp	#$00
		bpl	ChaDir4			; wait for ready
		and	#$1F
		beq	ChaDir1			; ok, get next
		phx
		jsr	printErr		; print error
		plx
ChaDir45	dex
		lda	CurrPath-1,x
		cmp	#"/"
		bne	ChaDir45
		stz	CurrPath,x		; truncate the path
ChaDir5		rts				; done


;
; List volume/drive info
;
LisVol		lda	#$F3
		jsr	SendIDECmd
		jsr	PrintResult
		jsr	Print_CR
		rts

;
; Rename File, Directory, or Volume
;
Rename		jsr	ParsePath
		cpy	#$00
		beq	Rename57		; empty, error
		dex				; now on /
		beq	Rename57		; just / , error
		stz	CurrPath,x		; remove /
Rename2		dex
		lda	CurrPath,x
		cmp	#"/"
		bne	Rename2			; backup to beginning of file name
		inx
		lda	CurrPath,x
		pha				; save 1st char of file name	
		stz	CurrPath,x		; null term directory chain
		jsr	ChaDir0
		jsr	IDEStat			; get status
		and	#$1F
		beq	Rename3			; cd successful
		pla				; restore stack
		rts				; cd error
Rename3		iny				; test new name entry
		lda	Buffer,y		; find first chr of new name
		beq	Rename55		; blank new name
		cmp	#" "
		beq	Rename3			; strip leading spaces
		cmp	#","
		beq	Rename3			; strip , separators
		cmp	#"/"		
		beq	Rename55		; error
		phy				; save 1st chr pointer
Rename4		iny
		lda	Buffer,y
		beq	Rename6			; if null, done
		cmp	#" "
		beq	Rename5			; if space, error
		cmp	#","
		beq	Rename5			; if , error
		cmp	#"/"			
		bne	Rename4			; if / error, else get next
Rename5		ply				; restore stack
Rename55	pla				; restore stack
Rename57	lda	#$1F	
		jsr	PrintErr
		rts
Rename6		ply				; restore y
		lda	#$C8
		jsr	SendIDECmd		; rename
		pla
		bra	Rename75		; jump into loop	
Rename7		lda	CurrPath,x
Rename75	beq	Rename8
		jsr	SendIDE
		inx
		bra	Rename7
Rename8		jsr	SendIDe			; send Null term
Rename82	lda	Buffer,y
		beq	Rename9
		jsr	SendIDE			; send new name
		iny
		bra	Rename82		
Rename9		jsr	SendIDE			; send null term new name
Rename92	jsr	IDEStat			; get status
		cmp	#$00
		bpl	Rename92		; Bit 7=0 - wait for command to be done		
		and	#$1F			; mask error code
		beq	Rename95		; done
		jsr	PrintErr		; print error
Rename95	rts		

;
; Delete File/Dir
;
Delete		lda	#$C4
		jmp	MultiUse

;
; Protect file/dir
;
Prot		lda	#$C6
		jmp	MultiUse

;
; unProtect File
;
UnPro		lda	#$C7
		jmp	MultiUse

;
; Multi-use Command Subroutine
;
MultiUse	sta	Temp			; Command passed in A
		jsr	ParsePath		;
		cpy	#$00			;
		bne	MultiUse0		;
		jsr	ChaDir0			; align currpath
		rts				; parse error - halt
MultiUse0	dex				; now on /
                stz	CurrPath,x		; remove ending / from path
MultiUse1	dex				;
		lda	CurrPath,x		;
		cmp	#"/"			;
		bne	MultiUse1		;
		inx				; move to next char
		txa				;
		tay				; save x pointer in Y		
		lda	CurrPath,x		; get first char
		pha				; save it for later
		stz	CurrPath,x		; null term (remove last dir in path - new dir)
		jsr	ChaDir0			; change to that DIR
		jsr	IDEStat			; get status
		and	#$1F			; mask error code
		beq	MultiUse2		; no errors
		plx				; fix stack
		rts				; CD failed
MultiUse2	lda	Temp			; Command code
		jsr	SendIDECmd		;
		pla				;
		bra	MultiUse33		; jump into loop
MultiUse3	iny				; get dir name until null term found
		lda	CurrPath,y		; 
MultiUse33	beq	MultiUse4		;
		jsr	SendIDE			; send to IDE
		bra	MultiUse3		;
MultiUse4	lda	#$00			; 
		jsr	SendIDE			; null terminate and execute
MultiUse5	jsr	IDEStat			; get status
		CMP	#$00			;
		bpl	MultiUse5		; Bit 7=0 - wait for command to be done		
		and	#$1F			; mask error code
		beq	MultiUse6		; done
		jmp	PrintErr		; print error
MultiUse6	rts		

;
; show the date and time
;
ShoDT		lda	#$E0			
		jsr	SendIDECmd
		jsr	PrintResult
		jsr	Print_CR
		rts

;
; Drive Free Space
;
FreeSp		lda	#$F1			
		jsr	SendIDECmd
		jsr	PrintResult
		jsr	Print_CR
		rts

;
; Load File into RAM
;
LoadF		jsr	ParsePath		;
		cpy	#$00			;
		bne	LoadF0			;
		jsr	ChaDir0			; align currpath
		rts				; parse error - halt
LoadF0		phy				; save pointer to buffer for ,$xxxx
		dex				; now on /
                stz	CurrPath,x		; remove ending / from path
LoadF1		dex				;
		lda	CurrPath,x		;
		cmp	#"/"			;
		bne	LoadF1			;
		inx				; move to next char
		txa				;
		tay				; save x pointer in Y		
		lda	CurrPath,x		; get first char
		pha				; save it for later
		stz	CurrPath,x		; null term (remove last dir in path - file name)
		jsr	ChaDir0			; change to that DIR
		jsr	IDEStat			; get status
		and	#$1F			; mask error code
		beq	LoadF2			; no errors
		plx				; fix stack
		plx				; fix stack
		sec
		rts				; CD failed
LoadF2		lda	#$C9			; Open File Command
		jsr	SendIDECmd		;
		lda	#$1F			; Read - Handle F
		jsr	SendIDE			;
		pla				;
		bra	LoadF33			; jump into loop
LoadF3		iny				; get dir name until null term found
		lda	CurrPath,y		; 
LoadF33		beq	LoadF4			;
		jsr	SendIDE			; send to IDE
		bra	LoadF3			;
LoadF4
		lda	#$00
		jsr	SendIDE			; null terminate and execute				
LoadF5		jsr	IDEStat			; get status
		cmp	#$00
		bpl	LoadF5			; wait for ready
		and	#$1F
		beq	LoadF52			; ok, get next
		jsr	PrintErr		;
		plx				; fix stack
		sec
		rts
LoadF52		lda	#$D0			; get file size
		jsr	SendIDECmd	
		lda	#$1F			; read handle 15
		jsr	SendIDE
		nop
		nop
		nop
		jsr	RXIDE			; get low byte of size
		sta	Ptr
		nop
		nop
		nop
		jsr	RXIDE			; get second byte
		sta	Ptr+1
		nop
		nop
		nop
		jsr	RXIDE			; get third
		sta	Ptr+2
		cmp	#$07			; our max size is $07FFFF
		bcc	LoadF55			; 
LoadF53		plx				; fix stack
		nop
		nop
		jsr	RXIDE			; null
LoadF54		lda	#$10			;file to big error
		jsr	PrintErr
		jmp	LoadF92			; close and exit
LoadF55		nop
		nop
		nop
		jsr	RXIDE			; get 4th byte of length
		bne	LoadF54			; anything other than 0 is too large
		sec
		lda	Ptr
		sbc	#$04
		lda	Ptr+1
		sbc	#$00
		lda	Ptr+2
		sbc	#$00			; test for more than 4 bytes (3 address + 1 data)
		bcs	LoadF6
		plx				; correct stack
		jmp	LoadF92			; nothing to load, close and exit
LoadF6		lda	#$CA			; Read File
		jsr	SendIDECmd
		lda	#$1F			; handle
		jsr	SendIDE
		lda	ptr
		jsr	SendIDE			;
		lda	ptr+1
		jsr	SendIDE			;
		lda	ptr+2
		jsr	SendIDE			;
		lda	#$00
		jsr	SendIDE			; 4th byte = 0
		ply				; restore pointer to buffer
		lda	buffer,y
		beq	LoadF62			; no extra, use default
		cmp	#","
		bne	LoadF62			; extra ignored
		lda	Ptr
		pha
		lda	Ptr+1
		pha
		lda	Ptr+2
		pha
		iny
		jsr	getHexDig
		bcs	LoadF68			; no digits, use 1st 3 from file
		lda	ptr			; override file's start address with the user's input
		sta	addr
		lda	ptr+1
		sta	addr+1
		lda	ptr+2
		sta	addr+2
		pla
		sta	Ptr+2
		pla
		sta	Ptr+1
		pla
		sta	Ptr		
		bra	LoadF71

LoadF68		pla
		sta	Ptr+2
		pla
		sta	Ptr+1
		pla
		sta	Ptr

LoadF62		nop				; y points to buffer
		jsr	RXIde			; get low byte of address
		sta	Addr
		nop
		nop
		jsr	RXide		
		sta	Addr+1
		nop
		nop
		jsr	RXIDE
		sta	Addr+2			;
		sec
		lda	ptr
		sbc	#$03
		sta	ptr
		lda	ptr+1
		sbc	#$00
		sta	ptr+1
		lda	ptr+2
		sbc	#$00
		sta	ptr+2			; ptr=ptr-3 (3 address)
LoadF71		ldx	#$00
LoadF72		lda	Loadtxt,x
		beq	loadF73
		jsr	output
		inx
		bra	loadF72
loadF73		lda	addr+2
		jsr	printdig
		lda	addr+1
		jsr	print1byte
		lda	addr
		jsr	print1byte
loadF74		inx
		lda	Loadtxt,x
		beq	LoadF75
		jsr	output
		bra	LoadF74
LoadF75		lda	Ptr+2
		jsr	printdig
		lda	Ptr+1
		jsr	print1byte
		lda	Ptr
		jsr	print1byte
		jsr	Print_cr
		sec
		lda	ptr
		sbc	#$01
		sta	ptr
		lda	ptr+1
		sbc	#$00
		sta	ptr+1
		lda	ptr+2
		sbc	#$00
		sta	ptr+2			; ptr=ptr-1 ( 1st data byte)
		lda	addr+2
		pha
		lda	addr+1
		pha
		lda	addr			; save start address for end
		pha
LoadF76		jsr	RXIde			; get next data byte
		sta	Temp
		#PHB				; save DBR
		lda	Addr+2			; get data DB
		PHA				; prep
		#PLB				; set it
		LDA	Temp			; get data byte
		sta	(Addr)			; save it
		#PLB				; restore DBR
		inc	Addr
		bne	LoadF8
;		lda	#"*"
;		jsr	output
		inc	Addr+1
		bne	LoadF8
;		lda	#"+"
;		jsr	output
		inc	Addr+2
		lda	Addr+2
		cmp	#$08			; end of mem?
		bcc	LoadF8
		bra	loadF79
loadF78		jsr	RXIDE			; flush rest of file		
LoadF79		sec
		lda	ptr			; ptr=ptr-1
		sbc	#$01
		sta	ptr
		lda	Ptr+1
		sbc	#$00
		sta	Ptr+1
		lda	Ptr+2
		sbc	#$00
		sta	Ptr+2
		bcs	LoadF78			; get next
		lda	#$1C	
		jsr	PrintErr		; memory wrap error
		bra	loadF9			; close and exit		
LoadF8		sec
		lda	ptr			; ptr=ptr-1
		sbc	#$01
		sta	ptr
		lda	Ptr+1
		sbc	#$00
		sta	Ptr+1
		lda	Ptr+2
		sbc	#$00
		sta	Ptr+2
		bcs	LoadF76			; get next
LoadF82		jsr	IDEStat
		and	#$1F
		beq	LoadF9			; get next byte
		cmp	#$0F			; end of file?
		beq	LoadF9
		jsr 	PrintErr
LoadF9		pla				; restore start address
		sta	Addr
		pla
		sta	Addr+1
		pla
		sta	Addr+2
LoadF92		lda	#$CE			; close file
		jsr	SendIDECMD
		lda	#$1F			; handle
		jsr	SendIDE
LoadF95		jsr	IDEStat			; get status
		cmp	#$00
		bpl	LoadF95			; wait for ready
		and	#$1F
		beq	LoadF99			; get next byte
		jsr	PrintErr
		sec
		rts
LoadF99		
		clc
                rts
LoadTxt		.byte	"Start: ", $00, "  Length: ", $00

LoadExe		jsr	LoadF			; load file and execute from load address
		bcs	LoadExe1
		lda	Addr
		sta	Ptr
		lda	Addr+1
		sta	Ptr+1
		lda	Addr+2
		sta	Ptr+2
		jmp	Exec0
LoadExe1	rts

;
; Save RAM to Disk
;
SaveF		jsr	ParsePath		;
		cpy	#$00			;
		bne	SaveF0			;
		jsr	ChaDir0			; align currpath
		rts				; parse error - halt
SaveF0		phy				; save pointer to buffer
		dex				; now on /
                stz	CurrPath,x		; remove ending / from path
SaveF1		dex				;
		lda	CurrPath,x		;
		cmp	#"/"			;
		bne	SaveF1			;
		inx				; move to next char
		txa				;
		tay				; save x pointer in Y		
		lda	CurrPath,x		; get first char
		phy				; save pointer to frst 
		sta	temp			; save it for later
		stz	CurrPath,x		; null term (remove last dir in path - new dir)
		jsr	ChaDir0			; change to that DIR
		jsr	IDEStat			; get status
		and	#$1F			; mask error code
		beq	SaveF2			; no errors
		ply				; fix stack
		ply				; fix stack
		rts				; CD failed

SaveF2		lda	#$CF			; Test File Command
		jsr	SendIDECmd		;
		lda	temp			; 1st char
		bra	SaveF33			; jump into loop
SaveF3		iny				; get dir name until null term found
		lda	CurrPath,y		; 
SaveF33		beq	SaveF4			;
		jsr	SendIDE			; send to IDE
		bra	SaveF3			;
SaveF4
		lda	#$00			; 
		jsr	SendIDE			; null terminate and execute				
SaveF5		jsr	IDEStat			; get status
		cmp	#$00
		bpl	SaveF5			; wait for ready
		and	#$1F
		beq	SaveF6			; exists
		cmp	#$04			; file not found
		beq	SaveF7			; 
		jsr	PrintErr		;
		ply
		ply
		rts
SaveF6
SaveF65		ldy	#$00
SaveF66		lda	OWPRompt,y
		beq	SaveF67
		jsr	Output
		iny
		bra	SaveF66	
SaveF67		jsr	INPUT_CHR
		and	#$DF			; force ucase
		cmp	#"Y"
		beq	SaveF7
		cmp	#"N"
		bne	SaveF65			; error do again	
		ply
		ply
		rts
SaveF7		lda	#$C9			; Open File Command
		jsr	SendIDECmd		;
		lda	#$2F			; Write - Handle F
		jsr	SendIDE			;
		lda	temp			;
		ply				;
		bra	SaveF73			; jump into loop
SaveF71		iny				; get dir name until null term found
		lda	CurrPath,y		; 
SaveF73		beq	SaveF75			;
		jsr	SendIDE			; send to IDE
		bra	SaveF71			;
SaveF75
		lda	#$00			; 
		jsr	SendIDE			; null terminate and execute				
SaveF77		jsr	IDEStat			; get status
		cmp	#$00
		bpl	SaveF77			; wait for ready
		and	#$1F
		beq	SaveF8			; good
		jsr	PrintErr
		ply
		rts
SaveF8		ply				; restore buffer pointer
		iny
		jsr	GetHexDig		; start address
		bcs	SaveF82
		lda	Ptr
		sta	Addr
		lda	Ptr+1
		sta	Addr+1
		lda	Ptr+2
		sta	Addr+2
		cmp	#$08
	  	bcc	SaveF83
SaveF81		lda	#$1C
		jsr	PrintErr
SaveF82		jmp	SaveF9			; close and exit
SaveF83		iny
		jsr	GetHexDig		; length *** change to ending addr
		bcc	SaveF84
		jmp	SaveF9
SaveF84		lda	Ptr+2
		cmp	#$08
		bcs	SaveF81
		sec
		lda	Ptr
		sbc	Addr
		sta	Ptr
		lda	Ptr+1
		sbc	Addr+1
		sta	Ptr+1
		lda	Ptr+2
		sbc	Addr+2
		sta	Ptr+2
		bcc	SaveF81			; start higher than end
		inc	Ptr
		bne	SaveF845
		inc	Ptr+1
		bne	SaveF845
		inc	Ptr+2			; add 1 for # bytes to save
SaveF845	clc
		lda	Ptr
		pha
		adc	#$03
		sta	Ptr
		lda	Ptr+1
		pha
		adc	#$00
		sta	Ptr+1
		lda	Ptr+2
		pha
		adc	#$00
		sta	Ptr+2			; add 3 for start address
		lda	#$CB			; write
		jsr	SendIDECMd		
		lda	#$2F 			; write handle F
		jsr	SendIDE			; 
		lda	ptr
		jsr	SendIDE
		lda	Ptr+1
		jsr	SendIDE
		lda	Ptr+2
		jsr	SendIDE
		lda	#$00
		jsr	SendIDE			; # bytes per write
		lda	Addr
		jsr	SendIde
		lda	Addr+1
		jsr	SendIDE
		lda	Addr+2
		jsr	SendIDE			; send start address
		pla
		sta	Ptr+2
		pla
		sta	Ptr+1
		pla
		sta	Ptr
SaveF85		#PHB				; save DBR
		lda	Addr+2			; get data DB
		PHA				; prep
		#PLB				; set it
		LDA	(Addr)			; get data byte
		#PLB				; restore DBR
		jsr	SendIDE
		inc	Addr
		bne	SaveF86
		inc	Addr+1
		bne	SaveF86
		inc	Addr+2
SaveF86		sec
		lda	Ptr
		sbc	#$01
		sta	Ptr
		lda	Ptr+1
		sbc	#$00
		sta	Ptr+1
		lda	Ptr+2
		sbc	#$00
		sta	Ptr+2
		bcs	SaveF85
SaveF9		lda	#$CE			; close file
		jsr	SendIDECMD
		lda	#$1F			; handle
		jsr	SendIDE
SaveF95		jsr	IDEStat			; get status
		cmp	#$00
		bpl	SaveF95			; wait for ready
		and	#$1F
		beq	SaveF99			; get next byte
		jsr	PrintErr
SaveF99		rts
OWPrompt	.byte	$0d, $0A, "File Exists.  Overwrite? (Y/N) ", $00

;
; Read memory locations
;
ReadM		ldy	#$02			; init Y
ReadM0		jsr	GetHexDig		; load Ptr with starting address
		bcc	ReadM05
		jmp	ReadM9			; error, so exit
ReadM05		lda	Buffer,y		; get terminator
		beq	ReadMp1b		; print 1 byte and exit
		cmp	#","
		bne	ReadM1
		jsr	Readmp1b		; read 1 byte then repeat
		iny
		bra	Readm0			; do next
ReadM1		cmp	#" "
		bne	ReadM2
		jsr	Readmp1b		; read 1 byte then repeat
		iny
		bra	Readm0			; do next
ReadM2		cmp	#":"
		beq	ReadMErr
		lda	Ptr			; - range selected
		sta	Addr
		lda	Ptr+1
		sta	Addr+1
		lda	Ptr+2
		sta	Addr+2			; move ptr to start address 
		iny
		jsr	GetHexDig		; get ending address
		bcs	ReadM9			; error, so exit
ReadM3		jsr	Print_cr
		jsr	DispAddr
		lda	#":"
		jsr	output
ReadM4		lda	#" "
		jsr	output
		#PHB				; save DBR
		lda	Addr+2			; get data DB
		PHA				; prep
		#PLB				; set it
		LDA	(Addr)			; get data byte
		#PLB				; restore DBR
		jsr	Print1Byte
Readm5		inc	Addr
		bne	Readm6
		inc	Addr+1
		bne	Readm6
		inc	Addr+2
		lda	Addr+2
		cmp	#$07	
		bcs	ReadM6
		lda	#$1C
		jsr	PrintErr
		rts
ReadMP1B	jsr	DispPtr
		lda	#":"
		jsr	output
		lda	#" "
		jsr	output
		#PHB				; save DBR
		lda	Ptr+2			; get data DB
		PHA				; prep
		#PLB				; set it
		LDA	(Ptr)			; get data byte
		#PLB				; restore DBR
		jsr	Print1Byte
		jsr	Print_cr
ReadM9		rts
ReadM6		sec
		Lda	Ptr
		sbc	addr
		lda	Ptr+1
		sbc	Addr+1
		lda	Ptr+2
		sbc	Addr+2
		bmi	ReadM7
		jsr	Scan_Input		; see if brk requested
		bcs	ReadM7			; yes, exit
		lda	Addr
		and	#$07
		bne	ReadM4			; print next byte in curr line	
		bra	ReadM3
ReadMErr	lda	#$1F
		jsr	PrintErr
		rts
ReadM7		jsr	print_cr	
		rts
;
;
;
ReadSec		ldy	#$02			; init Y
		jsr	GetHexDig		; load Ptr with Sector #
		bcs	ReadSec9		; list next 20 lines
		lda	#$D1			; read sector
		jsr	SendIDECmd
		lda	#$00			; buffer0 
		jsr	SendIDE
		lda	Ptr
		jsr	SendIDE
		lda	Ptr+1
		jsr	SendIDE
		lda	Ptr+2
		jsr	SendIDE
		lda	Ptr+3
		jsr	SendIDE
		lda	#$00
		sta	Ptr
		lda	#$10
		sta	Ptr+1
ReadSec1	jsr	RxIde		
		sta	(Ptr)
		inc 	Ptr
		bne	ReadSec1
		inc	Ptr+1
ReadSec2	lda	Ptr+1
		cmp	#$12
		bne	ReadSec1
ReadSec9	rts

DispPtr		lda	ptr+2
		jsr	printdig
		lda	ptr+1
		jsr	print1byte
		lda	ptr
		jsr	print1byte
		rts		

DispAddr	lda	Addr+2
		jsr	printdig
		lda	Addr+1
		jsr	print1byte
		lda	Addr
		jsr	print1byte
		rts		

;
; Write Memory
;
WriteM		ldy	#$02			; init Y
WriteM0		jsr	GetHexDig		; load Ptr with starting address
		bcc	WriteM1
		jmp	WriteMErr		; error, so exit
WriteM1		lda	Buffer,y		; get terminator
		beq	WriteMErr		; no data - error
		cmp	#":"
		beq	WriteM2
		jmp	WriteMErr
WriteM2		lda	ptr
		sta	Addr
		lda	ptr+1
		sta	Addr+1
		lda	ptr+2
		sta	Addr+2
WriteM3		iny
		jsr	GetHexDig
		bcs	WriteMErr
		lda	#$00
		cmp	Ptr+1
		bne	WriteMErr
		cmp	Ptr+2
		bne	WriteMErr
		#PHB				; save DBR
		lda	Addr+2			; get data DB
		PHA				; prep
		#PLB				; set it
		LDA	Ptr			; get data byte
		sta	(Addr)			; save it
		#PLB				; restore DBR
		lda	Buffer,y
		beq	WriteM9			; done
		cmp	#" "
		bne	WriteMErr
		inc	Addr
		bne	WriteM3
		inc	Addr+1
		bne	WriteM3
		inc	Addr+2
		bne	WriteM3
		lda	#$1C
		.byte	$2C			; bit command to mask next line
WriteMErr	lda	#$1F
		jsr	PrintErr
WriteM9		rts

;
;Execute code at mem location
;

Exec		ldy	#$02
		jsr	GetHexDig
		bcs	ExecErr
Exec0		#JSL	exec1, SysBank		; JSR to long jmp - user code uses RTL to return
		lda	#SysBank
		pha
		#PLB				; restore DBR to sysbank
		RTS
exec1		lda	Ptr+2			; bank reg
		pha
		#PLB				; set DBR to match bank of program code
		#JML	Ptr			; Long Jump to program
		rts		
ExecErr		lda	#$1F
		jsr	PrintErr
		rts

;
; xmodem transfer from PC to SBC-3
;
Download	ldy	#$02
		jsr	GetHexDig		; start address
		bcs	DownLoad2		; use file's address
		lda	Ptr
		sta	Addr
		lda	Ptr+1
		sta	Addr+1
		lda	Ptr+2
		sta	Addr+2
		cmp	#$08
	  	bcc	Download5
Download1	lda	#$1C
		jsr	PrintErr
		rts
Download2	jsr	XmodemRcv		; start rcv using flie's addr
		cmp	#$00
		beq	Download4
		jsr	PrintErr		; print the error
Download4	rts
Download5	jsr	XmodemRXA		; strart rcv using user addr
		cmp	#$00
		beq	Download6
		jsr	PrintErr		; print error
Download6	rts			
		rts

;
; Send file from SBC-3 to PC
;
upload		ldy	#$02
		jsr	GetHexDig		; start address
		bcs	Upload9			; error - no start address
		lda	Ptr
		sta	Addr
		lda	Ptr+1
		sta	Addr+1
		lda	Ptr+2
		sta	Addr+2
		cmp	#$08
	  	bcc	Upload2
Upload1		lda	#$1C			; address exceeds RAM
		jsr	PrintErr
		rts
Upload2		lda	buffer,y
		cmp	#","
		bne	Upload9			; syntax error
		iny
		jsr	GetHexDig		; stop address
		bcs	Upload9			; error - no stop address
		lda	ptr+2
		cmp	#$08
		bcs	upload1
		jsr	XmodemSend
		cmp	#$00
		bne	Upload95
		rts
Upload9		lda	#$1F			; syntax error
Upload95	jsr	PrintErr
		rts

;
; Format Volume
;
FormatVol	lda	#$F2			
		jsr	SendIDECmd		; Format Command
FormatV1	jsr	IdeStat
		cmp	#$00
		bpl	FormatV1		; wait for completion
		and	#$1F
		beq	FormatV2
		jmp	PrintErr
FormatV2	rts

;
;Set Clock
;
SetClock	ldy	#$02
		lda	buffer, y
		cmp	#$20
		bne	SetClock2
		iny	
		lda	#$E1			; set clock
		jsr	SendIDECmd
SetClock1	lda	buffer, y
		iny
		jsr	SendIDE
		cpy	#$11
		bne	SetClock1
		jsr	ShoDT
SetClock2	rts

;
; Mount/Initialize Volume
;

MountVol	ldy	#$01
MountVol1	iny
		lda	buffer, y
		cmp	#$20
		beq	MountVol1
		and	#$01			; mask out Drive 0 or 1
MountVol0	pha				; cold start DOS entry point			
		lda	#$F4			; init file system
		jsr	SendIDECmd
		pla
		jsr	SendIde			; send drive spec
MountVol2	jsr	IDEStat			; get status
		CMP	#$00
		bpl	MountVol2		; Bit 7=0 - wait for command to be done		
		and	#$1F			; mask error code
		beq	MountVol3		; done
		jsr	PrintErr		; print error
MountVol3	pla
		pla				; pop return address
		jmp	Init_DOS		; force jmp to DOS init		

;
; Parse path on the command line
;
ParsePath	ldx	#$ff
ParPath1	inx
		lda	CurrPath, x
		bne	ParPath1
		ldy	#$01			; x points to null at end of path
ParPath2	iny
		lda	buffer, y
		beq	ParPath25		; nothing
		cmp	#$20
		beq	ParPath2		; ignore space
		cmp	#","			; , = end
;		beq	ParPath25		; nothing
;		cmp	#"."
		bne	ParPath3		; 

ParPath25	ldy	#$00			; mark error - nothing to parse
		rts				; empty string
ParPath3	cmp	#"/"			; is 1st char / (root)?
		bne	ParPath5		; no, start from current dir
		ldx	#$01
		sta	CurrPath
		stz	CurrPath+1		; adjust CurrDir to root
ParPath4	iny
ParPath5	lda	buffer, y		; process 1st char of next dir
		beq	ParPath9		; end	
		cmp	#$20			; space = end
		beq	ParPath9
		cmp	#","			; , = end
		beq	ParPath9
		cmp	#"."
		bne	ParPath7
		lda	buffer+1,y		; look ahead
		beq	ParPath9		; end, ignor . and finish
		cmp	#"."			; ..
		bne	ParPath6		;
		lda	buffer+2,y		; look 2 ahead
		beq	ParPath55
		cmp	#"/"
		bne	ParErr			; invalid, so ignore rest and finish
ParPath55	dex
ParPath56	dex
		lda	CurrPath,x		; do ..	
		cmp	#"/"
		bne	ParPath56
		inx
		iny
		bra	ParPath4		; do next		
parPath6	cmp	#"/"			; ./
		bne	ParErr			; .X INVALID, so ignore rest and finish
		iny				; ./ is null, skip over
		bra	ParPath4		; do next dir
ParPath7	cmp	#"/"			; end of dir
		bne	ParPath8		; reg char
		cmp	CurrPath-1,x
		beq	ParPath4		; if already there, skip adding
		sta	CurrPath,x
		inx
		bra	ParPath4		; do next dir
ParPath8	sta	CurrPath,x
		inx
		iny
		lda	buffer,y
		beq	ParPath9
		cmp	#$20			; space = end
		beq	ParPath9
		cmp	#","			; , = end
		beq	ParPath9
		bra	ParPath7		; store and do next
ParErr		lda	#$1F			; general syntax error
		phx
		jsr	PrintErr
		plx
		ldy	#$00			; mark errors occurred
ParPath9	lda	CurrPath-1,x		;  
		cmp	#"/"			;
		beq	ParPath95
		lda	#"/"
		sta	CurrPath,x
		inx
ParPath95	stz	CurrPath, x		; null term path
		rts

;
; Print Current Directory
;
PrintCurr	ldx	#$00
PrintCurr1	lda	CurrPath, x
		beq	PrintCurr2
		jsr	output
		inx
		bra	Printcurr1
PrintCurr2	jsr	Print_CR
		rts

;
; print Results from IDE
;
PrintResult	jsr	RxIDE
		cmp	#$00
		beq	PrintResult1
		jsr	output
		bra	PrintResult
printResult1	rts

;
; Print Error message
;
PrintErr	pha				; save error code
		ldx	#$00
PrintErr1	lda	errtxt2,x
		beq	PrintErr2
		jsr	output
		inx
		bra	PrintErr1
PrintErr2	pla
		jsr	Print1byte		; print error code in HEX
		jsr	Print_CR
		rts
errtxt2		.byte   "Error: ", $00

;
; Get ascii chars and convert to hex byte(s)
;

GetHexDig	lda	#$00
		tax				; flag
		sta	Ptr
		sta	Ptr+1
		sta	Ptr+2			; reset Pointer space
GetHexDig0	lda	Buffer,y
		cmp	#$20
		bne	getHexDig1		; strip leading spaces
		iny
		bra	GetHexDig0
GetHexDig1	lda	Buffer,y		; load chr
		beq	GetHexDig9
		cmp	#"-"
		beq	GetHexDig9
		cmp	#","
		beq	GetHexDig9
		cmp	#" "
		beq	GetHexDig9
		cmp	#":"
		beq	GetHexDig9
		cmp	#$30
		bcc	GetHexDigErr		; below $30 - not digit, error
		cmp	#$3A
		bcc	GetHexDig2		; below $3A - digit
		cmp	#$41
		bcc	GetHexDigErr		; below A, not digit, error
		cmp	#$47
		bcs	GetHexDigErr		; above F is not digit, error
GetHexDig2	cmp	#$3A
		bcc	GetHexDig3
		sbc	#$07
GetHexDig3	and	#$0F
		inx
		sta	temp			; save nibble
		asl	Ptr			; shift rest of word left 4 times 
		rol	Ptr+1			; to make room for
		rol	Ptr+2			; the new nibble
		asl	Ptr
		rol	Ptr+1
		rol	Ptr+2
		asl	Ptr
		rol	Ptr+1
		rol	Ptr+2
		asl	Ptr
		rol	Ptr+1
		rol	Ptr+2
		lda	Ptr
		ora	Temp			; insert new nibble
		sta	Ptr			; save
		iny		
		bra	getHexDig1		; get next digit
GetHexDig9	cpx	#$00
		beq	GetHexDig99
		clc				; mark no errors
		rts
GetHexDigErr	lda	#$1F
		jsr	PrintErr
GetHexDig99	sec				; mark error
		rts


;
;Help   *** use REP/SEP to reduce
;
Help		LDX	#$00
Help1		lda	HelpTxt, x
		beq	Help2
		jsr	output
		inx
		bra	Help1
Help2		LDX	#$00
Help3		lda	HelpTxt2, x
		beq	Help4
		jsr	output
		inx
		bra	Help3
Help4		rts

HelpTxt		.byte 	"DOS Command List", $0d, $0a
		.byte 	"LD - List Dir", $0d, $0a
		.byte 	"MD - Make Dir", $0d, $0a
		.byte 	"CD - Cha Dir", $0d, $0a
		.byte 	"LV - List Vol", $0d, $0a
		.byte 	"RN - Rename", $0d, $0a
		.byte 	"DE - Delete File/Dir", $0d, $0a
		.byte 	"PR - Prot", $0d, $0a
		.byte 	"UP - unprot", $0d, $0a
		.byte 	"DT - Date/time", $0d, $0a
		.byte 	"FS - Free Space", $0d, $0a, $00

HelpTxt2	.byte 	"LF - load File", $0d, $0a
		.byte 	"SF - Save File", $0d, $0a
		.byte 	"RM - Read Mem", $0d, $0a
		.byte 	"WM - Write Mem", $0d, $0a
		.BYTE 	"DI - Disassemble Mem", $0d, $0a
		.BYTE 	"DX - Dis w/16 bit Imm", $0d, $0a
		.BYTE 	"DL - Xmodem Download", $0d, $0a
		.BYTE 	"UL - Xmodem Upload", $0d, $0a
		.byte 	"EX - Execute", $0d, $0a
		.byte 	"FV - Format Vol", $0d, $0a
		.byte 	"SC - Set Clock", $0d, $0a
		.byte 	"MV - Mount Volume", $0d, $0a, $00
;
; Send char to IDE
;
SendIDECmd	pha				;
SendIDECmd1	jsr	IdeStat			;
		and	#$A0			; bit 7 and 5
		cmp	#$80			; bit 7
		bne	SendIDeCmd1		;
		bra	SendIde1		;

SendIDE		pha
SendIDE0	jsr	IdeStat			; get status
		and	#$20			; IDE ready for command and ready to receive data
		bne	SendIDE0		; not ready, check status again
SendIDE1	lda	#255-SPIIDE		;
		#AND_AL	SPISSR, 8		; select device
		#STA_AL	SPISSR, 8
		lda	#$01			; send data
		#STA_AL spidr, 8		;
SendIDE4	#LDA_AL	SPISR, 8            
               	bpl	SendIDE4		; wait for tx to end
		nop
		nop
		nop
		nop				; wait for SPI to be ready
		#LDA_AL	spidr, 8		; clr flags	
		pla				; send data byte
		#STA_AL	spidr, 8		; start shift
SendIDE5	#LDA_AL	SPISR,8            
               	bpl	SendIDE5		; wait for tx to end
		#LDA_AL	spidr, 8		; clr SPI flag
		lda	#SPIIDE			;
		#ORA_AL	SPISSR, 8		; Deselect device
		#STA_AL	SPISSR, 8
		rts
;
; Get Char from IDE
;
RxIde		jsr	IdeStat
		and	#$40			; IDE data ready to send
		beq	RxIDE			; not ready, check status again
		lda	#255-SPIIDE		;
		#AND_AL	SPISSR, 8		; select device
		#STA_AL	SPISSR, 8
		lda	#$03			; send data
		#STA_AL spidr, 8		;
RxIDE4		#LDA_AL SPISR, 8            
               	bpl	RxIDE4			; wait for tx to end
		nop
		nop
		nop
		nop				; wait for SPI to be ready
		#LDA_AL SPIDR, 8		; clr flags	
		lda	#$00			; send null
		#STA_AL spidr, 8		; start shift
RXIDE5		#LDA_AL SPISR, 8            
               	bpl	RxIDE5			; wait for tx to end
		#LDA_AL SPIDR, 8		; get received data
		pha				; save it
		lda	#SPIIDE			;
		#ORA_AL	SPISSR, 8		; Deselect device
		#STA_AL	SPISSR, 8
		pla				; restore data
		rts

;
; Read IDE status
;
IdeStat		lda	#255-SPIIDE		;
		#AND_AL	SPISSR, 8		; select device
		#STA_AL	SPISSR, 8
		#LDA_AL SPIDR, 8		; clr flags
		lda	#$02			; get status
		#STA_AL spidr, 8		;
IdeStat1	#LDA_AL SPISR, 8            
               	bpl	IdeStat1		; wait for tx to end
		nop
		nop
		nop
		nop				; wait for SPI to be ready
		#LDA_AL SPIDR, 8		; clr flags	
		lda	#$00			; null
		#STA_AL spidr, 8		; start shift
IdeStat2	#LDA_AL SPISR, 8            
               	bpl	IdeStat2		; wait for tx to end
		#LDA_AL SPIDR, 8		; get STATUS
		pha
		lda	#SPIIDE			;
		#ORA_AL	SPISSR, 8		; Deselect device
		#STA_AL	SPISSR, 8
		pla				; restore data
		rts

;
; history
;
; V1.4  1-07-2010 - added LX and . for load and execute
; V1.3  12-7-2009 - moved DiskOS to bank 7 
; V1.2  12-7-2009 - moved I/O to bank 8 
; V1.1  8-25-2009 - bug fix with directory tracking
; V1.0  4-14-2009 - first release
;
;end of file
;
