;
;---------------------------------------------------------------------
;  SBC Firmware V6.2, 11-06-08, by Daryl Rictor
;  For 65816 running Native mode
;
;*********************************************************************		 
;
; Local Non-Zero Page Variables
;

RegData		.byte	"PBR=  PC=.  DBR=  B=  A=~X=.  Y=.  D=.  S=.~E=  P=  (NVMXDIZC):"
;
PrintReg	ldx	#$ff			;
		ldy	#$ff			;
Printreg0	Jsr	Print_CR		; Lead with a CR
Printreg1	iny				;
		lda	Regdata,y		;
		cmp	#"."			;
		beq	Printreg2		; if ., print a second byte
		cmp	#"~"			; 
		beq	Printreg0		; if ~, print CR and do new line
		jsr	output			; print prompt data
		cmp	#":"			; 
		beq	Printreg3		; if :, expand P into bits and exit
		cmp	#"="			; 
		bne	Printreg1		;
Printreg2	inx				;
		lda	PBR,x			;  
		jsr	Print1Byte		;
		bra	Printreg1		;
Printreg3	lda	PBR,x			; get Preg
		ldx	#$08			; 
Printreg4	rol				;
		tay				;
		lda	#$31			;
		bcs	Printreg5		;
		dec				;
Printreg5	jsr	Output			;
		tya				;
		dex				;
		bne	Printreg4		;
						; fall into the print CR routine

Print_CR	PHA				; Save Acc
		LDA	#$0D			; "cr"
		JSR	OUTPUT			; send it
;		LDA	#$0A			; "lf"
;		JSR	OUTPUT			; send it
		PLA				; Restore Acc
		RTS				; 
;
Print2Byte	JSR	Print1Byte		;  prints AAXX hex digits
		TXA				;
Print1Byte	PHA				;  prints AA hex digits
		LSR				;  MOVE UPPER NIBBLE TO LOWER
		LSR				;
		LSR				;
		LSR				;
		JSR	PrintDig		;
		PLA				;
PrintDig	AND	#$0F			;
		CMP	#$0A			;
		BCC	PrintDig1		;
		ADC	#$66			;
PrintDig1	EOR	#$30			;
		jmp	output			;
PrintXSP1	JSR	Print1SP		;
		dex				;
PrintXSP	cpx	#$00			;
		bne	PrintXSP1		;
		rts				;
Print2SP	jsr	Print1SP		; print 2 SPACES
Print1SP	LDA	#$20			; print 1 SPACE
		JMP	OUTPUT			;
Input		jsr	Print_CR		; New Line
		lda	#$10			; DOS Prompt filled triangle '>'
		jsr	Output			; Print Prompt
		ldy	#$ff			; pointer
InputWait	jsr	Input_Chr		; get a character
		cmp	#$20			; is ctrl char?
		BCS	InputSave		; no, echo chr 
		cmp	#$0d			; cr
		Beq	InputDone		; done
		cmp	#$1B			; esc
		beq	Input			; cancel and new line
		cmp	#$08			; bs
		beq	backspace		;
		cmp	#$09			; TAB key
		beq	tabkey				;
		cmp	#$02			; Ctrl-B
		bne	InputWait		; Ignore other codes
		brk				; Force a keyboard Break cmd
backspace	cpy	#$ff			;
		beq	InputWait		; nothing to do
		dey				; remove last char
		Lda	#$08			; backup one space
		jsr	Output			;
;		Lda	#$20			; Print space (destructive BS)
;		jsr	Output			;
;		Lda	#$08			; backup one space
;		jsr	Output			;
		BRA	InputWait		; ready for next key
tabkey		lda	#$20			; convert tab to space
		iny				; move cursor
		bmi	InputTooLong		; line too long?
		sta	Buffer,y		; no, save space in buffer
		jsr	output			; print the space too
		tya				; test to see if tab is on multiple of 8
		and	#$07			; mask remainder of cursor/8
		bne	tabkey			; not done, add another space
		bra	InputWait		; done. 
InputSave	CMP	#$61			;	ucase
		BCC	InputSave1		;
		cmp	#$7B
		bcs	InputSave1
		SBC	#$1F			;
InputSave1	INY				;
		BMI	InputTooLong		; get next char (up to 127)
		STA	Buffer,y		;
		JSR	Output			; OutputCharacter
		BRA	InputWait		;
InputDone	INY				;
InputTooLong	LDA	#$0d			; force CR at end of 128 characters 
		sta	Buffer,y		;
		JSR	Output			;
;		lda	#$0a			; lf Char	
;		JSR	Output			;
		RTS				;
;
bell		LDA	#$07			; Ctrl G Bell
		jmp	Output			; 
;
BRKroutine	pha
		lda	#SYSBANK
		pha
		#PLB
		pla
		sta	ACC			; save A	 Monitor"s break handler
		#XBA				;
		STA	B			; save B 
		#REP	$30			; 16 bit mode
		txa				; get 16 bit X
		#XBA				; switch low & hi bytes for printing
		sta	Xreg			; save X 
		tya				; get 16 bit Y
		#XBA				; switch low & hi bytes for printing
		sta	Yreg			; save Y
		#SEP	$30			; 8 bit mode
		pla				;  
		sta	Preg			; save P
		pla				; PCL
		plx				; PCH
		sec				;
		sbc	#$02			;
		sta	PCL			; backup to BRK cmd
		bcs	Brk2			;
		dex				;
Brk2		stx	PCH			; save PC
		pla				; 
		sta	PBR			; save PBR
		#PHB				; 
		pla				;
		sta	DBR			; save DBR
		lda	#$00			; E=0 if reset 
		#XCE				; get E bit
		bcc	Brk3			; 
		inc				; E=1 if set
Brk3		#XCE				; restore E back
		sta	E			; save E
		#TDC				;
		sta	D+1			;
		#XBA				;
		sta	D			; save D
		#TSC				;
		sta	SPTR+1			;
		#XBA			  	;
		sta	SPTR			; save S
		jsr	Bell			; Beep speaker
		jsr	PrintReg		; dump register contents 
		lda	#$01			;
		#XBA				;
		lda	#$FF			; 
		#TCS				; clear stack
		cli				; enable interrupts again
		jmp	Main_Loop		; start DiskOS
;end of file