;
;*******************************************************************************
;
;  65SPI and ATMega8 Interface Routines for SBC-3  (c) Daryl Rictor Nov 6, 2008
;
;*******************************************************************************
;
;*********************** Register Definitions *******************************

; SPI command structure
; $00 - null (used to receive data)
; $01 - Get Status - see status register bit definitions
; $02 - Get RS-232 RX byte
; $03 - send RS-232 TX byte
; $04 - Get RS-232 parameters
; $05 - Set RS-232 parms : bit 0-2 = speed, bit 3 = 7/8 bit, bit 4 = parity y/n, bit 5 = parity odd/even, bit 6 = stop 1/2
; $06 - get KB byte 
; $07 - send kb cmd	0= ASCII mode, 1= scancode mode, 2= future > 2 = KB CMD byte
; $08 - init RS232 - Set to default settings
; $09 - init KB - Re-init kb and set to ASCII mode
;
; 65SPI Status Register bit definitions  
; 7= IRQ - 1 = operation complete, data waiting  0 = no attention needed
; 6= IER - 1 = Enable interrupt output, 0= disable interrupt output
; 5= BSY - 1 = shifting in progress, 0= shifting idle
; 4= FRX - 1 = Fast receive, reading SPIDR initiates a new shift; 0 = fast receive disabled
; 3= TMO - 1 = put MOSI pin in hi-z, 0 = MOSI output enabled 
; 2= ECE - 1 = External clock enabled (SS7 is an input), 0 = ext clock disabled SS7 is an output
; 1= CPOL - Clock Polarity } selects shift mode 0-3			 
; 0= CPHA - Clock phase	 }
;
; 65SPI Control Register bit definitions
; 7= not used
; 6= IER - 1 = Enable interrupt output, 0= disable interrupt output
; 5= not used
; 4= FRX - 1 = Fast receive, reading SPIDR initiates a new shift; 0 = fast receive disabled
; 3= TMO - 1 = put MOSI pin in hi-z, 0 = MOSI output enabled 
; 2= ECE - 1 = External clock enabled (SS7 is an input), 0 = ext clock disabled SS7 is an output
; 1= CPOL - Clock Polarity } selects shift mode 0-3			 
; 0= CPHA - Clock phase	 }
;
;ATM8 Status bit definitions
; 7= KB data waiting
; 6= RX data waiting
; 5= TX Ready
; 4= RX Frame error
; 3= RX overrun
; 2= RX parity
; 1= KB Missing
; 0= not used
;

;
; COnstants
;
SPIATM8 	= 	$40			; Slave address of ATMega8

;*********************** SPI Interface Routines *******************************
SPI_init	#LDA_AL	Via1pcr, 8		;
		and	#$0F			;
		#STA_AL	Via1pcr, 8		; set CB1 to neg flag
SPI_init1	#LDA_AL	Via1IFR, 8		;
		and	#$10			; wait for CB1 to fall
		beq	SPI_init1		; wait for ATM8 to signal ready
		lda	#$FF			;
		#STA_AL	SPISSR, 8		; init Slave Select register, all high
		#LDA_AL	Via1PRB, 8		; clear Via1 CB1 flag
		lda	#$00			;
		#STA_AL	SPICR, 8		; control reg - no IRQ, int clock, mode 0

		lda	#$01			; NTSC SCLK divisor (for 7.16 MHz CPU)

		#STA_AL	SPIDIV, 8		; sclk divisor (ATM8 max is 3.6MHz) NTSC=0, PAL=1
		#LDA_AL	SPIDR, 8		; clear data register flag
		rts				

GET_STATUS	lda	#255-SPIATM8		;
		#AND_AL	SPISSR, 8		; select ATM8 SS
		#STA_AL	SPISSR, 8
		#LDA_AL	Via1PRB, 8		; clear VIA1 flag		
		jsr	SPI_STATUS		; get status
		pha				; save status
		lda	#SPIATM8		; 
		#ORA_AL	SPISSR, 8		; deselect ATM8
		#STA_AL	SPISSR, 8
		pla				; restore status
		RTS				;

SPI_STATUS	lda	#$01			; GET ATM8 STATUS
		#STA_AL	SPIDR, 8		; send get Status cmd
SPI_STAT1	#LDA_AL	Via1IFR, 8		;
		and	#$10			; CB1 mask
		beq	SPI_STAT1		; wait for ATM8 to signal ready
		#LDA_AL	SPIDR, 8		; clear flag
		#LDA_AL	Via1PRB, 8		; clear Via1 CB1 flag
		lda	#$00			; null cmd
		#STA_AL	SPIDR, 8		; send data to kb
SPI_STAT2	#LDA_AL	SPISR, 8				
		bpl	SPI_STAT2		; wait for tx to end
		#LDA_AL	SPIDR, 8		; read SPI status
		rts

;----------------------------------------------------------------------------
SPIKB_init	lda	#255-SPIATM8
		#AND_AL	SPISSR, 8		; select ATM8 SS
		#STA_AL	SPISSR, 8
		#LDA_AL	Via1PRB, 8		; clear VIA1 flag
		jsr	SPI_Status		;
		pha				; save status
		lda	#SPIATM8		; 
		#ORA_AL	SPISSR, 8		; deselect ATM8
		#STA_AL	SPISSR, 8
		pla				; restore status
		and	#$02			; mask KB missing
		beq	SPIKB_init9		; no errors found
		brk				; stop - KB missing error
		.byte 	$EE			; KB missing
SPIKB_init9	rts				; done

SET_KB		pha				; save  A
SET_KB0		lda	#SPIATM8
		#AND_AL	SPISSR, 8		; select ATM8 SS
		#STA_AL	SPISSR, 8
		#LDA_AL	Via1PRB, 8		; clear VIA1 flag
		lda	#$07			;
		#STA_AL	SPIDR, 8		; send Send KB data cmd
SET_KB1		#LDA_AL	Via1IFR, 8		;
		and	#$10			; CB1 mask
		beq	SET_KB1			; wait for tx to end
		#LDA_AL	SPIDR, 8		; clear data ready flag		
		#LDA_AL	Via1PRB, 8		; clear Via1 CB1 flag
		pla				; 
		#STA_AL	SPIDR, 8		; send data to kb
SET_KB2		#LDA_AL	SPISR, 8		;
		bpl	SET_KB2			; wait for tx to end
		#LDA_AL	SPIDR, 8		; read SPI data reg to clear flags
		lda	#SPIATM8		; 
		#ORA_AL	SPISSR, 8		; deselect ATM8
		#STA_AL	SPISSR, 8
		rts

INPUT_CHR
SPI_KBIN	lda	#255-SPIATM8		;
		#AND_AL	SPISSR, 8		; select ATM8 SS
		#STA_AL	SPISSR, 8
		#LDA_AL	Via1PRB, 8		; clear Via1 CB1 flag
SPI_KBIN1	jsr	SPI_STATUS		; get status
		bpl	SPI_KBIN1		; keep testing for KB data
SPI_KBIN2	lda	#$06
		#STA_AL	SPIDR, 8		; send get KB data
SPI_KBIN3	#LDA_AL	Via1IFR, 8		;
		and	#$10
		beq	SPI_KBIN3		; wait for ATM8 to signal ready
		#LDA_AL	SPIDR, 8		; clear data ready flag		
		#LDA_AL	Via1PRB, 8		; clear Via1 CB1 flag
		lda	#$00			; null cmd
		#STA_AL	SPIDR, 8		; send null to get data
SPI_KBIN4	#LDA_AL	SPISR, 8		; 
		bpl	SPI_KBIN4		; wait for tx to end
		#LDA_AL	SPIDR, 8		; read KB data
		pha
		lda	#SPIATM8		; 
		#ORA_AL	SPISSR, 8		; deselect ATM8
		#STA_AL	SPISSR, 8
		pla
		sec
		rts
					
SCAN_INPUT
SPI_KBSCAN	lda	#255-SPIATM8		;
		#AND_AL	SPISSR, 8		; select ATM8 SS
		#STA_AL	SPISSR, 8
		#LDA_AL	Via1PRB, 8		; clear Via1 CB1 flag
		jsr	SPI_Status		; get status
		bmi	SPI_KBIN2		; if byte ready, get it and return w/carry set				
		lda	#SPIATM8		; 
		#ORA_AL	SPISSR, 8		; deselect ATM8
		#STA_AL	SPISSR, 8
		clc				; otherwise, return with carry clear
		rts					
;----------------------------------------------------------------------------

SPISER_init	lda	#255-SPIATM8
		#AND_AL	SPISSR, 8		; select ATM8 SS
		#STA_AL	SPISSR, 8
		#LDA_AL	Via1PRB, 8		; clear VIA1 flag
		jsr	SPI_Status		;
		pha				; save status
		lda	#SPIATM8		;*** ADD code to set comm mode?		 
		#ORA_AL	SPISSR, 8		; deselect ATM8
		#STA_AL	SPISSR, 8
		pla				; restore status
		and	#$7C			; mask SER flags
		cmp	#$20			; TX ready flag set
		beq	SPISER_init9		; no errors found
		brk				; stop
		.byte	$CC			; RS232 error
SPISER_init9	rts				;

SCAN_SERIN	lda	#255-SPIATM8		;
		#AND_AL	SPISSR, 8		; select ATM8 SS
		#STA_AL	SPISSR, 8
		#LDA_AL	Via1PRB, 8		; clear Via1 CB1 flag
		jsr	SPI_Status		; get status
		and	#$40			; RX data ready
		sec
		bne	SCAN_SERIN1		; if byte ready, return w/carry set				
		clc				; otherwise, return with carry clear
SCAN_SERIN1	lda	#SPIATM8		; 
		#ORA_AL	SPISSR, 8		; deselect ATM8
		#STA_AL	SPISSR, 8
		rts				  

SER_IN		lda	#255-SPIATM8		;
		#AND_AL	SPISSR, 8		; select ATM8 SS
		#STA_AL	SPISSR, 8
		#LDA_AL	Via1PRB, 8		; clear Via1 CB1 flag
SER_IN1		jsr	SPI_STATUS		; get status
		and	#$40			; RX data ready
		beq	SER_IN1			; keep testing for RS-232 data
SER_IN2		lda	#$02
		#STA_AL	SPIDR, 8		; send get Rs-232 RX data
SER_IN3		#LDA_AL	Via1IFR, 8
		and	#$10
		beq	SER_IN3			; wait for ATM8 to signal ready
		#LDA_AL	Via1PRB, 8		; clear Via1 CB1 flag
		#LDA_AL	SPIDR, 8		; clear data ready flag		
		lda	#$00			; null cmd
		#STA_AL	SPIDR, 8		; send null to get data
SER_IN4		#LDA_AL	SPISR, 8		; 
		bpl	SER_IN4			; wait for tx to end
		#LDA_AL	SPIDR, 8		; read KB data
		pha
		lda	#SPIATM8		; 
		#ORA_AL	SPISSR, 8		; deselect ATM8
		#STA_AL	SPISSR, 8
		pla
		sec
		rts

SER_OUT		pha				; called with data in A
		lda	#255-SPIATM8		;
		#AND_AL	SPISSR, 8		; select ATM8 SS
		#STA_AL	SPISSR, 8
		#LDA_AL	Via1PRB, 8		; clear VIA1 flag
SER_OUT0	jsr	SPI_Status		; get status
		and	#$20			;
		beq	SER_OUT0		; wait for tx to be empty
		lda	#$03			;
		#STA_AL	SPIDR, 8		; send RS-232 data
SER_OUT1	#LDA_AL	Via1IFR, 8		;
		and	#$10			; CB1 mask
		beq	SER_OUT1		; wait for tx to end
		#LDA_AL	Via1PRB, 8		; clear Via1 CB1 flag
		#LDA_AL	SPIDR, 8		; clear data ready flag		
		pla				; 
		#STA_AL	SPIDR, 8		; send data to SER port
SER_OUT2	#LDA_AL	SPISR, 8		;
		bpl	SER_OUT2		; wait for tx to end
		#LDA_AL	SPIDR, 8		; read SPI data reg to clear flags
		lda	#SPIATM8		; 
		#ORA_AL	SPISSR, 8		; deselect ATM8
		#STA_AL	SPISSR, 8
		rts

GET_SER		lda	#255-SPIATM8		; Get Serial port Parameters
		#AND_AL	SPISSR, 8		; select ATM8 SS
		#STA_AL	SPISSR, 8
		#LDA_AL	Via1PRB, 8		; clear Via1 CB1 flag
GET_SER2	lda	#$04
		#STA_AL	SPIDR, 8		; send get KB data
GET_SER3	#LDA_AL	Via1IFR, 8		;
		and	#$10			;
		beq	GET_SER3		; wait for ATM8 to signal ready
		#LDA_AL	SPIDR, 8		; clear data ready flag		
		#LDA_AL	Via1PRB, 8		; clear Via1 CB1 flag
		lda	#$00			; null cmd
		#STA_AL	SPIDR, 8		; send null to get data
GET_SER4	#LDA_AL	SPISR, 8		; 
		bpl	GET_SER4		; wait for tx to end
		#LDA_AL	SPIDR, 8		; read KB data
		pha				; save parm in stack
		lda	#SPIATM8		; 
		#ORA_AL	SPISSR, 8		; deselect ATM8
		#STA_AL	SPISSR, 8
		pla				; return parm from stack
		rts				; return with parm in A

SET_SER		pha				; called with config in A
		lda	#255-SPIATM8		;
		#AND_AL	SPISSR, 8		; select ATM8 SS
		#STA_AL	SPISSR, 8
		#LDA_AL	Via1PRB, 8		; clear VIA1 flag
		lda	#$05			;
		#STA_AL	SPIDR, 8		; send RS-232 config cmd
SET_SER1	#LDA_AL	Via1IFR, 8		;
		and	#$10			; CB1 mask
		beq	SET_SER1		; wait for tx to end
		#LDA_AL	Via1PRB, 8		; clear Via1 CB1 flag
		#LDA_AL	SPIDR, 8		; clear data ready flag		
		pla				; 
		#STA_AL	SPIDR, 8		; send data to SER port
SET_SER2	#LDA_AL	SPISR, 8		;
		bpl	SET_SER2		; wait for tx to end
		#LDA_AL	SPIDR, 8		; read SPI data reg to clear flags
		lda	#SPIATM8		; 
		#ORA_AL	SPISSR, 8		; deselect ATM8
		#STA_AL	SPISSR, 8
		rts
