; /////////////////////////////////////////////////////////////////////////////////
;//                                                                               //
;//  This program and its associated documentation are provided for your personal //
;//  use only and appear here exclusively by permission of the copyright holder.  //
;//  Please contact the copyright holder before re-distributing, re-publishing    //
;//  or disseminating this copyrighted work. This code is not GPL or in the       //
;//  public domain. Please respect the author"s copyright.                        //
;//                                                                               //
;//  No waranty, either expressed or implied, are given.  I assume no liability   //
;//  for its use in any project or device.                                        //
;//                                                                               //
;//  Your use of this program indicates your acceptance of all license terms.     //
;//  This particular version is freeware as long as the copyright messages are    //
;//  left intact.                                                                 //
;//                                                                               //
; /////////////////////////////////////////////////////////////////////////////////
;
;*******************************************************************************
;
;  SBC-3 Text Mode Generator				(c) Daryl Rictor Nov 6, 2008
;
;*******************************************************************************
;
; 320 x 200 color video text generator
;
; color text now supported
;
; 65816 in native mode
;
; v1.1  3-22-09 : Added color text, fixed tab, added output of <$20 and >$7F
;

CURSOR		=	"_"
curln		=	$F0			; 0-24
curpos		=	$F1			; 0-39
chrptr		=	$F2			; 2 bytes  pointer to character storage area $xFA00-$xFDE7
chrptr_H	=	$F3			; 2 bytes  pointer to character storage area $xFA00-$xFDE7
vidptr		=	$F4			; 2 bytes  pointer to display area $x0000-$xF9FF
vidptr_H	=	$F5			; 2 bytes  pointer to display area $x0000-$xF9FF
cnt1		=	$F6
cnt2		=	$F7
cnt3		=	$F8
tmp1		=	$F9			; 2 bytes
tmp1_H		=	$FA			; 2 bytes
tmp2		=	$FB			; 3 bytes
tmp2_H		=	$FC			; 3 bytes
tmp2_B		=	$FD			; 3 bytes
dispage		=	$FE			; display block pointer 0-7, 1 is default
lopix		=	$FDEE			; color of 0 bit pixels (actual address is in display page, $01 by default) 
hipix		=	$FDEF			; color of 1 bit pixels (actual address is in display page) 

;
;
;
;
;
dispinit	lda	#$01			; default display page is $1xxxx
		sta	dispage			; 
; this code puts the banner on the screen.  User's can replace with their own code
		#PHB				; save DBR from calling program
		pha				; push page 1 display page
		#REP	$30			; set 16 bit Acc and Index
		#LDAI16  $03E7			; 16bit load Acc immediate = text count = 999
		#LDXI16  Banner			; Banner in Memory
		#LDYI16  $FA00			; first location to move to (the next byte higher)
		#MVN	$01,$00			; 
		#SEP	$30			; restore 8 bit Acc and Index
		#PLB				; set DBR to display page
		lda	#$48
		sta	lopix			; set background bit to $48 (blue)
		lda	#$BF			; set foreground bit to $BF (yellow)
		sta	hipix
		jsr	drawscrn		;	  
		#PLB				; restore DBR
		stz	curpos			;
		lda	#23			;
		sta	curln			;
		rts				;
;
output				
putchr		PHX				;
		PHY				;
		PHA				;
		#PHB				; save DBR
		ldx	dispage			;
		phx				;
		#PLB				; set DBR to display page
		cmp	#$00
		bmi	putch7			; above 127, print it				
		cmp	#$20			;
		bpl	putch6			; printable chr
		cmp	#08			; Backspace
		bne	putch1			;
		jsr	del			;
		bra	putch9			;
putch1		cmp	#09			; Tab		
		bne	putch2			;
		jsr	TAB			;
		bra	putch9			;
putch2		cmp	#10			; Line Feed 
		bne	putch3			;
		jsr	CTLF			;
		bra	putch9			;
putch3		cmp	#12			; Formfeed (CLS)
		bne	putch4			;
		jsr	clearsc			;
		bra	putch9			;
putch4		cmp	#13			; Carriage return 
		bne	putch7			; else print special chars
		jsr	CTCR			;
		bra	putch9			;
putch6		cmp	#$7F			; del key 
		bne	putch7			;
		jsr	del			;
		bra	putch9			;
putch7		pha				;
		jsr	setchrptr		;
		pla				;
		sta	(chrptr)		;
		jsr	drawchr			; 
		inc	curpos			;
		lda	curpos			; 
		cmp	#40			;
		bne	putch8			;
		stz	curpos			;
		lda	curln			;
		cmp	#24			;
		beq	putch75			;
		inc	curln			;
		bra	putch8			;
putch75		jsr	scrollu			;
putch8		jsr	setchrptr		;
		lda	#CURSOR			;
		sta	(chrptr)		;
		jsr	drawchr			;
putch9		#PLB				; restore DBR
		PLA				; restore registers
		PLY				;
		PLX				;
		rts				;
;
CTLF		rts				; ***
		jsr	setchrptr		;
		lda	#$20			;
		sta	(chrptr)		;
		jsr	drawchr			;
		lda	curln			;
		cmp	#24			;
		bne	CTLF2			;
		jmp	TAB9			;
CTLF2		inc	curln			;
CTLF3		jsr	setchrptr		;
		lda	#CURSOR			;
		sta	(chrptr)		;
		jmp	drawchr			;
;
CTCR		jsr	setchrptr		;
		lda	#$20			;
		sta	(chrptr)		;
		jsr	drawchr			;
		lda	#39			;
		cmp	curpos			;
		beq	CTCR8			;
		inc	curpos			;
		bra	CTCR			;
CTCR8		stz	curpos			;
		lda	curln			;
		cmp	#24			;
		beq	TAB9			;
CTCR9		inc	curln			;
CTCR0		jsr	setchrptr		;
		lda	#CURSOR			;
		sta	(chrptr)		;
		jmp	drawchr			;
;
TAB		jsr	setchrptr		;
		lda	#$20			; space
		sta	(chrptr)		;
		jsr	drawchr			;
		inc	curpos			;
		lda	#40			;
		cmp	curpos			;
		beq	TAB8			;
		lda	#$07			;
		and	curpos			;
		bne	TAB			;
TAB5		jsr	setchrptr		;
		lda	#CURSOR			;
		sta	(chrptr)		;
		jmp	drawchr			;
TAB8		lda	#$00			;
		sta	curpos			;
		lda	curln			;
		cmp	#24			;
		beq	TAB9			;
		inc	curln			;
		bra	TAB5			;
TAB9		jsr	scrollu			;
		jsr	setchrptr		;
		lda	#CURSOR			;
		sta	(chrptr)		;
		jmp	drawchr			;
;
del		jsr	setchrptr		;
		lda	#$20			; space
		sta	(chrptr)		;
		jsr	Drawchr			; draw it 
		lda	curpos			; is curpos = 0?
		bne	del3			; no - move cursor left one
		lda	curln			; yes, is curline = 0?
		bne	del2			; no - 
		bra	showcurs		; yes - reset cursor to same pos (home) 
del2		dec	curln			; move cursor up one line
		lda	#40			; move cursor to far right
		sta	curpos			;
del3		dec	curpos			;
		jsr	setchrptr		;
del4		bra	showcurs		;
;
clearsc		clc				; used for clarity in list output - not needed
		#PHB				; get current disp page 
		PLA				; put in A
		PHA				; save to stack
		#STA_AL clearsc1+1, $00 	; self modifying code - insert page into block move command
		#STA_AL clearsc1+2, $00 	; self modifying code - insert page into block move command
		#STA_AL clearsc2+1, $00 	; self modifying code - insert page into block move command
		#STA_AL clearsc2+2, $00 	; self modifying code - insert page into block move command
		lda	#$20			; first text chr = space
		sta	$FA00			;
		lda	lopix			; first pixel byte = background pixel color
		sta	$0000			;
		#REP	$30			; set 16 bit Acc and Index
		#LDAI16 $03E7			; 16bit load Acc immediate = text count = 999
		#LDXI16 $FA00			; first byte to copy (our space loaded above)
		#LDYI16 $FA01			; first location to move to (the next byte higher)
clearsc1	#MVN  $01,$01			; self replacing code used here - copy spaces
		#LDAI16 $F9FF			; 16bit load Acc immediate = pixel count = 63999
		#LDXI16 $0000			; first pixel
		#LDYI16 $0001			; second pixel
clearsc2	#MVN  $01,$01			; self replacing code used here - copy black pixels
		#SEP	$30			; restore 8 bit Acc and Index
		#PLB				; restore disp page
		stz	curln			; set cursor to home pos 0,0
		stz	curpos			;
		#LDA_AL chrbase, $00		; set chrptr to Home $FA00
		sta	chrptr			; 
		#LDA_AL chrbase+1, $00  	;
		sta	chrptr+1		;
showcurs	lda	#CURSOR			; load cursor chr
		sta	(chrptr)		; save to text data
		jmp	drawchr			; draw it to pixel data and return
;
drawchr		ldx	curln			;
		#LDA_ALX Linebase, $00  	;
		sta	vidptr+1		;
		lda	curpos			;
		asl				;
		tax				;
		#LDA_ALX colcnt, $00		;
		sta	vidptr			;
		inx				;
		clc				;
		#LDA_ALX colcnt, $00		;
		adc	vidptr+1		; 
		sta	vidptr+1		; vidptr set
		ldx	#$08			; 8 rows for each char
		stz	cnt3			; row counter
drawch3		lda	(chrptr)		; get chr to print
		tay				; save in Y
		stz	tmp2			;
		clc				;
		lda	cnt3			;
		adc	#>PRIFONT		;
		sta	tmp2+1			;
		stz	tmp2+2			;
		#LDA_DIIL tmp2			; get row font byte - LDA Direct Indirect Indexed long - LDA [d],y
		ldy	#$00			;
drawch4		rol				;
		pha				;
		lda	hipix			;
		bcs	drawch5			;	
		lda	lopix			;
drawch5		sta	(vidptr),y		;
		pla				;
		iny				;
		cpy	#$08			;
		bne	drawch4			;
		clc				;
		lda	vidptr			;
		adc	#$40			;
		sta	vidptr			;
		lda	vidptr+1		;
		adc	#$01			;
		sta	vidptr+1		;
		inc	cnt3			;
		dex				; 
		bne	drawch3			;
		rts				;

scrollu		#PHB				; get disp page
		PLA				; into Acc
		PHA				; save for end
		#STA_AL scrollu1+1, $00 	; self modifying code - insert page into block move command
		#STA_AL scrollu1+2, $00 	; self modifying code - insert page into block move command
		#STA_AL scrollu2+1, $00 	; self modifying code - insert page into block move command
		#STA_AL scrollu2+2, $00 	; self modifying code - insert page into block move command
		#STA_AL scrollu3+1, $00 	; self modifying code - insert page into block move command
		#STA_AL scrollu3+2, $00 	; self modifying code - insert page into block move command
		#STA_AL scrollu4+1, $00 	; self modifying code - insert page into block move command
		#STA_AL scrollu4+2, $00 	; self modifying code - insert page into block move command
		#REP	$30			; set 16 bit Acc & Index
		#LDAI16  $03BF			;
		#LDXI16  $FA28			;
		#LDYI16  $FA00			;
scrollu1	#MVN  $01,$01			; self replacing code used here - move line 1 to line 0, then 2 to 1, etc thru line 25 to 24
		#LDAI16  $EFFF			;
		#LDXI16  $0A00			;
		#LDYI16  $0000			;
scrollu2	#MVN  $01,$01			; self replacing code used here - move pixels to match above
		#SEP	$30			; back to 8 bit mode 
		#PLB				; restore DBR 
		lda	#$20			; put spaces on line 25
		sta	$FDC0			; text line 25
		lda	lopix			; black pixels
		sta	$F000			; line 25 equiv
		#PHB				; save DBR 
		#REP	$30			; 16 bit mode again 
		#LDAI16  $0028			;
		#LDXI16  $FDC0			;
		#LDYI16  $FDC1			;
scrollu3	#MVN  $01,$01			; self replacing code used here - fill in spaces on line 25
		#LDAI16  $09FF			;
		#LDXI16  $F000			;
		#LDYI16  $F001			;
scrollu4	#MVN  $01,$01			; self replacing code used here - fill in black pixels
		#SEP	$30			; 8 bit Acc & index
		#PLB				; restore text page to DBR
		rts				; return

setchrptr	lda	curln			;  get cursor line
		asl				;  mult x 2
		tax				;  move to index
		clc				; 
		#LDA_ALX chrbase, $00		; get base address from table - 16bits
		adc	curpos			;  add cursor position	 
		sta	chrptr			; 
		inx				;  
		#LDA_ALX chrbase, $00		; 
		adc	#$00			; 
		sta	chrptr+1		;  done
		rts

drawscrn	lda	#$00			;
		sta	vidptr			; 
		sta	vidptr+1		; set pointer to first pixel $0000
		sta	tmp1			;
		lda	#$FA			;
		sta	tmp1+1			; set text pointer to first character $FA00
		lda	#25			;
		sta	cnt3			; set text row counter to 25
drawsc1		stz	cnt1			; rows per char font counter	 
drawsc2		stz	cnt2			; char per row counter
drawsc3		ldy	cnt2			; char column offset 
		lda	(tmp1),y		; get chr to print
		tay				; save chr code in Y for font lookup
		stz	tmp2			; set tmp to font table based on which
		clc				;  
		lda	cnt1			; row we are drawing
		adc	#>PRIFONT		;
		sta	tmp2+1			;
		stz	tmp2+2			; table is located on block $00 
		#LDA_DIIL tmp2			; get row font byte  LDA [d],y
		ldx	#$08			; 8 pixels
drawsc4		rol				; move left pixel in carry flag
		pha				; save remaining pixels
		lda	hipix			; WHITE pixel
		bcs	drawsc5			; if pixel is 1, save white pixel		
		lda	lopix			; else, set pixel to $00 (BLACK)
drawsc5		sta	(vidptr)		; save pixel color to pixel data
		inc	vidptr			;
		bne	drawsc6			;
		inc	vidptr+1		; move to next pixel
drawsc6		pla				; get saved remaining pixels
		dex				; dec pixel counter
		bne	drawsc4			; draw remaining pixels
		inc	cnt2			; point to next text char on line
		lda	cnt2			; # chrs per row
		cmp	#40			; chr per row
		bne	drawsc3			; more chrs to draw
		inc	cnt1			; next font row counter 
		lda	cnt1			; 
		cmp	#8			; check if 8 rows done 
		bne	drawsc2			; do rest of rows
		clc				; set chr text pointer to next line
		lda	tmp1			;
		adc	#40			; add 40 to old line
		sta	tmp1			;
		lda	#$00			;
		adc	tmp1+1			; 
		sta	tmp1+1			;
		dec	cnt3			; chr row counter
		bne	drawsc1			; if not done, do more
		rts				; done

chrbase		.word	$FA00			; base address for each text character
		.word	$FA28			; text area is above pixel data
		.word	$FA50			; from $FA00-$FDE7
		.word	$FA78
		.word	$FAA0
		.word	$FAC8
		.word	$FAF0
		.word	$FB18
		.word	$FB40
		.word	$FB68
		.word	$FB90
		.word	$FBB8
		.word	$FBE0
		.word	$FC08
		.word	$FC30
		.word	$FC58
		.word	$FC80
		.word	$FCA8
		.word	$FCD0
		.word	$FCF8
		.word	$FD20
		.word	$FD48
		.word	$FD70
		.word	$FD98
		.word	$FDC0

Linebase	.byte	$00			; upper byte of base address for video page
		.byte	$0A
		.byte	$14
		.byte	$1E
		.byte	$28
		.byte	$32
		.byte	$3C
		.byte	$46
		.byte	$50
		.byte	$5A
		.byte	$64
		.byte	$6E
		.byte	$78
		.byte	$82
		.byte	$8C
		.byte	$96
		.byte	$A0
		.byte	$AA
		.byte	$B4
		.byte	$BE
		.byte	$C8
		.byte	$D2
		.byte	$DC
		.byte	$E6
		.byte	$F0

Rowbase		.word	$0000			; offset for each row of pixels in character font
		.word	$0140
		.word	$0280
		.word	$03C0
		.word	$0500
		.word	$0640
		.word	$0780
		.word	$08C0

colcnt		.word	$0000			; offset for each column of piexels in character font
		.word	$0008
		.word	$0010
		.word	$0018
		.word	$0020
		.word	$0028
		.word	$0030
		.word	$0038
		.word	$0040
		.word	$0048
		.word	$0050
		.word	$0058
		.word	$0060
		.word	$0068
		.word	$0070
		.word	$0078
		.word	$0080
		.word	$0088
		.word	$0090
		.word	$0098
		.word	$00A0
		.word	$00A8
		.word	$00B0
		.word	$00B8
		.word	$00C0
		.word	$00C8
		.word	$00D0
		.word	$00D8
		.word	$00E0
		.word	$00E8
		.word	$00F0
		.word	$00F8
		.word	$0100
		.word	$0108
		.word	$0110
		.word	$0118
		.word	$0120
		.word	$0128
		.word	$0130
		.word	$0138
;
; end of program
;