; /////////////////////////////////////////////////////////////////////////////////
;//                                                                               //
;//  This program and its associated documentation are provided for your personal //
;//  use only and appear here exclusively by permission of the copyright holder.  //
;//  Please contact the copyright holder before re-distributing, re-publishing    //
;//  or disseminating this copyrighted work. This code is not GPL or in the       //
;//  public domain. Please respect the author"s copyright.                        //
;//                                                                               //
;//  No waranty, either expressed or implied, are given.  I assume no liability   //
;//  for its use in any project or device.                                        //
;//                                                                               //
;//  Your use of this program indicates your acceptance of all license terms.     //
;//  This particular version is freeware as long as the copyright messages are    //
;//  left intact.                                                                 //
;//                                                                               //
; /////////////////////////////////////////////////////////////////////////////////
;
;****************************************************************************
; SBC-3 Reset & Interrupt Handlers
;****************************************************************************

framecnt	=	$FF			; NMI will inc this each frame

;		*= 	$FF00			; put this in last page of RAM

;--------------Reset handler----------------------------------------------
Reset		SEI				; diable interupts
		CLD				; clear decimal mode		 
		LDX	#$FF			;
		TXS				; init stack pointer
		CLC				; Clear carry flag, prep for native mode
		#XCE				; XCE - Set native Mode ON
		lda	#$00			; set DBR reg to page $00
		pha				; put on stack
		#PLB				; PLB -> set DBR
clr_mem						; Fill RAM with $00 except the ROM Shadow $08000-$0FFFF
		#REP 	$30			; set 16 bit registers
		#LDAI16 $0000			; zero
		#LDXI16 $7FCE			; # of bytes to store
Clrm1		DEX					
		DEX					
		#STA_ALX $0031, $00		; fill $0000-$01FF with $00
		BNE 	Clrm1
		#STA_ALX $7FFE, $00		; fill $7FFE-$7FFF with $00
		#LDXI16 $0000			; # bytes (0x10000)
Clrm3					DEX					
		DEX					
		#STA_ALX $0000, $01		; fill $10000-$1FFFF with $00
		#STA_ALX $0000, $02		; fill $20000-$2FFFF with $00
		#STA_ALX $0000, $03		; fill $30000-$3FFFF with $00
		#STA_ALX $0000, $04		; fill $40000-$4FFFF with $00
		#STA_ALX $0000, $05		; fill $50000-$5FFFF with $00
		#STA_ALX $0000, $06		; fill $60000-$6FFFF with $00
		#STA_ALX $0000, $07		; fill $70000-$7FFFF with $00
		BNE	Clrm3			; loop thru all 65536 bytes per block
		#SEP	$30		 	; 8 bit registers

		jsr	VIA_init		; init the VIA's
		jsr	dispinit		; init text display on page 1
		jsr	SPI_init		; init SPI Master device
		jsr	SPIKB_init		; init SPI KB at device 6
		jsr	SPISER_init		; init SPI RS-232 at device 6
		CLI				; Enable interrupt system
		JMP	Init_DOS_COLD		; Cold start DiskOS

NMIVID				 pha		; VID NMI Handler - save a
		#LDA_AL $0030, $00		; reset IRQ flag by read of VDR
		#LDA_AL framecnt, $00	 	; get frame counter
		inc				; inc +1
		#STA_AL framecnt, $00	 	; save new counter
		pla				; restore A
IRQRET				 RTI		; return from Interrupt
;
; 65C816 Firmware Jump Vectors
; (E)=0 Native Mode
; NULjmp = $FFE0
; NULjmp = $FFE2
; COPjmp = $FFE4
; BRKjmp = $FFE6
; ABTjmp = $FFE8
; NMIjmp = $FFEA						 
; NULjmp = $FFEC						 
; IRQjmp = $FFEE
; (E)=1 Emulation Mode
; NULjmp = $FFF0
; NULjmp = $FFF2
; COPjmp = $FFF4
; NULjmp = $FFF6
; ABTjmp = $FFF8
; NMIjmp = $FFFA						 
; RESjmp = $FFFC						 
; IRQjmp = $FFFE						 

		*=		$FFC0
		.BYTE "SBC-3.1 (C) Daryl Rictor 2008", $0d, $0a, $00

		*=		$FFE0

		.word	Reset			; Reserved
		.word	Reset			; Reserved
		.word	Reset			; COP
		.word	BRKroutine		; BRK
		.word	Reset			; ABT
		.word	NMIVID			; NMI
		.word	Reset			; Reserved
		.word	IRQRET			; IRQ

		.word	Reset			; Reserved
		.word	Reset			; Reserved
		.word	Reset			; COP
		.word	Reset			; Reserved
		.word	Reset			; ABT
		.word	NMIVID			; NMI
		.word	Reset			; RES
		.word	IRQRET			; IRQ/BRK

;end of file
