;
;****************************************************************************
; SBC-3 Reset & Interrupt Handlers
;****************************************************************************

;		*= 	$FF00			; put this in last page of RAM

;------------- Reset handler----------------------------------------------
Reset		SEI				; diable interupts
		CLD				; clear decimal mode		 
		CLC				; Clear carry flag, prep for native mode
		#XCE				; XCE - Set native Mode ON
		#SEP	$30			; restore 8 bit Acc and Index
		lda	#$01			; high addr of stack
		#XBA				; A -> B
		lda	#$FF			; low addr of stack
		#TCS				; clear stack
		lda	#$00			; 
		pha				; put on stack
		pha				; put on stack
		#PLD				; PLD -> set D register (direct page) $0000
; copy ROM to Block 7
		#REP 	$30			; set 16 bit registers
		#LDXI16  $8000			; # of bytes to store
Cpyrom		#LDA_ALX  $7ffe, $00     	; get $08000-$0FFFF
		#STA_ALX  $7ffe, SYSBANK	; put $78000-$7FFFF
		DEX					
		DEX					
		BNE 	Cpyrom
		#SEP	$30		 	; 8 bit registers
		lda	#SYSBANK
		pha
		#PLB				; set DBR to $07
		lda	#$10			; 
		trb	VDR			; Write protect off
		#JMP_AL	 Clr_mem, SYSBANK	; jump into block 7 and continue there...

clr_mem						; Fill RAM with $00 except the top page of block 0 -  $0FF00-$0FFFF

		#REP 	$30			; set 16 bit registers
		#LDAI16  $0000			; zero
		#LDXI16  $FECE			; # of bytes to store
Clrm1		DEX					
		DEX					
		#STA_ALX  $0031, $00		; fill $00031-$0FEFE with $00
		BNE 	Clrm1
		#STA_ALX  $FEFE, $00		; fill $0FEFE-$0FEFF with $00
		#LDXI16  $0000			; # bytes (0x10000)
Clrm3		DEX					
		DEX					
		#STA_ALX  $0000, $01		; fill $10000-$1FFFF with $00
		#STA_ALX  $0000, $02		; fill $20000-$2FFFF with $00
		#STA_ALX  $0000, $03		; fill $30000-$3FFFF with $00
		#STA_ALX  $0000, $04		; fill $40000-$4FFFF with $00
		#STA_ALX  $0000, $05		; fill $50000-$5FFFF with $00
		#STA_ALX  $0000, $06		; fill $60000-$6FFFF with $00
		BNE	Clrm3			; loop thru all 65536 bytes per block

		#LDXI16  $8000			; # bytes (0x8000)
Clrm4		DEX					
		DEX					
		#STA_ALX  $0000, $07		; fill $70000-$77FFF with $00
		BNE	Clrm4			; loop thru 
		#SEP	$30		 	; 8 bit registers

		lda	#SYSBANK
		pha
		#PLB				; set DBR to $07
		jsr	VIA_init		; init the VIA's
		jsr	dispinit		; init text display on page 1
		jsr	SPI_init		; init SPI Master device
		jsr	SPIKB_init		; init SPI KB at device 6
		jsr	SPISER_init		; init SPI RS-232 at device 6
		CLI				; Enable interrupt system
		JMP	Init_DOS_COLD		; Cold start DiskOS

NMIVID         pha                     		; VID NMI Handler - save a
               #LDA_AL VDR, IOBANK   		; reset IRQ flag by read of VDR
               #LDA_AL frmcnt, SYSBANK 		; get frame counter
               inc                     		; inc +1
               #STA_AL frmcnt, SYSBANK		; save new counter
               bne NMIVID2				
               #LDA_AL frmcnt+1, SYSBANK	; get frame counter
               inc                     		; inc +1
               #STA_AL frmcnt+1, SYSBANK	; save new counter
               bne NMIVID2				
               #LDA_AL frmcnt+2, SYSBANK	; get frame counter
               inc                     		; inc +1
               #STA_AL frmcnt+2, SYSBANK	; save new counter
NMIVID2        pla                     		; restore A
IRQRET	       RTI				; return from Interrupt, restore P register
BRKJMP	       #JMP_AL  BrkRoutine, SYSBANK	; break in bank $07

;
; 65C816 Firmware Jump Vectors
; (E)=0 Native Mode
; NULjmp = $FFE0
; NULjmp = $FFE2
; COPjmp = $FFE4
; BRKjmp = $FFE6
; ABTjmp = $FFE8
; NMIjmp = $FFEA						 
; NULjmp = $FFEC						 
; IRQjmp = $FFEE
; (E)=1 Emulation Mode
; NULjmp = $FFF0
; NULjmp = $FFF2
; COPjmp = $FFF4
; NULjmp = $FFF6
; ABTjmp = $FFF8
; NMIjmp = $FFFA						 
; RESjmp = $FFFC						 
; IRQjmp = $FFFE						 

		*=		$FFC0
		.BYTE "SBC-3.1 (C) Daryl Rictor 2008", $0d, $0a, $00

		*=		$FFE0

		.word	Reset			; Reserved
		.word	Reset			; Reserved
		.word	Reset			; COP
		.word	BRKJMP			; BRK
		.word	Reset			; ABT
		.word	NMIVID			; NMI
		.word	Reset			; Reserved
		.word	IRQRET			; IRQ

		.word	Reset			; Reserved
		.word	Reset			; Reserved
		.word	Reset			; COP
		.word	Reset			; Reserved
		.word	Reset			; ABT
		.word	IRQRET			; NMI
		.word	Reset			; RES
		.word	IRQRET			; IRQ/BRK

;end of file
