/*  3-D Maze, adapted from code found under the 
 *  user contributions on www.cc65.org
 *
 *  Modified and enhanced for my the SBC-3 project.
 *  
 */

#include <sbc3.h>
#include <stdio.h>
#include <stdlib.h>		// random
#include <time.h>
#define random(MAX) rand()%MAX
#include <tgi.h>
#include <conio.h>		// cgetc
#include <unistd.h>		// sleep

/* WARNING: The code relies on the X and Y dimensions (width and height) to
 * fit within an unsigned char. The product (the number of cells) may be
 * greater, however.
 */

enum { UP, RIGHT, DOWN, LEFT };	// equally NORTH,EAST,SOUTH,WEST

#define RIGHT  1		// walls and borders
#define BOTTOM 2

typedef struct {
    unsigned char Walls;
    unsigned char Border;
    unsigned short Path;
    unsigned char Hist;
} CELL;


struct {
    unsigned char Width, Height;
    unsigned NumCells;
    unsigned char Dir[3];
    CELL *Maze;
    unsigned Entry;
    unsigned CenterX;
    unsigned CenterY;
} g;


#define Cell( Row, Col )       ((Row) * g.Width + (Col))
#define IsConnected( C1, C2 )  (g.Maze[ (C1) ].Path == g.Maze[ (C2) ].Path)
#define GoodDir( Cell, Dir )   ((g.Maze[ (Cell) ].Border & (Dir)) == 0)

#define ROTATE_LEFT( Direction )   ((Direction + 3) & 0x3)
#define ROTATE_RIGHT( Direction )  ((Direction + 1) & 0x3)

enum { CENTER_WALL, LEFT_WALL, RIGHT_WALL, RIGHT_EDGE, LEFT_EDGE, HIST };
enum { UNMOVED, MOVED, BLOCKED, EXITED };

int t;			// tracking dots toggle



void Disp (unsigned char Direction, unsigned a, unsigned b) {

    int x;
    int y;

    for (y=0; y < g.Height; y++) {
       for (x=0;x < g.Width; x++) {
          if (g.Maze[Cell(y,x)].Hist == 1) {
              if (t) tgi_circle(x*10+5, y*10+5,1);		
          }
          if (y<g.Height-1) {
            if (g.Maze[Cell(y,x)].Walls == 3) {
              tgi_line(x*10,y*10+10,x*10+10,y*10+10);
              tgi_line(x*10+10,y*10,x*10+10,y*10+10);
	    }
            if (g.Maze[Cell(y,x)].Walls == 2)   tgi_line(x*10,y*10+10,x*10+10,y*10+10);
            if (g.Maze[Cell(y,x)].Walls == 1)   tgi_line(x*10+10,y*10,x*10+10,y*10+10);
          } else {
            if (g.Maze[Cell(y,x)].Walls == 1)   tgi_line(x*10+10,y*10,x*10+10,y*10+10);
            if (g.Maze[Cell(y,x)].Walls == 3)   tgi_line(x*10+10,y*10,x*10+10,y*10+10);
            if (x != g.Entry)   tgi_line(x*10,y*10+10,x*10+10,y*10+10);
          }
       } 
    }	

    tgi_line(0,0,0,g.Height*10);    
    tgi_line(0,0,g.Width*10,0);    

    switch (Direction) {

    case RIGHT:
      tgi_line(a*10+2,b*10+3,a*10+8,b*10+5);
      tgi_line(a*10+2,b*10+7,a*10+8,b*10+5);
      tgi_line(a*10+2,b*10+3,a*10+2,b*10+7);
      break;

    case LEFT:
      tgi_line(a*10+8,b*10+3,a*10+2,b*10+5);
      tgi_line(a*10+8,b*10+7,a*10+2,b*10+5);
      tgi_line(a*10+8,b*10+3,a*10+8,b*10+7);

      break;
    case UP:
      tgi_line(a*10+3,b*10+8,a*10+5,b*10+2);
      tgi_line(a*10+7,b*10+8,a*10+5,b*10+2);
      tgi_line(a*10+3,b*10+8,a*10+7,b*10+8);

      break;
    case DOWN:
      tgi_line(a*10+3,b*10+2,a*10+5,b*10+8);
      tgi_line(a*10+7,b*10+2,a*10+5,b*10+8);
      tgi_line(a*10+3,b*10+2,a*10+7,b*10+2);

      break;
    }
}


// Draws a wall shape

void DrawShape (unsigned char shape, unsigned Size)
{
    unsigned OldSize = Size * 2;

    // vertical lines shared across all shapes (closes the shapes)
    tgi_line (g.CenterX - Size, g.CenterY - Size,
	      g.CenterX - Size, g.CenterY + Size);
    tgi_line (g.CenterX + Size, g.CenterY - Size,
	      g.CenterX + Size, g.CenterY + Size);

    switch (shape) {
    case CENTER_WALL:
	tgi_line (g.CenterX - Size, g.CenterY - Size,
	     	  g.CenterX + Size, g.CenterY - Size);
	tgi_line (g.CenterX - Size, g.CenterY + Size,
	     	  g.CenterX + Size, g.CenterY + Size);
	break;

    case LEFT_WALL:
       if (g.CenterY < OldSize) {
	tgi_line (60 , 0,
	     	  g.CenterX - Size, g.CenterY - Size);
	tgi_line (61, g.CenterY * 2 - 1,
	     	  g.CenterX - Size, g.CenterY + Size);
       }
       else {
	tgi_line (g.CenterX - OldSize, g.CenterY - OldSize,
	     	  g.CenterX - Size, g.CenterY - Size);
	tgi_line (g.CenterX - OldSize, g.CenterY + OldSize,
	     	  g.CenterX - Size, g.CenterY + Size);
       } 
	break;

    case RIGHT_WALL:
       if (g.CenterY < OldSize) {
	tgi_line (320-60 , 0,
	     	  g.CenterX + Size, g.CenterY - Size);
	tgi_line (320-61, g.CenterY * 2 - 1,
	     	  g.CenterX + Size, g.CenterY + Size);
	} else {
	tgi_line (g.CenterX + OldSize, g.CenterY - OldSize,
	     	  g.CenterX + Size, g.CenterY - Size);
	tgi_line (g.CenterX + OldSize, g.CenterY + OldSize,
	     	  g.CenterX + Size, g.CenterY + Size);
	}
	break;

    case RIGHT_EDGE:
	tgi_line (g.CenterX + OldSize, g.CenterY - Size,
		  g.CenterX + Size, g.CenterY - Size);
	tgi_line (g.CenterX + OldSize, g.CenterY + Size,
		  g.CenterX + Size, g.CenterY + Size);
	break;

    case LEFT_EDGE:
	tgi_line (g.CenterX - OldSize, g.CenterY - Size,
		  g.CenterX - Size, g.CenterY - Size);
	tgi_line (g.CenterX - OldSize, g.CenterY + Size,
		  g.CenterX - Size, g.CenterY + Size);
	break;

    case HIST:
        if (t) tgi_circle(g.CenterX, g.CenterY + ((OldSize+Size)/2),Size/5 + 1);
	break;

    }
}


// moves one cell in direction 'Direction' starting from cell *x,*y
//   assumes *x,*y is within the maze and Direction is a valid direction.

int MoveXY (unsigned char Direction, unsigned *x, unsigned *y)
{
    switch (Direction) {
    case (UP):
	if (*y == 0)
	    return (BLOCKED);
	if (g.Maze[Cell ((*y) - 1, *x)].Walls & BOTTOM)
	    return (BLOCKED);
	(*y)--;
	return (MOVED);

    case (RIGHT):
	if (g.Maze[Cell (*y, *x)].Walls & RIGHT)
	    return (BLOCKED);
	(*x)++;
	return (MOVED);

    case (DOWN):
	if (g.Maze[Cell (*y, *x)].Walls & BOTTOM) {
	    if (*x != g.Entry)
		return (BLOCKED);
	    else
               if (*y == g.Height-1)
 		 return (EXITED);
               else
  		 return (BLOCKED);
 	}
	(*y)++;
	return (MOVED);

    case (LEFT):
	if (*x == 0)
	    return (BLOCKED);
	if (g.Maze[Cell (*y, (*x) - 1)].Walls & RIGHT)
	    return (BLOCKED);
	(*x)--;
	return (MOVED);
    }

    return (BLOCKED);
}


//  Calculate which shapes are needed for a given cell

void DrawBlock (unsigned char Direction, unsigned x, unsigned y, unsigned Size)
{
    unsigned tx, ty;		// temporary position for movement tests
    unsigned char movement;	// result of movement tests

    // place hist mark on floor    
    if (g.Maze[Cell(y,x)].Hist == 1) DrawShape (HIST, Size);


    // Look to the left - draw nessary shapes
    tx = x;
    ty = y;

    movement = MoveXY (ROTATE_LEFT (Direction), &tx, &ty);

    if (movement == BLOCKED) {
	DrawShape (LEFT_WALL, Size);
    } else {
	if (movement == MOVED) {
	    if (MoveXY
		(ROTATE_RIGHT (ROTATE_LEFT (Direction)), &tx, &ty) == BLOCKED) {
		DrawShape (LEFT_EDGE, Size);
	    } else {
	    }			// Draw nothing - there is just empty space to the left.
	} else {
	}			// Draw nothing - the Exit is to the left
    }

    // Look to the right - draw nessary shapes
    tx = x;
    ty = y;

    movement = MoveXY (ROTATE_RIGHT (Direction), &tx, &ty);

    if (movement == BLOCKED) {
	DrawShape (RIGHT_WALL, Size);
    } else {
	if (movement == MOVED) {
	    if (MoveXY
		(ROTATE_LEFT (ROTATE_RIGHT (Direction)), &tx, &ty) == BLOCKED) {
		DrawShape (RIGHT_EDGE, Size);
	    } else {
	    }			// Draw nothing - there is just empty space to the right.
	} else {
	}			// Draw nothing - the Exit is to the right
    }

    // Look straight ahead
    if (MoveXY (Direction, &x, &y) == BLOCKED) 
	DrawShape (CENTER_WALL, Size);

    
}


// Draws the maze as seen from cell x,y facing direction Direction

void Draw3d (unsigned char Direction, unsigned x, unsigned y)
{
    unsigned Size = g.CenterX / 2;

    while (1) {
	DrawBlock (Direction, x, y, Size);

	if (MoveXY (Direction, &x, &y) != MOVED)
	    break;

	// reduce shape size
	Size /= 2;
        if (Size < 2) Size = 2;
    }
}


// Interactive exploration of the maze in 3d

void Explore (void)
{
    int h;			// help screen toggle
    int c;   	   		// character input
    unsigned x, y;	       	// "position" of viewpoint in maze
    unsigned char Direction;  	// direction of view (enumerated)
    unsigned char Status;	// results of attempted move.

    g.CenterX = tgi_getxres () / 2;
    g.CenterY = tgi_getyres () / 2;
    tgi_setcolor (COLOR_WHITE);

    // random starting position and direction
    x = random (g.Width);
    y = random (g.Height);
    Direction = random (4);
    Draw3d (Direction, x, y);

    
    c = 0;
    h = 0;
    t = 0;

    g.Maze[Cell(y,x)].Hist = 1;

    while (1) {


	Status = UNMOVED;

	c = cgetc ();

	if (c == 'q' || c == 'Q')
	    break;

	switch (c) {
	case 'J':	   	// Rotate Left
	case 'j':
	case '4':
#if defined(CH_CURS_LEFT)
	case CH_CURS_LEFT:
#endif
	    Direction = ROTATE_LEFT (Direction);
	    break;

	case 'L':		// Rotate Right
	case 'l':
	case '6':
#if defined(CH_CURS_RIGHT)
	case CH_CURS_RIGHT:
#endif
	    Direction = ROTATE_RIGHT (Direction);
	    break;

	case 'I':		// Move Forward
	case 'i':
	case '8':
#if defined(CH_CURS_UP)
	case CH_CURS_UP:
#endif
	    Status = MoveXY (Direction, &x, &y);
	    break;

	case 'K':
	case 'k':
	case '2':
#if defined(CH_CURS_DOWN)
	case CH_CURS_DOWN:
#endif
	    Status = MoveXY ((Direction + 2) % 4, &x, &y);
	    break;

	case 'h':		// toggle help screen
	case 'H':
            h = 1 - h;
            break;

	case 't':		// toggle tracking dots
	case 'T':
            t = 1 - t;
            break;

	default:
	    break;
	}

	if (Status == BLOCKED) {
	    // beep the speaker

	} else {
	    if (Status == EXITED) {
		break;
	    } else {
		tgi_clear ();
                g.Maze[Cell(y,x)].Hist = 1;
                if (h)  { Disp (Direction, x, y);
                } else  { Draw3d (Direction, x, y);
                }

	    }
	}
    }

    if (Status == EXITED) {
	printf ("Congratulations, you found your way out!\n");
    }
}


// Joins two cells together into the same path.

void Join (int CellNum, int Dir)
{
    unsigned i, OldPath;

    g.Maze[CellNum].Walls &= ~Dir;	// clear the wall between them

    // now merge the paths

    OldPath = g.Maze[CellNum + g.Dir[Dir]].Path;
	
    i = g.NumCells-1;
    do {
	if (g.Maze[i].Path == OldPath) {
	    g.Maze[i].Path = g.Maze[CellNum].Path;
	}
	--i;
    } while (i);
}


// Randomly connects a cell with either its bottom or right neighbor.

int Connect (int CellNum)
{
    unsigned Neighbor;
    unsigned char Dir;

    Dir = random (2) + 1;	// random direction, 1 or 2

    Neighbor = CellNum + g.Dir[Dir];

    if (!GoodDir (CellNum, Dir) || IsConnected (CellNum, Neighbor)) {
	Dir = 3 - Dir;		// get the OTHER neighbor

	Neighbor = CellNum + g.Dir[Dir];

	if (!GoodDir (CellNum, Dir) || IsConnected (CellNum, Neighbor)) {
	    return (0);
	}
    }

    Join (CellNum, Dir);

    return (1);
}


// Generates the maze.

void Generate (void)
{
    unsigned ACell, Cnt;

    // Connect a random cell in the maze to another path.
    // Continue doing this until there is only one path in the maze.

    do {
	for (ACell = random (g.NumCells), Cnt = g.NumCells; Cnt; ++ACell, --Cnt) {
	    if (ACell == g.NumCells)
	     	ACell = 0;

	    if (Connect (ACell))
	     	break;
	}
    }
    while (Cnt);
}


// Initialization code.

void Init (void)
{
    unsigned i;
    unsigned char Row, Col;

    _randomize ();

    g.Dir[1] = 1; 		// right neighbor
    g.Dir[2] = g.Width;		// bottom neighbor

    if ((g.Maze = calloc (g.NumCells, sizeof (CELL))) == NULL) {
	puts ("ERROR: Out of memory.");
	exit (-1);
    }

    for (Row = 0; Row < g.Height; ++Row) {
	g.Maze[Cell (Row, g.Width - 1)].Border |= RIGHT;
    }

    for (Col = 0; Col < g.Width; ++Col) {
	g.Maze[Cell (g.Height - 1, Col)].Border |= BOTTOM;
    }

    for (i = 0; i < g.NumCells; ++i) {
	g.Maze[i].Path = i;
	g.Maze[i].Walls = RIGHT + BOTTOM;
    }
}


int main (int Argc, char *Argv[])
{
    long T;
    long C;
    unsigned sec;
    unsigned sec10;

    if (Argc > 1) {
    	g.Height = atoi (Argv[2]);
    } else {
     	g.Height = 10;
    }
    if (Argc > 0) {
     	g.Width = atoi (Argv[1]);
    } else {
     	g.Width = 10;
    }

    g.NumCells = g.Width * g.Height;

    if (g.Width < 2 || g.Width > 50 || g.Height < 2 || g.Height > 50) {
    	puts ("Bad dimensions.");
    } else {
    	Init ();

    	// open up a random column on the bottom
    	g.Entry = random (g.Width);

	printf ("Generating labyrinth...\r");
	T = clock ();
    	Generate ();
       	T = clock () - T;

    tgi_install(&sbc3_tgi);
    tgi_init ();

	C = clock ();
    	Explore ();
       	C = clock () - C;

    tgi_done();

    }
    sec =  (T / CLK_TCK);
    sec10 = (( T % CLK_TCK) * 1000) / CLK_TCK;
    printf ("Generation: %u.%03u seconds\r", sec, sec10);

    sec =  (C / CLK_TCK);
    sec10 = (( C % CLK_TCK) * 1000) / CLK_TCK;
    printf ("Exit time: %u.%03u seconds\r", sec, sec10);
    return 0;
}
