; /////////////////////////////////////////////////////////////////////////////////
;//                                                                               //
;//  This program and its associated documentation are provided for your personal //
;//  use only and appear here exclusively by permission of the copyright holder.  //
;//  Please contact the copyright holder before re-distributing, re-publishing    //
;//  or disseminating this copyrighted work. This code is not GPL or in the       // 
;//  public domain. Please respect the author's copyright.                        //
;//                                                                               //
;//  No waranty, either expressed or implied, are given.  I assume no liability   //
;//  for its use in any project or device.                                        //
;//                                                                               //
;//  Your use of this program indicates your acceptance of all license terms.     //
;//  This particular version is freeware as long as the copyright messages are    //
;//  left intact.                                                                 //  
;//                                                                               //
; /////////////////////////////////////////////////////////////////////////////////
;
;---------------------------------------------------------------------
;  SBC Firmware V6.2, 11-06-08, by Daryl Rictor
;  For 65816 running Native mode
;
;*********************************************************************       
;  local Zero-page variables
;
SrcBlk         =     $E0               ; 1 byte upper address bits 16-23 
DestBlk        =     $E1               ; 1 byte upper address bits 16-23 
linecnt        =     $E2               ; 1 byte
Modejmp        =     $E3               ; 1 byte
Hexdigcnt      =     $E4               ; 1 byte
OPCtxtptr      =     $E5               ; 1 byte
Memchr         =     $E6               ; 1 byte
Startaddr      =     $E7               ; 2 bytes
Startaddr_H    =     $E8               ;  
Addrptr        =     $E9               ; 2 bytes
Addrptr_H      =     $EA               ;
Hexdigits      =     $EB               ; 3 bytes
Hexdigits_H    =     $EC               ;
Hexdigits_B    =     $ED               ;
Memptr         =     $EE               ; 2 bytes
Memptr_H       =     $EF               ;
;
; Local Non-Zero Page Variables
;
buffer         =     $0200             ; keybd input buffer (127 chrs max)
;
; register holding locations
;
PBR            =     $02f0             ;  PC Bank
PCH            =     $02f1             ;  PC High
PCL            =     $02f2             ;  PC low 
DBR            =     $02f3             ;  Data bank
B              =     $02f4             ;  B accumulator
ACC            =     $02f5             ;  Accumulator (A)
XREG           =     $02f6             ; hold X reg 16 bit
YREG           =     $02f8             ; hold Y reg 16 bit
D              =     $02fa             ; D reg 16 bit 
SPTR           =     $02fc             ; stack pointer 16 bit
E              =     $02fe             ; Emulation mode bit
PREG           =     $02ff             ; hold status register (P) 8 bit
;               
; *************************************************************************
; kernal commands
; *************************************************************************
; PrintReg     - Print register contents (needs BRKroutine to load values)
; Print2Byte   - prints AAXX hex digits
; Print1Byte   - prints AA hex digits
; PrintDig     - prints A hex nibble (low 4 bits)
; Print_CR     - prints a CR (ASCII 13)and LF (ASCII 10)
; PrintXSP     - prints # of spaces in X Reg
; Print2SP     - prints 2 spaces
; Print1SP     - prints 1 space
; Input        - print <CR> and prompt then get a line of input, store at buffer
; Input_Chr    - get one byte from input port, waits for input
; Scan_Input   - Checks for an input character (no waiting)
; Output       - send one byte to the output port
; Bell         - send ctrl-g (bell) to output port
; Delay        - delay loop
; *************************************************************************
;
RegData        .byte"PBR=  PC=.  DBR=  B=  A=~X=.  Y=.  D=.  S=.~E=  P=  (NVMXDIZC):"
;
PrintReg       ldx   #$ff              ;
               ldy   #$ff              ;
Printreg0      Jsr   Print_CR          ; Lead with a CR
Printreg1      iny                     ;
               lda   Regdata,y         ;
               cmp   #"."              ;
               beq   Printreg2         ; if ., print a second byte
               cmp   #"~"              ; 
               beq   Printreg0         ; if ~, print CR and do new line
               jsr   output            ; print prompt data
               cmp   #":"              ; 
               beq   Printreg3         ; if :, expand P into bits and exit
               cmp   #"="              ; 
               bne   Printreg1         ;
Printreg2      inx                     ;
               lda   PBR,x             ;  
               jsr   Print1Byte        ;
               bra   Printreg1         ;
Printreg3      lda   PBR,x             ; get Preg
               ldx   #$08              ; 
Printreg4      rol                     ;
               tay                     ;
               lda   #$31              ;
               bcs   Printreg5         ;
               dec                     ;
Printreg5      jsr   Output            ;
               tya                     ;
               dex                     ;
               bne   Printreg4         ;
                                       ; fall into the print CR routine
Print_CR       PHA                     ; Save Acc
               LDA   #$0D              ; "cr"
               JSR   OUTPUT            ; send it
               LDA   #$0A              ; "lf"
               JSR   OUTPUT            ; send it
               PLA                     ; Restore Acc
               RTS                     ; 
;
Print2Byte     JSR   Print1Byte        ;  prints AAXX hex digits
               TXA                     ;
Print1Byte     PHA                     ;  prints AA hex digits
               LSR                     ;  MOVE UPPER NIBBLE TO LOWER
               LSR                     ;
               LSR                     ;
               LSR                     ;
               JSR   PrintDig          ;
               PLA                     ;
PrintDig       AND   #$0F              ;
               CMP   #$0A              ;
               BCC   PrintDig1         ;
               ADC   #$66              ;
PrintDig1      EOR   #$30              ;
               jmp   output            ;
PrintXSP1      JSR   Print1SP          ;
               dex                     ;
PrintXSP       cpx   #$00              ;
               bne   PrintXSP1         ;
               rts                     ;
Print2SP       jsr   Print1SP          ; print 2 SPACES
Print1SP       LDA   #$20              ; print 1 SPACE
               JMP   OUTPUT            ;
Input          jsr   Print_CR          ; New Line
               lda   #">"              ; Monitor Prompt ">"
               jsr   Output            ; Print Prompt
               ldy   #$ff              ; pointer
InputWait      jsr   Input_Chr         ; get a character
               cmp   #$20              ; is ctrl char?
               BCS   InputSave         ; no, echo chr 
               cmp   #$0d              ; cr
               Beq   InputDone         ; done
               cmp   #$1B              ; esc
               beq   Input             ; cancel and new line
               cmp   #$08              ; bs
               beq   backspace         ;
               cmp   #$09              ; TAB key
               beq   tabkey            ;
               cmp   #$02              ; Ctrl-B
               bne   InputWait         ; Ignore other codes
               brk                     ; Force a keyboard Break cmd
backspace      cpy   #$ff              ;
               beq   InputWait         ; nothing to do
               dey                     ; remove last char
               Lda   #$08              ; backup one space
               jsr   Output            ;
;               Lda   #$20              ; Print space (destructive BS)
;               jsr   Output            ;
;               Lda   #$08              ; backup one space
;               jsr   Output            ;
               BRA   InputWait         ; ready for next key
tabkey         lda   #$20              ; convert tab to space
               iny                     ; move cursor
               bmi   InputTooLong      ; line too long?
               sta   Buffer,y          ; no, save space in buffer
               jsr   output            ; print the space too
               tya                     ; test to see if tab is on multiple of 8
               and   #$07              ; mask remainder of cursor/8
               bne   tabkey            ; not done, add another space
               bra   InputWait         ; done. 
InputSave      CMP   #$61              ;   ucase
               BCC   InputSave1        ;
               SBC   #$20              ;
InputSave1     INY                     ;
               BMI   InputTooLong      ; get next char (up to 127)
               STA   Buffer,y          ;
               JSR   Output            ; OutputCharacter
               BRA   InputWait         ;
InputDone      INY                     ;
InputTooLong   LDA   #$0d              ; force CR at end of 128 characters 
               sta   Buffer,y          ;
               JSR   Output            ;
;              lda   #$0a              ; lf Char   
;              JSR   Output            ;
               RTS                     ;
;
bell           LDA  #$07               ; Ctrl G Bell
               jmp  Output             ; 
;
Delay          PHA                     ; use A to execute a delay loop
delay1         DEC                     ;
               BNE   delay1            ;
               PLA                     ;
               DEC                     ;
               BNE   Delay             ;
GRTS           RTS                     ;
;
BRKroutine     sta   ACC               ; save A    Monitor"s break handler
               #XBA                    ;
               STA   B                 ; save B 
               #REP  $30               ; 16 bit mode
               txa                     ; get 16 bit X
               #XBA                    ; switch low & hi bytes for printing
               sta   Xreg              ; save X 
               tya                     ; get 16 bit Y
               #XBA                    ; switch low & hi bytes for printing
               sta   Yreg              ; save Y
               #SEP  $30               ; 8 bit mode
               pla                     ;  
               sta   Preg              ; save P
               pla                     ; PCL
               plx                     ; PCH
               sec                     ;
               sbc   #$02              ;
               sta   PCL               ; backup to BRK cmd
               bcs   Brk2              ;
               dex                     ;
Brk2           stx   PCH               ; save PC
               pla                     ; 
               sta   PBR               ; save PBR
               #PHB                    ; 
               pla                     ;
               sta   DBR               ; save DBR
               lda   #$00              ; E=0 if reset 
               #XCE                    ; get E bit
               bcc   Brk3              ; 
               inc                     ; E=1 if set
Brk3           #XCE                    ; restore E back
               sta   E                 ; save E
               #TDC                    ;
               sta   D+1               ;
               #XBA                    ;
               sta   D                 ; save D
               #TSC                    ;
               sta   SPTR+1            ;
               #XBA                    ;
               sta   SPTR              ; save S
               jsr   Bell              ; Beep speaker
               jsr   PrintReg          ; dump register contents 
               lda   #$01              ;
               #XBA                    ;
               lda   #$FF              ; 
               #TCS                    ; clear stack
               cli                     ; enable interrupts again
               jmp   Monitor           ; start the monitor

;*************************************************************************
;     
;  Monitor Program 
;
;*************************************************************************
MonitorBoot    
               jsr   bell              ; beep ready
               JSR   Version           ;
               stz   SrcBlk            ; init source and destination banks to 00
               stz   DestBlk           ;
Monitor        lda   #$00              ;
               PHA                     ;
               #PLB                    ; set DBR to $00
               lda   #$01              ;  
               #XBA                    ;
               lda   #$FF              ; 
               #TCS                    ; set stack to $01FF
               JSR   input             ; line input
               LDA   #$00              ; clr vars
               TAY                     ; set to 1st character in line
               stz   LineCnt           ; normal list vs range list 
               stz   Memchr            ; view or modify
Mon01          stz   Hexdigits         ; holds parsed hex
               stz   Hexdigits_H       ;
               stz   Hexdigits_B       ;
               JSR   ParseHexDig       ; Get any Hex chars
               LDX   #CmdCount         ; get # of cmds currently used
Mon08          CMP   CmdAscii,X        ; is non hex cmd chr?
               BEQ   Mon09             ; yes x= cmd number
               DEX                     ;
               BPL   Mon08             ;
               BRA   Monitor           ; no
Mon09          PHX                     ; save command
               PHY                     ; Save input line pointer
               TXA                     ;
               ASL                     ; ptr * 2
               TAX                     ;  
               JSR   Mon10             ; Execute cmd
               PLY                     ;
               PLX                     ;
               BEQ   Monitor           ; done
               BRA   Mon01             ;
Mon10          JMP   (Cmdjmptbl,X)     ;
;--------------- Routines used by the Monitor commands ----------------------
ParseHexDig    STZ   Hexdigcnt         ;  cntr
ParseHex01     LDA   buffer,Y          ;
               INY                     ;
               TAX                     ;  SAVE A
               sec
               sbc   #$30              ; drop ASCII to decimal
               bcc   ParseHex09        ; not Hex - exit
               cmp   #$0A              ; > digit 9?
               BMI   ParseHex03        ; ishex
               cmp   #$11              ; Hex A
               bmi   ParseHex09        ; not Hex - exit
               cmp   #$17              ; Above Hex F
               bpl   ParseHex09        ; not Hex - exit
               sec                     ;  
               sbc   #$07              ; adjust A-F 
ParseHex03     LDX   #$04              ; shift hexdigits over for new nibble
ParseHex04     ASL   Hexdigits         ;
               ROL   Hexdigits_H       ;
               ROL   Hexdigits_B       ;
               DEX                     ;
               BNE   ParseHex04        ;
               TSB   Hexdigits         ; merge new nibble  
               DEC   Hexdigcnt         ; mark as valid hexdigit
               BRA   ParseHex01        ; get next 
ParseHex09     lda   Hexdigcnt         ; if more than 2 bytes entered, 
               beq   ParseHex10        ;
               cmp   #$FC              ;
               bcs   ParseHex10        ;
               lda   Hexdigits_B       ; save highest byte to Source Bank select
               sta   SrcBlk            ; 
ParseHex10     TXA                     ; restore A
               RTS                     ; Stored in HexDigits if HexDigCnt <> 0
;
Help_cmd       lda   #<Helptxt         ; lower byte - Menu of Commands
               sta   addrptr           ;
               lda   #>Helptxt         ; upper byte
               sta   addrptr+1         ;
Help_cmd1      lda   (addrptr)         ;
               beq   Help_Cmd4         ; $00 = done
               cmp   #$7e              ; "~" = new line
               bne   Help_cmd2         ;
               jsr   Print_CR          ;     
               bra   Help_cmd3         ;
Help_cmd2      jsr   Output            ;
Help_cmd3      jsr   Inc_addrptr       ;
               bra   Help_cmd1         ; 
Help_cmd4      rts 
;                     
Version        jsr   Print_CR          ; 
               ldx   #$00              ; set txt pointer
               lda   porttxt,x         ; get message text
PortReadyMsg   inx                     ;
               JSR   Output            ; put character to Port
               lda   porttxt,x         ; get message text
               bne   PortReadyMsg      ; 
               rts                     ;
;
Excute_cmd     lda   SrcBlk            ; if SrcBlk > 0, we need Long jump
               bne   exe2              ;
               jsr   exe1              ; else, just jsr to JMP 
               jmp   Monitor           ;
exe1           JMP   (Hexdigits)       ; JMP
exe2           lda   #$00              ;
               PHA                     ; force return to Bank 0
               jsr   exe3              ; JSR to long jmp
               jmp   Monitor           ; return to monitor
exe3           #JML  Hexdigits         ; Long Jump to program
;
COLN_cmd       lda   #$02              ; set secondary command mode to 2: Modify 
               .byte $2c               ; mask out next line to bypass 
DOT_cmd        lda   #$01              ; set secondary command mode to 1: Dump
               sta   Memchr            ; set modify flag
DOT_cmd1       LDX   Hexdigits         ; move address to addrptr
               LDA   Hexdigits+1       ;
               STX   Addrptr           ;
               STA   Addrptr+1         ;
               inc   LineCnt           ; range list command
               RTS                     ;
;
CR_cmd         CPY   #$01              ; CR alone?
               Beq   CR_cmd1           ; yes
               jsr   SP_cmd            ; no - do space command then exit
               stz   Memchr            ; 
               rts                     ;
CR_cmd1        LDA   Addrptr           ; CR alone - move addrptr to hexdigits
               ORA   #$07              ; to simulate entering an address
               STA   Hexdigits         ; 
               LDA   Addrptr+1         ;
               STA   Hexdigits+1       ;
               jsr   SP_cmd36          ; do space command then exit
               stz   Memchr            ;
               rts                     ;
SP_cmd         LDA   Hexdigcnt         ; Space command entry
               BEQ   SP_cmd5           ; any digits to process? no - done
               LDX   Memchr            ; yes - is sec cmd code 0?
               BEQ   SP_cmd1           ; yes - 
               DEX                     ; no - Is sec cmd = 1?       
               BEQ   SP_cmd33          ; yes 
               #PHB                    ; no - modify mem command
               lda   SrcBlk            ; save data to source bank
               PHA                     ; 
               #PLB                    ; set DBR for view
               LDA   Hexdigits         ; 
               STA   (Addrptr)         ;
               #PLB                    ; restore DBR
               JMP   Inc_addrptr       ; set to next address and return
SP_cmd1        JSR   DOT_cmd1          ; sec dig = 0  move address to addrptr
               BRA   SP_cmd33          ;
SP_cmd2        LDA   Addrptr           ; CR cmd entry 
               BIT   #$07              ; 
               Bne   SP_cmd4           ; else print next byte
SP_cmd3        pla                     ;
               sta   Addrptr+1         ;
               pla                     ;
               sta   Addrptr           ;
               LDA   #" "              ; 
               JSR   OUTPUT            ;
               bra   SP_cmd32          ; 
SP_cmd31       LDA   Addrptr           ; 
               BIT   #$07              ; 
               Beq   SP_cmd33          ; end of line
SP_cmd32       jsr   TX_cmd
               SEC                     ;  checks if range done
               LDA   Addrptr           ;
               SBC   Hexdigits         ;
               LDA   Addrptr+1         ;
               SBC   Hexdigits+1       ;
               jsr   Inc_addrptr       ;
               BCC   SP_cmd31          ; loop until range done
SP_brk         STZ   Memchr            ; reset sec cmd code
SP_cmd5        RTS                     ; done or no digits to process
SP_cmd33       JSR   Print_CR          ; "." cmd - display address and data 
               jsr   Scan_Input        ; see if brk requested
               bcs   SP_brk            ; if so, stop 
SP_cmd36       LDA   Addrptr+1         ; print address
               LDX   Addrptr           ;
               PHX                     ;
               PHA                     ;
               JSR   Print2Byte        ;
               LDA   #" "              ; 
               JSR   OUTPUT            ;
               LDA   #"-"              ; 
               JSR   OUTPUT            ;
SP_cmd4        LDA   #" "              ; 
               JSR   OUTPUT            ;
               jsr   GetMemVal         ;               
               JSR   Print1Byte        ;  print hex byte values
SP_cmd44       SEC                     ;  checks if range done
               LDA   Addrptr           ;
               SBC   Hexdigits         ;
               LDA   Addrptr+1         ;
               SBC   Hexdigits+1       ;
               jsr   Inc_addrptr       ;
               BCC   SP_cmd2           ; loop until range done
               BRA   SP_cmd3           ; finish last line text
TX_cmd         jsr   GetMemVal         ;
               AND   #$7F              ;
               CMP   #$7F              ;
               BEQ   TX_Cmd5           ;
               CMP   #$20              ; " "
               BCS   TX_Cmd6           ;
TX_Cmd5        LDA   #$2E              ; "." use "." if not printable char
TX_Cmd6        Jmp   OUTPUT            ;
;
Dest_cmd       LDX   Hexdigits         ;  ">" cmd code
               LDA   Hexdigits+1       ;
               STX   Memptr            ;  move address to memptr
               STA   Memptr+1          ;
               RTS                     ;  
;
Inc_addrptr    INC   Addrptr           ;  increments addrptr
               BNE   Inc_addr1         ;
               INC   Addrptr+1         ;
Inc_addr1      RTS                     ;
;
GetMemVal      #PHB                    ;
               lda   SrcBlk            ;  get view data from source bank
               PHA                     ; 
               #PLB                    ;  set DBR for view
               LDA   (Addrptr)         ;
               #PLB                    ;  restore DBR
               rts                     ;
;
Insert_cmd     lda   Linecnt           ;  "I" cmd code
               beq   Insert_3          ; abort if no . cmd entered
               lda   $30               ; get VDR
               pha                     ; save it
               and   #$EF              ; turn off write protect		
               sta   $30               ;
               lda   DestBlk           ; get dest bank
               sta   Insert_1+1        ; modify MVN
               lda   SrcBlk            ; get src bank
               sta   Insert_1+2        ; modify MVN
               #PHB                    ; save DBR 
               #REP  $30               ; set 16 bit Acc & Index
               ldx   Memptr            ; src addr
               clc                     ;
               lda   Memptr            ;
               sbc   Addrptr           ;
               pha                     ;
               adc   Hexdigits         ;
               tay                     ; dest addr 
               pla                     ;
               inc                     ; # bytes
Insert_1       #MVP  $00, $00          ; Move positive
               #SEP  $30               ; set 8 bit Acc & Index
               #PLB                    ; restore DBR
               pla                     ;
               sta   $30               ; restore VDR & write protect
Insert_3       RTS                     ;
;
Move_cmd       lda   Linecnt           ; use 16 bit mode and MVN to allow moves accross banks 
               beq   Move_cmd2         ; abort if no . cmd was used
               lda   $30               ; get VDR
               pha                     ; save it
               and   #$EF              ; turn off write protect		
               sta   $30               ;
               lda   DestBlk           ; get dest bank
               sta   Move_cmd1+1       ; modify MVN
               lda   SrcBlk            ; get src bank
               sta   Move_cmd1+2       ; modify MVN
               #PHB                    ; save DBR 
               #REP  $30               ; set 16 bit Acc & Index
               ldx   Addrptr           ; X= src
               ldy   Hexdigits         ; Y= dest
               sec                     ; C= count-1
               lda   Memptr            ;
               sbc   Addrptr           ;
Move_cmd1      #MVN  $00, $00          ; Move positive
               #SEP  $30               ; set 8 bit Acc & Index
               #PLB                    ; restore DBR
               pla                     ;
               sta   $30               ; restore VDR & write protect
Move_cmd2      rts                     ;
;
Serparm        lda   Hexdigcnt         ; set serial port parms
               beq   PutFile2          ; if no hex data, abort cmd
               lda   Hexdigits         ; get parm value
               jmp   Set_Ser           ; set the parms
Upload         LDA   hexDigcnt         ; Upload command
               beq   upload1           ; if entered U alone, use bank stored in Dest bank 
               jsr   DestB_cmd         ; else, update Dest bank then do upload
upload1        jmp   XModemRcv         ; 
;
PutFile        lda   Linecnt           ; Send data block via Xmodem to RS-232 port
               beq   PutFile2          ; abort if no . cmd was used
               jmp   XmodemSend        ; send file
PutFile2       rts                     ; 
;
SrcB_cmd       LDA   HexDigits         ;
               STA   SrcBlk            ;
               RTS                     ;
;
DestB_cmd      LDA   HexDigits         ; 
               STA   DestBlk           ;
               RTS                     ;
;   
X_Cmd          lda   #IMML_mode-SIRQ_mode  ;
               sta   IMM_mode+1        ;
LIST_cmd       #PHB                    ;
               lda   SrcBlk            ;  get list data from source bank
               PHA                     ; 
               #PLB                    ;  set DBR for List
               lda   LineCnt           ;  Check for normal/range
               beq   List_cmd_1        ;  0 = normal  1=range 
               LDA   Addrptr           ;  Dissassemble range of code
               LDX   Addrptr+1         ;  move addrptr to startaddr
               STA   Startaddr         ;
               STX   Startaddr+1       ;
List_range     sec                     ;
               lda   Startaddr         ;
               sbc   Addrptr           ;
               lda   Startaddr+1       ; 
               sbc   Addrptr+1         ;
               bcc   List_done         ;
               jsr   List_Line         ;  list one line
               jsr   Scan_Input        ; see if brk requested
               bcs   List_done         ; if so, stop List
               SEC                     ;  checks if range done
               LDA   Hexdigits         ;
               SBC   Startaddr         ;
               LDA   Hexdigits+1       ;
               SBC   Startaddr+1       ;
               BCS   List_range        ;  if not, loop until done
               stz   LineCnt           ;
List_done      #PLB                    ; restore DBR 
               stz   IMM_mode+1        ; 
               RTS                     ;
List_cmd_1     LDA   Hexdigcnt         ; Dissassemble one page of cmds
               BEQ   List1             ; followed with more pages 
               LDX   Hexdigits         ;
               LDA   Hexdigits+1       ;
               STX   Startaddr         ;
               STA   Startaddr+1       ;
List1          LDA   #$14              ; one page of dissassembly
               STA   Linecnt           ;
List2          JSR   List_line         ;
               DEC   Linecnt           ;
               BNE   List2             ;
               bra   List_done         ;
List_line      JSR   Print_CR          ; 
               JSR   List_one          ; one line of dissassembly
               CLC                     ;
               LDA   Startaddr         ; inc address pointer to next cmd
               ADC   Hexdigcnt         ;
               STA   Startaddr         ;
               BCC   List3             ;
               INC   Startaddr+1       ;
List3          RTS                     ;
List_one       LDA   (Startaddr)       ; Dissassemble one CMD from Startaddr
               TAX                     ; Initialize List Cmd pointers
               #LDA_ALX OPCtxtidx, $00 ; 
               STA   OPCtxtptr         ;
               #LDA_ALX OPCaddmode, $00  ; 
               STA   Modejmp           ;
               LSR                     ;
               TAX                     ;
               #LDA_ALX ModeByteCnt, $00  ; 
               STA   Hexdigcnt         ;
               lda   #$24              ; Imm Mode
               cmp   Modejmp           ;
               bne   List35            ;
               lda   IMM_mode+1        ;
               beq   List35            ;
               INC   Hexdigcnt         ; up parms from 2 to three for Long IMM
List35         LDA   Startaddr+1       ;
               LDX   Startaddr         ;
               JSR   Print2Byte        ; print address 
               JSR   Print1SP          ; print " "
               LDA   #$2D              ;  "-"
               JSR   OUTPUT            ;
               JSR   Print1SP          ; print " "
               LDX   #$01              ;---------
List4          LDY   #$00              ;print up to 4 ascii chars...
List5          CPY   Hexdigcnt         ;  two spaces...
               BCS   List6             ;  up to three hex chars...
               LDA   (Startaddr),Y     ;  two spaces
               CPX   #$00              ;
               BNE   List8             ;
               JSR   Print1Byte        ;
               BRA   List7             ;
List6          CPX   #$00              ;
               BNE   List7             ;
               JSR   Print2SP          ;
List7          LDA   #" "              ; 
List8          AND   #$7F              ;
               CMP   #$7F              ;
               BEQ   List9             ;
               CMP   #" "              ;
               BCS   List10            ;
List9          LDA   #"."              ; use "." if not printable char
List10         JSR   OUTPUT            ;
               INY                     ;
               CPY   #$04              ; up to 4 ascii chars
               BCC   List5             ;
               JSR   Print1SP          ;
               DEX                     ;
               BEQ   List4             ;---------
               LDA   OPCtxtptr         ; get opcode text
               ASL                     ;
               TAX                     ;
List11         #LDA_ALX PKOPCTXT, $00  ; Print 3 chr opcode from two byte packed data 
               tay                     ; save for chr 2  
               and   #$1F              ; mask this chr
               ora   #$40              ; make ascii               
               JSR   OUTPUT            ; print chr 1
               inx                     ; get next byte
               #LDA_ALX PKOPCTXT, $00  ; 
               and   #$07              ; get 3 bits for chr 2
               sta   OPCtxtptr         ; save
               tya                     ;
               asl                     ; get bit 7
               rol   OPCtxtptr         ; move to chr 2
               asl                     ; get bit 6
               rol   OPCtxtptr         ; move to chr 2
               lda   OPCtxtptr         ; get chr 2
               ora   #$40              ; make ascii
               JSR   OUTPUT            ; print chr 2
               #LDA_ALX PKOPCTXT, $00  ; get chr 3 
               lsr                     ; shift right into place
               lsr                     ;
               lsr                     ;
               ora   #$40              ; make ascii
               JSR   OUTPUT            ; print chr 3
               lda   #$20              ; " "
               jsr   Output            ; print space
               bra   List13            ;
List12         JSR   Print2SP          ;
List13         LDA   Modejmp           ; setup to print operand
               TAX                     ;
               JMP   (ModeJmpTbl,X)    ; goto operand printing command 
;
; Address mode print routines for List and Xlist
;
INDSR_Y_mode   jsr   IND0_mode         ; (
               jsr   DP_mode           ; $HH
               jsr   STACK_mode        ; ,s
               jsr   IND1_mode         ; )
               bra   Y_mode            ; ,y = ($HH,s),y
STABS_mode     LDA   #"#"              ; print #$HH
               JSR   output            ;
               bra   ABS_mode          ;
IMM_mode       bra   SIRQ_mode         ; self modifying code for 16 bit immediates
SIRQ_mode      LDA   #"#"              ; print #$HH
               JSR   output            ;
DP_mode        LDA   #"$"              ; print $HH
               JSR   output            ;
               LDY   #$01              ;
Byte_mode      LDA   (Startaddr),Y     ;
               JMP   Print1Byte        ;
DP_X_mode      JSR   DP_Mode           ; print $HH,X
X_mode         LDA   #","              ; print ,X
               JSR   output            ;
               LDA   #"X"              ; 
               JMP   output            ;
DP_Y_mode      JSR   DP_Mode           ; print $HH,Y
Y_mode         LDA   #","              ; Print ,Y
               JSR   output            ;
               LDA   #"Y"              ; 
               JMP   output            ;
SIDP_mode                              ;
INDDP_mode     JSR   IND0_mode         ; Print ($HH)
               JSR   DP_Mode           ;
IND1_mode      LDA   #")"              ; Print )
               JMP   output            ;
INDDPL_mode    JSR   INDL0_mode        ; Print [$HH]
               JSR   DP_Mode           ;
INDL1_mode     LDA   #"]"              ; Print ]
               JMP   output            ;
INDDP_X_mode   JSR   IND0_mode         ; Print ($HH,X)
               JSR   DP_Mode           ;
               JSR   X_mode            ;
               BRA   IND1_mode         ;
INDDP_Y_mode   JSR   INDDP_Mode        ; Print ($HH),Y
               BRA   Y_mode            ;
INDDPL_Y_mode  JSR   INDDPL_Mode       ; Print [$HH],Y
               BRA   Y_mode            ;
IMML_mode      LDA   #"#"              ; print #$HHHH
               JSR   output            ;
ABS_mode       LDA   #"$"              ; Print $HHHH
               JSR   output            ;
ABS1_mode      LDY   #$02              ; 2 bytes
               JSR   Byte_mode         ;
               DEY                     ;
               BRA   Byte_mode         ;
ABSL_mode      LDA   #"$"              ; Print $HHHHHH
               JSR   output            ;
               LDY   #$03              ; 3 bytes
               JSR   Byte_mode         ;
               BRA   ABS1_mode         ;
ABSL_X_mode    JSR   ABSL_mode         ; Print $HHHHHH,X
               BRA   X_mode            ;
ABS_X_mode     JSR   ABS_mode          ; Print $HHHH,X
               BRA   X_mode            ;
ABS_Y_mode     JSR   ABS_mode          ; Print $HHHH,Y
               BRA   Y_mode            ;
INDABS_mode    JSR   IND0_mode         ; Print ($HHHH)
               JSR   ABS_mode          ;
               BRA   IND1_mode         ;
INDABSX_mode   JSR   IND0_mode         ; Print ($HHHH,X)
               JSR   ABS_mode          ;
               JSR   X_mode            ;
               BRA   IND1_mode         ;
INDABSL_mode   JSR   IND0_mode         ; Print ($HHHH,X)
               LDA   #"$"              ; print $HH
               JSR   output            ;
               LDY   #$03              ;
               JSR   Byte_mode         ;
               JSR   ABS1_mode         ;
               BRA   IND1_mode         ;
IMPLIED_mode   RTS                     ; Implied/Accumulator mode 
IND0_mode      LDA   #"("              ; Print (
               JMP   output            ;
INDL0_mode     LDA   #"["              ; Print [
               JMP   output            ;
REL_mode       LDA   #"$"              ; Print $HHHH as Relative Branch
               JSR   output            ;
               LDX   #$00              ;
               LDY   #$01              ;
               LDA   (Startaddr),Y     ;
               STA   Memptr            ;
               bpl   REL1_mode         ;
               ldx   #$FF              ; 
REL1_mode      STX   memptr+1          ; 
REL2_mode      #REP  $30               ;
               #LDAI16 $0002           ;
               clc                     ;
               adc   Startaddr         ;
               adc   memptr            ;
               #SEP  $30               ;
               tax                     ;
               #XBA                    ;
               JMP   Print2Byte        ;
RELL_mode                              ;
SRELL_mode     LDA   #"$"              ; Print $HHHH as Relative Branch
               JSR   output            ;
               LDY   #$01              ;
               LDA   (Startaddr),Y     ;
               sta   memptr            ;
               iny                     ;
               LDA   (Startaddr),Y     ;
               sta   memptr+1          ;
               bra   REL2_mode         ; 
BLOCK_mode     jsr   DP_mode           ;
               LDA   #","              ;
               jsr   output            ;
               LDA   #"$"              ; Print $
               JSR   output            ;
               ldy   #$02              ;
               jmp   Byte_mode         ;
ACCUM_mode     LDA   #"A"              ; Print A
               JMP   output            ;
SR_mode        jsr   DP_mode           ; $HH
STACK_mode     LDA   #","              ; Print ,s
               JSR   output            ;
               LDA   #"s"              ; Print ,s
               JMP   output            ;
;
;-----------DATA TABLES ------------------------------------------------
;     
; MONOTOR COMMANDS
;
CmdAscii       .byte $0D               ;  enter    cmd codes
               .byte $20               ;  SPACE
               .byte $2E               ;  .
               .byte $3A               ;  :
               .byte $3E               ;  >  
               .byte $3f               ;  ? - Help
               .byte $47               ;  g - Go
               .byte $49               ;  i - Insert
               .byte $4C               ;  l - List
               .byte $4D               ;  m - Move
               .byte $4F               ;  o - Set port options
               .byte $50               ;  p - Put File
               .byte $52               ;  r - Registers
               .byte $53               ;  s - Source block
               .byte $54               ;  t - Destination block 
               .byte $55               ;  U - Uploader
               .byte $56               ;  v - Version
               .byte $58               ;  x - Long immediates in List 
;
; Jump table for Monitor commands
;    
Cmdjmptbl      .word CR_cmd            ;   enter   cmd jmp table
               .word SP_cmd            ;    space
               .word DOT_cmd           ;     .
               .word COLN_cmd          ;     :
               .word Dest_cmd          ;     >  
               .word Help_Cmd          ;     ?
               .word Excute_cmd        ;     g
               .word Insert_Cmd        ;     i
               .word LIST_cmd          ;     l
               .word Move_cmd          ;     m
               .word serparm           ;     o
               .word PutFile           ;     p
               .word Printreg          ;     r
               .word SrcB_cmd          ;     s
               .word DestB_cmd         ;     t
               .word Upload            ;     u
               .word Version           ;     v
               .word X_cmd             ;     x
;
; 
CmdCount       =  Cmdjmptbl- CmdAscii  ; number of commands to scan for
;
; opcode pointer table - points to assembly language command text
;
OPCtxtidx      .byte $0B               ; $00  BRK  Stack/Interrupt
               .byte $29               ; $01  ORA  DP Indexed Indirect X
               .byte $14               ; $02  COP  Stack/Interrupt
               .byte $29               ; $03  ORA  Stack Relative
               .byte $50               ; $04  TSB  Direct Page
               .byte $29               ; $05  ORA  Direct Page
               .byte $02               ; $06  ASL  Direct Page
               .byte $29               ; $07  ORA  DP Indirect Long
               .byte $31               ; $08  PHP  Stack (Push)
               .byte $29               ; $09  ORA  Immediate
               .byte $02               ; $0A  ASL  Accumulator
               .byte $2F               ; $0B  PHD  Stack (Push)
               .byte $50               ; $0C  TSB  Absolute
               .byte $29               ; $0D  ORA  Absolute
               .byte $02               ; $0E  ASL  Absolute
               .byte $29               ; $0F  ORA  Absolute Long
               .byte $09               ; $10  BPL  Program Counter Relative
               .byte $29               ; $11  ORA  DP Indirect Indexed Y
               .byte $29               ; $12  ORA  DP Indirect
               .byte $29               ; $13  ORA  SR Indirect Indexed Y
               .byte $4F               ; $14  TRB  Direct Page
               .byte $29               ; $15  ORA  DP Indexed X
               .byte $02               ; $16  ASL  DP Indexed X
               .byte $29               ; $17  ORA  DP Indirect Long Indexed Y
               .byte $0F               ; $18  CLC  Implied
               .byte $29               ; $19  ORA  Absolute Indexed Y
               .byte $1B               ; $1A  INC  Accumulator
               .byte $4D               ; $1B  TCS  Implied
               .byte $4F               ; $1C  TRB  Absolute
               .byte $29               ; $1D  ORA  Absolute Indexed X
               .byte $02               ; $1E  ASL  Absolute Indexed X
               .byte $29               ; $1F  ORA  Absolute Long Indexed X
               .byte $21               ; $20  JSR  Absolute
               .byte $01               ; $21  AND  DP Indexed Indirect X
               .byte $21               ; $22  JSR  Absolute Long
               .byte $01               ; $23  AND  Stack Relative
               .byte $06               ; $24  BIT  Direct Page
               .byte $01               ; $25  AND  Direct Page
               .byte $3B               ; $26  ROL  Direct Page
               .byte $01               ; $27  AND  DP Indirect Long
               .byte $37               ; $28  PLP  Stack (Pull)
               .byte $01               ; $29  AND  Immediate
               .byte $3B               ; $2A  ROL  Accumulator
               .byte $36               ; $2B  PLD  Stack (Pull)
               .byte $06               ; $2C  BIT  Absolute
               .byte $01               ; $2D  AND  Absolute
               .byte $3B               ; $2E  ROL  Absolute
               .byte $01               ; $2F  AND  Absolute Long
               .byte $07               ; $30  BMI  Program Counter Relative
               .byte $01               ; $31  AND  DP Indirect Indexed Y
               .byte $01               ; $32  AND  DP Indirect
               .byte $01               ; $33  AND  SR Indirect Indexed Y
               .byte $06               ; $34  BIT  DP Indexed X
               .byte $01               ; $35  AND  DP Indexed X
               .byte $3B               ; $36  ROL  DP Indexed X
               .byte $01               ; $37  AND  DP Indirect Long Indexed Y
               .byte $41               ; $38  SEC  Implied
               .byte $01               ; $39  AND  Absolute Indexed Y
               .byte $17               ; $3A  DEC  Accumulator
               .byte $51               ; $3B  TSC  Implied
               .byte $06               ; $3C  BIT  Absolute Indexed X
               .byte $01               ; $3D  AND  Absolute Indexed X
               .byte $3B               ; $3E  ROL  Absolute Indexed X
               .byte $01               ; $3F  AND  Absolute Long Indexed X
               .byte $3D               ; $40  RTI  Stack/RTI
               .byte $1A               ; $41  EOR  DP Indexed Indirect X
               .byte $59               ; $42  WDM  implied
               .byte $1A               ; $43  EOR  Stack Relative
               .byte $27               ; $44  MVP  Block Move
               .byte $1A               ; $45  EOR  Direct Page
               .byte $25               ; $46  LSR  Direct Page
               .byte $1A               ; $47  EOR  DP Indirect Long
               .byte $2D               ; $48  PHA  Stack (Push)
               .byte $1A               ; $49  EOR  Immediate
               .byte $25               ; $4A  LSR  Accumulator
               .byte $30               ; $4B  PHK  Stack (Push)
               .byte $1F               ; $4C  JMP  Absolute
               .byte $1A               ; $4D  EOR  Absolute
               .byte $25               ; $4E  LSR  Absolute
               .byte $1A               ; $4F  EOR  Absolute Long
               .byte $0D               ; $50  BVC  Program Counter Relative
               .byte $1A               ; $51  EOR  DP Indirect Indexed Y
               .byte $1A               ; $52  EOR  DP Indirect
               .byte $1A               ; $53  EOR  SR Indirect Indexed Y
               .byte $26               ; $54  MVN  Block Move
               .byte $1A               ; $55  EOR  DP Indexed X
               .byte $25               ; $56  LSR  DP Indexed X
               .byte $1A               ; $57  EOR  DP Indirect Long Indexed Y
               .byte $11               ; $58  CLI  Implied
               .byte $1A               ; $59  EOR  Absolute Indexed Y
               .byte $33               ; $5A  PHY  Stack (Push)
               .byte $4C               ; $5B  TCD  Implied
               .byte $1F               ; $5C  JMP  Absolute Long
               .byte $1A               ; $5D  EOR  Absolute Indexed X
               .byte $25               ; $5E  LSR  Absolute Indexed X
               .byte $1A               ; $5F  EOR  Absolute Long Indexed X
               .byte $3F               ; $60  RTS  Stack (RTS)
               .byte $00               ; $61  ADC  DP Indexed Indirect X
               .byte $2C               ; $62  PER  Stack (PC Relative Long)
               .byte $00               ; $63  ADC  Stack Relative
               .byte $49               ; $64  STZ  Direct Page
               .byte $00               ; $65  ADC  Direct Page
               .byte $3C               ; $66  ROR  Direct Page
               .byte $00               ; $67  ADC  DP Indirect Long
               .byte $34               ; $68  PLA  Stack (Pull)
               .byte $00               ; $69  ADC  Immediate
               .byte $3C               ; $6A  ROR  Accumulator
               .byte $3E               ; $6B  RTL  Stack (RTL)
               .byte $1F               ; $6C  JMP  Absolute Indirect
               .byte $00               ; $6D  ADC  Absolute
               .byte $3C               ; $6E  ROR  Absolute
               .byte $00               ; $6F  ADC  Absolute Long
               .byte $0E               ; $70  BVS  Program Counter Relative
               .byte $00               ; $71  ADC  DP Indirect Indexed Y
               .byte $00               ; $72  ADC  DP Indirect
               .byte $00               ; $73  ADC  SR Indirect Indexed Y
               .byte $49               ; $74  STZ  DP Indexed X
               .byte $00               ; $75  ADC  DP Indexed X
               .byte $3C               ; $76  ROR  DP Indexed X
               .byte $00               ; $77  ADC  DP Indirect Long Indexed Y
               .byte $43               ; $78  SEI  Implied
               .byte $00               ; $79  ADC  Absolute Indexed Y
               .byte $39               ; $7A  PLY  Stack/Pull
               .byte $4E               ; $7B  TDC  Implied
               .byte $1F               ; $7C  JMP  Absolute Indexed Indirect
               .byte $00               ; $7D  ADC  Absolute Indexed X
               .byte $3C               ; $7E  ROR  Absolute Indexed X
               .byte $00               ; $7F  ADC  Absolute Long Indexed X
               .byte $0A               ; $80  BRA  Program Counter Relative
               .byte $45               ; $81  STA  DP Indexed Indirect X
               .byte $0C               ; $82  BRL  Program Counter Relative Long
               .byte $45               ; $83  STA  Stack Relative
               .byte $48               ; $84  STY  Direct Page
               .byte $45               ; $85  STA  Direct Page
               .byte $47               ; $86  STX  Direct Page
               .byte $45               ; $87  STA  DP Indirect Long
               .byte $19               ; $88  DEY  Implied
               .byte $06               ; $89  BIT  Immediate
               .byte $53               ; $8A  TXA  Implied
               .byte $2E               ; $8B  PHB  Stack (Push)
               .byte $48               ; $8C  STY  Absolute
               .byte $45               ; $8D  STA  Absolute
               .byte $47               ; $8E  STX  Absolute
               .byte $45               ; $8F  STA  Absolute Long
               .byte $03               ; $90  BCC  Program Counter Relative
               .byte $45               ; $91  STA  DP Indirect Indexed Y
               .byte $45               ; $92  STA  DP Indirect
               .byte $45               ; $93  STA  SR Indirect Indexed Y
               .byte $48               ; $94  STY  DP Indexed X
               .byte $45               ; $95  STA  DP Indexed X
               .byte $47               ; $96  STX  DP Indexed Y
               .byte $45               ; $97  STA  DP Indirect Long Indexed Y
               .byte $56               ; $98  TYA  Implied
               .byte $45               ; $99  STA  Absolute Indexed Y
               .byte $54               ; $9A  TXS  Implied
               .byte $55               ; $9B  TXY  Implied
               .byte $49               ; $9C  STZ  Absolute
               .byte $45               ; $9D  STA  Absolute Indexed X
               .byte $49               ; $9E  STZ  Absolute Indexed X
               .byte $45               ; $9F  STA  Absolute Long Indexed X
               .byte $24               ; $A0  LDY  Immediate
               .byte $22               ; $A1  LDA  DP Indexed Indirect X
               .byte $23               ; $A2  LDX  Immediate
               .byte $22               ; $A3  LDA  Stack Relative
               .byte $24               ; $A4  LDY  Direct Page
               .byte $22               ; $A5  LDA  Direct Page
               .byte $23               ; $A6  LDX  Direct Page
               .byte $22               ; $A7  LDA  DP Indirect Long
               .byte $4B               ; $A8  TAY  Implied
               .byte $22               ; $A9  LDA  Immediate
               .byte $4A               ; $AA  TAX  Implied
               .byte $35               ; $AB  PLB  Stack (Pull)
               .byte $24               ; $AC  LDY  Absolute
               .byte $22               ; $AD  LDA  Absolute
               .byte $23               ; $AE  LDX  Absolute
               .byte $22               ; $AF  LDA  Absolute Long
               .byte $04               ; $B0  BCS  Program Counter Relative
               .byte $22               ; $B1  LDA  DP Indirect Indexed Y
               .byte $22               ; $B2  LDA  DP Indirect
               .byte $22               ; $B3  LDA  SR Indirect Indexed Y
               .byte $24               ; $B4  LDY  DP Indexed X
               .byte $22               ; $B5  LDA  DP Indexed X
               .byte $23               ; $B6  LDX  DP Indexed Y
               .byte $22               ; $B7  LDA  DP Indirect Long Indexed Y
               .byte $12               ; $B8  CLV  Implied
               .byte $22               ; $B9  LDA  Absolute Indexed Y
               .byte $52               ; $BA  TSX  Implied
               .byte $57               ; $BB  TYX  Implied
               .byte $24               ; $BC  LDY  Absolute Indexed X
               .byte $22               ; $BD  LDA  Absolute Indexed X
               .byte $23               ; $BE  LDX  Absolute Indexed Y
               .byte $22               ; $BF  LDA  Absolute Long Indexed X
               .byte $16               ; $C0  CPY  Immediate
               .byte $13               ; $C1  CMP  DP Indexed Indirect X
               .byte $3A               ; $C2  REP  Immediate
               .byte $13               ; $C3  CMP  Stack Relative
               .byte $16               ; $C4  CPY  Direct Page
               .byte $13               ; $C5  CMP  Direct Page
               .byte $17               ; $C6  DEC  Direct Page
               .byte $13               ; $C7  CMP  DP Indirect Long
               .byte $1D               ; $C8  INY  Implied
               .byte $13               ; $C9  CMP  Immediate
               .byte $18               ; $CA  DEX  Implied
               .byte $58               ; $CB  WAI  Implied
               .byte $16               ; $CC  CPY  Absolute
               .byte $13               ; $CD  CMP  Absolute
               .byte $17               ; $CE  DEC  Absolute
               .byte $13               ; $CF  CMP  Absolute Long
               .byte $08               ; $D0  BNE  Program Counter Relative
               .byte $13               ; $D1  CMP  DP Indirect Indexed Y
               .byte $13               ; $D2  CMP  DP Indirect
               .byte $13               ; $D3  CMP  SR Indirect Indexed Y
               .byte $2B               ; $D4  PEI  Stack (Direct Page Indirect)
               .byte $13               ; $D5  CMP  DP Indexed X
               .byte $17               ; $D6  DEC  DP Indexed X
               .byte $13               ; $D7  CMP  DP Indirect Long Indexed Y
               .byte $10               ; $D8  CLD  Implied
               .byte $13               ; $D9  CMP  Absolute Indexed Y
               .byte $32               ; $DA  PHX  Stack (Push)
               .byte $46               ; $DB  STP  Implied
               .byte $1F               ; $DC  JMP  Absolute Indirect Long
               .byte $13               ; $DD  CMP  Absolute Indexed X
               .byte $17               ; $DE  DEC  Absolute Indexed X
               .byte $13               ; $DF  CMP  Absolute Long Indexed X
               .byte $15               ; $E0  CPX  Immediate
               .byte $40               ; $E1  SBC  DP Indexed Indirect X
               .byte $44               ; $E2  SEP  Immediate
               .byte $40               ; $E3  SBC  Stack Relative
               .byte $1C               ; $E4  INX  Direct Page
               .byte $40               ; $E5  SBC  Direct Page
               .byte $1B               ; $E6  INC  Direct Page
               .byte $40               ; $E7  SBC  DP Indirect Long
               .byte $1C               ; $E8  INX  Implied
               .byte $40               ; $E9  SBC  Immediate
               .byte $28               ; $EA  NOP  Implied
               .byte $5A               ; $EB  XBA  Implied
               .byte $15               ; $EC  CPX  Absolute
               .byte $40               ; $ED  SBC  Absolute
               .byte $1B               ; $EE  INC  Absolute
               .byte $40               ; $EF  SBC  Absolute Long
               .byte $05               ; $F0  BEQ  Program Counter Relative
               .byte $40               ; $F1  SBC  DP Indirect Indexed Y
               .byte $40               ; $F2  SBC  DP Indirect
               .byte $40               ; $F3  SBC  SR Indirect Indexed Y
               .byte $2A               ; $F4  PEA  Stack (absolute)
               .byte $40               ; $F5  SBC  DP Indexed X
               .byte $1B               ; $F6  INC  DP Indexed X
               .byte $40               ; $F7  SBC  DP Indirect Long Indexed Y
               .byte $42               ; $F8  SED  Implied
               .byte $40               ; $F9  SBC  Absolute Indexed Y
               .byte $38               ; $FA  PLX  Stack/Pull
               .byte $5B               ; $FB  XCE  Implied
               .byte $21               ; $FC  JSR  Absolute Indexed Indirect
               .byte $40               ; $FD  SBC  Absolute Indexed X
               .byte $1B               ; $FE  INC  Absolute Indexed X
               .byte $40               ; $FF  SBC  Absolute Long Indexed X
;
; Compressed Assembly language Opcode Text
;
PKOPCTXT       .word $1901             ; ADC   2 byte packed data table
               .word $2381             ; AND   16 bit format is  3333 3222 2201 1111
               .word $64C1             ; ASL   where A=1 and Z=26
               .word $18C2             ; BCC
               .word $98C2             ; BCS
               .word $8942             ; BEQ
               .word $A242             ; BIT
               .word $4B42             ; BMI
               .word $2B82             ; BNE
               .word $6402             ; BPL
               .word $0C82             ; BRA
               .word $5C82             ; BRK
               .word $6482             ; BRL
               .word $1D82             ; BVC
               .word $9D82             ; BVS
               .word $1B03             ; CLC
               .word $2303             ; CLD
               .word $4B03             ; CLI
               .word $B303             ; CLV
               .word $8343             ; CMP
               .word $83C3             ; COP
               .word $C403             ; CPX
               .word $CC03             ; CPY
               .word $1944             ; DEC
               .word $C144             ; DEX
               .word $C944             ; DEY
               .word $93C5             ; EOR
               .word $1B89             ; INC
               .word $C389             ; INX
               .word $CB89             ; INY
               .word $634A             ; JML
               .word $834A             ; JMP
               .word $64CA             ; JSL
               .word $94CA             ; JSR
               .word $090C             ; LDA
               .word $C10C             ; LDX
               .word $C90C             ; LDY
               .word $94CC             ; LSR
               .word $758D             ; MVN
               .word $858D             ; MVP
               .word $83CE             ; NOP
               .word $0C8F             ; ORA
               .word $0950             ; PEA
               .word $4950             ; PEI
               .word $9150             ; PER
               .word $0A10             ; PHA
               .word $1210             ; PHB
               .word $2210             ; PHD
               .word $5A10             ; PHK
               .word $8210             ; PHP
               .word $C210             ; PHX
               .word $CA10             ; PHY
               .word $0B10             ; PLA
               .word $1310             ; PLB
               .word $2310             ; PLD
               .word $8310             ; PLP
               .word $C310             ; PLX
               .word $CB10             ; PLY
               .word $8152             ; REP
               .word $63D2             ; ROL
               .word $93D2             ; ROR
               .word $4D12             ; RTI
               .word $6512             ; RTL
               .word $9D12             ; RTS
               .word $1893             ; SBC
               .word $1953             ; SEC
               .word $2153             ; SED
               .word $4953             ; SEI
               .word $8153             ; SEP
               .word $0D13             ; STA
               .word $8513             ; STP
               .word $C513             ; STX
               .word $CD13             ; STY
               .word $D513             ; STZ
               .word $C054             ; TAX
               .word $C854             ; TAY
               .word $20D4             ; TCD
               .word $98D4             ; TCS
               .word $1914             ; TDC
               .word $1494             ; TRB
               .word $14D4             ; TSB
               .word $1CD4             ; TSC
               .word $C4D4             ; TSX
               .word $0E14             ; TXA
               .word $9E14             ; TXS
               .word $CE14             ; TXY
               .word $0E54             ; TYA
               .word $C654             ; TYX
               .word $4857             ; WAI
               .word $6917             ; WDM
               .word $0898             ; XBA
               .word $28D8             ; XCE
;
; Opcode addressing mode table
;
OPCaddmode     .byte $34               ; $00  BRK  Stack/Interrupt
               .byte $16               ; $01  ORA  DP Indexed Indirect X
               .byte $34               ; $02  COP  Stack/Interrupt
               .byte $32               ; $03  ORA  Stack Relative
               .byte $14               ; $04  TSB  Direct Page
               .byte $14               ; $05  ORA  Direct Page
               .byte $14               ; $06  ASL  Direct Page
               .byte $20               ; $07  ORA  DP Indirect Long
               .byte $00               ; $08  PHP  Stack (Push)
               .byte $24               ; $09  ORA  Immediate
               .byte $36               ; $0A  ASL  Accumulator
               .byte $00               ; $0B  PHD  Stack (Push)
               .byte $02               ; $0C  TSB  Absolute
               .byte $02               ; $0D  ORA  Absolute
               .byte $02               ; $0E  ASL  Absolute
               .byte $0E               ; $0F  ORA  Absolute Long
               .byte $26               ; $10  BPL  Program Counter Relative
               .byte $1E               ; $11  ORA  DP Indirect Indexed Y
               .byte $1C               ; $12  ORA  DP Indirect
               .byte $2A               ; $13  ORA  SR Indirect Indexed Y
               .byte $14               ; $14  TRB  Direct Page
               .byte $18               ; $15  ORA  DP Indexed X
               .byte $18               ; $16  ASL  DP Indexed X
               .byte $22               ; $17  ORA  DP Indirect Long Indexed Y
               .byte $00               ; $18  CLC  Implied
               .byte $08               ; $19  ORA  Absolute Indexed Y
               .byte $36               ; $1A  INC  Accumulator
               .byte $00               ; $1B  TCS  Implied
               .byte $02               ; $1C  TRB  Absolute
               .byte $06               ; $1D  ORA  Absolute Indexed X
               .byte $06               ; $1E  ASL  Absolute Indexed X
               .byte $10               ; $1F  ORA  Absolute Long Indexed X
               .byte $02               ; $20  JSR  Absolute
               .byte $16               ; $21  AND  DP Indexed Indirect X
               .byte $0E               ; $22  JSR  Absolute Long
               .byte $32               ; $23  AND  Stack Relative
               .byte $14               ; $24  BIT  Direct Page
               .byte $14               ; $25  AND  Direct Page
               .byte $14               ; $26  ROL  Direct Page
               .byte $20               ; $27  AND  DP Indirect Long
               .byte $00               ; $28  PLP  Stack (Pull)
               .byte $24               ; $29  AND  Immediate
               .byte $36               ; $2A  ROL  Accumulator
               .byte $00               ; $2B  PLD  Stack (Pull)
               .byte $02               ; $2C  BIT  Absolute
               .byte $02               ; $2D  AND  Absolute
               .byte $02               ; $2E  ROL  Absolute
               .byte $0E               ; $2F  AND  Absolute Long
               .byte $26               ; $30  BMI  Program Counter Relative
               .byte $1E               ; $31  AND  DP Indirect Indexed Y
               .byte $1C               ; $32  AND  DP Indirect
               .byte $2A               ; $33  AND  SR Indirect Indexed Y
               .byte $18               ; $34  BIT  DP Indexed X
               .byte $18               ; $35  AND  DP Indexed X
               .byte $18               ; $36  ROL  DP Indexed X
               .byte $22               ; $37  AND  DP Indirect Long Indexed Y
               .byte $00               ; $38  SEC  Implied
               .byte $08               ; $39  AND  Absolute Indexed Y
               .byte $36               ; $3A  DEC  Accumulator
               .byte $00               ; $3B  TSC  Implied
               .byte $06               ; $3C  BIT  Absolute Indexed X
               .byte $06               ; $3D  AND  Absolute Indexed X
               .byte $06               ; $3E  ROL  Absolute Indexed X
               .byte $10               ; $3F  AND  Absolute Long Indexed X
               .byte $00               ; $40  RTI  Stack/RTI
               .byte $16               ; $41  EOR  DP Indexed Indirect X
               .byte $00               ; $42  WDM  implied
               .byte $32               ; $43  EOR  Stack Relative
               .byte $12               ; $44  MVP  Block Move
               .byte $14               ; $45  EOR  Direct Page
               .byte $14               ; $46  LSR  Direct Page
               .byte $20               ; $47  EOR  DP Indirect Long
               .byte $00               ; $48  PHA  Stack (Push)
               .byte $24               ; $49  EOR  Immediate
               .byte $36               ; $4A  LSR  Accumulator
               .byte $00               ; $4B  PHK  Stack (Push)
               .byte $02               ; $4C  JMP  Absolute
               .byte $02               ; $4D  EOR  Absolute
               .byte $02               ; $4E  LSR  Absolute
               .byte $0E               ; $4F  EOR  Absolute Long
               .byte $26               ; $50  BVC  Program Counter Relative
               .byte $1E               ; $51  EOR  DP Indirect Indexed Y
               .byte $1C               ; $52  EOR  DP Indirect
               .byte $2A               ; $53  EOR  SR Indirect Indexed Y
               .byte $12               ; $54  MVN  Block Move
               .byte $18               ; $55  EOR  DP Indexed X
               .byte $18               ; $56  LSR  DP Indexed X
               .byte $22               ; $57  EOR  DP Indirect Long Indexed Y
               .byte $00               ; $58  CLI  Implied
               .byte $08               ; $59  EOR  Absolute Indexed Y
               .byte $00               ; $5A  PHY  Stack (Push)
               .byte $00               ; $5B  TCD  Implied
               .byte $0E               ; $5C  JMP  Absolute Long
               .byte $06               ; $5D  EOR  Absolute Indexed X
               .byte $06               ; $5E  LSR  Absolute Indexed X
               .byte $10               ; $5F  EOR  Absolute Long Indexed X
               .byte $00               ; $60  RTS  Stack (RTS)
               .byte $16               ; $61  ADC  DP Indexed Indirect X
               .byte $30               ; $62  PER  Stack (PC Relative Long)
               .byte $32               ; $63  ADC  Stack Relative
               .byte $14               ; $64  STZ  Direct Page
               .byte $14               ; $65  ADC  Direct Page
               .byte $14               ; $66  ROR  Direct Page
               .byte $20               ; $67  ADC  DP Indirect Long
               .byte $00               ; $68  PLA  Stack (Pull)
               .byte $24               ; $69  ADC  Immediate
               .byte $36               ; $6A  ROR  Accumulator
               .byte $00               ; $6B  RTL  Stack (RTL)
               .byte $0A               ; $6C  JMP  Absolute Indirect
               .byte $02               ; $6D  ADC  Absolute
               .byte $02               ; $6E  ROR  Absolute
               .byte $0E               ; $6F  ADC  Absolute Long
               .byte $26               ; $70  BVS  Program Counter Relative
               .byte $1E               ; $71  ADC  DP Indirect Indexed Y
               .byte $1C               ; $72  ADC  DP Indirect
               .byte $2A               ; $73  ADC  SR Indirect Indexed Y
               .byte $18               ; $74  STZ  DP Indexed X
               .byte $18               ; $75  ADC  DP Indexed X
               .byte $18               ; $76  ROR  DP Indexed X
               .byte $22               ; $77  ADC  DP Indirect Long Indexed Y
               .byte $00               ; $78  SEI  Implied
               .byte $08               ; $79  ADC  Absolute Indexed Y
               .byte $00               ; $7A  PLY  Stack/Pull
               .byte $00               ; $7B  TDC  Implied
               .byte $04               ; $7C  JMP  Absolute Indexed Indirect
               .byte $06               ; $7D  ADC  Absolute Indexed X
               .byte $06               ; $7E  ROR  Absolute Indexed X
               .byte $10               ; $7F  ADC  Absolute Long Indexed X
               .byte $26               ; $80  BRA  Program Counter Relative
               .byte $16               ; $81  STA  DP Indexed Indirect X
               .byte $28               ; $82  BRL  Program Counter Relative Long
               .byte $32               ; $83  STA  Stack Relative
               .byte $14               ; $84  STY  Direct Page
               .byte $14               ; $85  STA  Direct Page
               .byte $14               ; $86  STX  Direct Page
               .byte $20               ; $87  STA  DP Indirect Long
               .byte $00               ; $88  DEY  Implied
               .byte $24               ; $89  BIT  Immediate
               .byte $00               ; $8A  TXA  Implied
               .byte $00               ; $8B  PHB  Stack (Push)
               .byte $02               ; $8C  STY  Absolute
               .byte $02               ; $8D  STA  Absolute
               .byte $02               ; $8E  STX  Absolute
               .byte $0E               ; $8F  STA  Absolute Long
               .byte $26               ; $90  BCC  Program Counter Relative
               .byte $1E               ; $91  STA  DP Indirect Indexed Y
               .byte $1C               ; $92  STA  DP Indirect
               .byte $2A               ; $93  STA  SR Indirect Indexed Y
               .byte $18               ; $94  STY  DP Indexed X
               .byte $18               ; $95  STA  DP Indexed X
               .byte $1A               ; $96  STX  DP Indexed Y
               .byte $22               ; $97  STA  DP Indirect Long Indexed Y
               .byte $00               ; $98  TYA  Implied
               .byte $08               ; $99  STA  Absolute Indexed Y
               .byte $00               ; $9A  TXS  Implied
               .byte $00               ; $9B  TXY  Implied
               .byte $02               ; $9C  STZ  Absolute
               .byte $06               ; $9D  STA  Absolute Indexed X
               .byte $06               ; $9E  STZ  Absolute Indexed X
               .byte $10               ; $9F  STA  Absolute Long Indexed X
               .byte $24               ; $A0  LDY  Immediate
               .byte $16               ; $A1  LDA  DP Indexed Indirect X
               .byte $24               ; $A2  LDX  Immediate
               .byte $32               ; $A3  LDA  Stack Relative
               .byte $14               ; $A4  LDY  Direct Page
               .byte $14               ; $A5  LDA  Direct Page
               .byte $14               ; $A6  LDX  Direct Page
               .byte $20               ; $A7  LDA  DP Indirect Long
               .byte $00               ; $A8  TAY  Implied
               .byte $24               ; $A9  LDA  Immediate
               .byte $00               ; $AA  TAX  Implied
               .byte $00               ; $AB  PLB  Stack (Pull)
               .byte $02               ; $AC  LDY  Absolute
               .byte $02               ; $AD  LDA  Absolute
               .byte $02               ; $AE  LDX  Absolute
               .byte $0E               ; $AF  LDA  Absolute Long
               .byte $26               ; $B0  BCS  Program Counter Relative
               .byte $1E               ; $B1  LDA  DP Indirect Indexed Y
               .byte $1C               ; $B2  LDA  DP Indirect
               .byte $2A               ; $B3  LDA  SR Indirect Indexed Y
               .byte $18               ; $B4  LDY  DP Indexed X
               .byte $18               ; $B5  LDA  DP Indexed X
               .byte $1A               ; $B6  LDX  DP Indexed Y
               .byte $22               ; $B7  LDA  DP Indirect Long Indexed Y
               .byte $00               ; $B8  CLV  Implied
               .byte $08               ; $B9  LDA  Absolute Indexed Y
               .byte $00               ; $BA  TSX  Implied
               .byte $00               ; $BB  TYX  Implied
               .byte $06               ; $BC  LDY  Absolute Indexed X
               .byte $06               ; $BD  LDA  Absolute Indexed X
               .byte $08               ; $BE  LDX  Absolute Indexed Y
               .byte $10               ; $BF  LDA  Absolute Long Indexed X
               .byte $24               ; $C0  CPY  Immediate
               .byte $16               ; $C1  CMP  DP Indexed Indirect X
               .byte $34               ; $C2  REP  Immediate (SIRQ)
               .byte $32               ; $C3  CMP  Stack Relative
               .byte $14               ; $C4  CPY  Direct Page
               .byte $14               ; $C5  CMP  Direct Page
               .byte $14               ; $C6  DEC  Direct Page
               .byte $20               ; $C7  CMP  DP Indirect Long
               .byte $00               ; $C8  INY  Implied
               .byte $24               ; $C9  CMP  Immediate
               .byte $00               ; $CA  DEX  Implied
               .byte $00               ; $CB  WAI  Implied
               .byte $02               ; $CC  CPY  Absolute
               .byte $02               ; $CD  CMP  Absolute
               .byte $02               ; $CE  DEC  Absolute
               .byte $0E               ; $CF  CMP  Absolute Long
               .byte $26               ; $D0  BNE  Program Counter Relative
               .byte $1E               ; $D1  CMP  DP Indirect Indexed Y
               .byte $1C               ; $D2  CMP  DP Indirect
               .byte $2A               ; $D3  CMP  SR Indirect Indexed Y
               .byte $2E               ; $D4  PEI  Stack (Direct Page Indirect)
               .byte $18               ; $D5  CMP  DP Indexed X
               .byte $18               ; $D6  DEC  DP Indexed X
               .byte $22               ; $D7  CMP  DP Indirect Long Indexed Y
               .byte $00               ; $D8  CLD  Implied
               .byte $08               ; $D9  CMP  Absolute Indexed Y
               .byte $00               ; $DA  PHX  Stack (Push)
               .byte $00               ; $DB  STP  Implied
               .byte $0C               ; $DC  JMP  Absolute Indirect Long
               .byte $06               ; $DD  CMP  Absolute Indexed X
               .byte $06               ; $DE  DEC  Absolute Indexed X
               .byte $10               ; $DF  CMP  Absolute Long Indexed X
               .byte $24               ; $E0  CPX  Immediate
               .byte $16               ; $E1  SBC  DP Indexed Indirect X
               .byte $34               ; $E2  SEP  Immediate (SIRQ)
               .byte $32               ; $E3  SBC  Stack Relative
               .byte $14               ; $E4  INX  Direct Page
               .byte $14               ; $E5  SBC  Direct Page
               .byte $14               ; $E6  INC  Direct Page
               .byte $20               ; $E7  SBC  DP Indirect Long
               .byte $00               ; $E8  INX  Implied
               .byte $24               ; $E9  SBC  Immediate
               .byte $00               ; $EA  NOP  Implied
               .byte $00               ; $EB  XBA  Implied
               .byte $02               ; $EC  CPX  Absolute
               .byte $02               ; $ED  SBC  Absolute
               .byte $02               ; $EE  INC  Absolute
               .byte $0E               ; $EF  SBC  Absolute Long
               .byte $26               ; $F0  BEQ  Program Counter Relative
               .byte $1E               ; $F1  SBC  DP Indirect Indexed Y
               .byte $1C               ; $F2  SBC  DP Indirect
               .byte $2A               ; $F3  SBC  SR Indirect Indexed Y
               .byte $2C               ; $F4  PEA  Stack (absolute)
               .byte $18               ; $F5  SBC  DP Indexed X
               .byte $18               ; $F6  INC  DP Indexed X
               .byte $22               ; $F7  SBC  DP Indirect Long Indexed Y
               .byte $00               ; $F8  SED  Implied
               .byte $08               ; $F9  SBC  Absolute Indexed Y
               .byte $00               ; $FA  PLX  Stack/Pull
               .byte $00               ; $FB  XCE  Implied
               .byte $04               ; $FC  JSR  Absolute Indexed Indirect
               .byte $06               ; $FD  SBC  Absolute Indexed X
               .byte $06               ; $FE  INC  Absolute Indexed X
               .byte $10               ; $FF  SBC  Absolute Long Indexed X
;     
; Address Mode Byte count     
;
ModeByteCnt    .byte $01               ; $00  Implied  
               .byte $03               ; $02  Absolute  HHHH
               .byte $03               ; $04  Absolute Indexed Indirect  (HHHH,X)
               .byte $03               ; $06  Absolute Indexed X  HHHH,X
               .byte $03               ; $08  Absolute Indexed Y  HHHH,Y
               .byte $03               ; $0A  Absolute Indirect  (HHHH)
               .byte $04               ; $0C  Absolute Indirect Long  (HHHHHH)
               .byte $04               ; $0E  Absolute Long  HHHHHH
               .byte $04               ; $10  Absolute Long Indexed X  HHHHHH,X
               .byte $03               ; $12  Block Move  HH,HH
               .byte $02               ; $14  Direct Page  HH
               .byte $02               ; $16  DP Indexed Indirect X  (HH,X)
               .byte $02               ; $18  DP Indexed X  HH,X
               .byte $02               ; $1A  DP Indexed Y  HH,Y
               .byte $02               ; $1C  DP Indirect  (HH)
               .byte $02               ; $1E  DP Indirect Indexed Y  (HH),Y
               .byte $02               ; $20  DP Indirect Long  [HH]
               .byte $02               ; $22  DP Indirect Long Indexed Y  [HH],y
               .byte $02               ; $24  Immediate  #HHhh
               .byte $02               ; $26  Program Counter Relative  HH
               .byte $03               ; $28  Program Counter Relative Long  HHHH
               .byte $02               ; $2A  SR Indirect Indexed Y  (S+HH),Y
               .byte $03               ; $2C  Stack (absolute)  HHHH
               .byte $02               ; $2E  Stack (Direct Page Indirect)  HH
               .byte $03               ; $30  Stack (PC Relative Long)  HHHH
               .byte $02               ; $32  Stack Relative  (HH)
               .byte $02               ; $34  Stack/Interrupt  HH
               .byte $01               ; $36  Accumulator  A
  
;
; Address mode jump table for List and Xlist
;
ModeJmpTbl     .word IMPLIED_mode      ; $00  Implied  
               .word ABS_mode          ; $02  Absolute  HHHH
               .word INDABSX_mode      ; $04  Absolute Indexed Indirect  (HHHH,X)
               .word ABS_X_mode        ; $06  Absolute Indexed X  HHHH,X
               .word ABS_Y_mode        ; $08  Absolute Indexed Y  HHHH,Y
               .word INDABS_mode       ; $0A  Absolute Indirect  (HHHH)
               .word INDABSL_mode      ; $0C  Absolute Indirect Long  (HHHHHH)
               .word ABSL_mode         ; $0E  Absolute Long  HHHHHH
               .word ABSL_X_mode       ; $10  Absolute Long Indexed X  HHHHHH,X
               .word BLOCK_mode        ; $12  Block Move  HH,HH
               .word DP_mode           ; $14  Direct Page  HH
               .word INDDP_X_mode      ; $16  DP Indexed Indirect X  (HH,X)
               .word DP_X_mode         ; $18  DP Indexed X  HH,X
               .word DP_Y_mode         ; $1A  DP Indexed Y  HH,Y
               .word INDDP_mode        ; $1C  DP Indirect  (HH)
               .word INDDP_Y_mode      ; $1E  DP Indirect Indexed Y  (HH),Y
               .word INDDPL_mode       ; $20  DP Indirect Long  [HH]
               .word INDDPL_Y_mode     ; $22  DP Indirect Long Indexed Y  [HH],y
               .word IMM_mode          ; $24  Immediate  #HHhh
               .word REL_mode          ; $26  Program Counter Relative  HH
               .word RELL_mode         ; $28  Program Counter Relative Long  HHHH
               .word INDSR_Y_mode      ; $2A  SR Indirect Indexed Y  (S+HH),Y
               .word STABS_mode        ; $2C  Stack (absolute)  HHHH
               .word SIDP_mode         ; $2E  Stack (Direct Page Indirect)  HH
               .word SRELL_mode        ; $30  Stack (PC Relative Long)  HHHH
               .word SR_mode           ; $32  Stack Relative  (HH)
               .word SIRQ_mode         ; $34  Stack/Interrupt  HH
               .word ACCUM_mode        ; $36  Accumulator  A
;
; Help screen
;
HelpTxt        .byte "~Current commands are :~"
               .byte "Syntax = {} required, [] optional,~"
               .byte "         HHHH hex address, DD hex data~"
               .byte "~"
               .byte "[HHHHHH][ HHHHHH]{Return}~"
               .byte "[HHHHHH]{.HHHH}{Return}~"
               .byte "[HHHHHH]{:DD}[ DD]{Return}~"
               .byte "[HHHHHH]{G}{Return}~"
               .byte "{HHHH.HHHH>HHHH}{I}{Return}~"
               .byte "[HHHHHH]{L}{Return}~"
               .byte "[HHHHHH]{.HHHH}{L}{Return}~"
               .byte "{HHHH.HHHH>HHHH}{M}{Return}~"
               .byte "{HH}{O}{Return}~"
               .byte "{HHHH.HHHH}{P}{Return}~"
               .byte "{R}{Return}~"
               .byte "{HH}{S}{Return}~"
               .byte "{HH}{T}{Return}~"
               .byte "[HH]{U}{Return}~"
               .byte "{V}{Return}~"
               .byte "[HHHHHH]{X}{Return}~"
               .byte "[HHHHHH]{.HHHH}{X}{Return}~"
               .byte "{?}{Return}~~"
               .byte $00
;
; Ready Prompt
;
Porttxt        .byte "65816 Monitor v6.2 (11-6-08) Ready"
               .byte $0d, $0a
               .byte "(Press ? for help)"
               .byte $00
;
; 6.1 Rewritten for 65816 native mode
; 6.2 replaced xmodem upload (u) with upload(u) AND put(p) for 2-way file transfer 
;
;end of file