;******************************************************************************
; NTSC/PAL Video Generation Code (16MHZ System Clock)
;
;  Designed and written by Daryl Rictor (c)2003-2004
;******************************************************************************

DROPSYNC:
	cbi		portc, syncpin			; drop the csync pin (timer = 12)

HOUSEKEEPING:        
	; hsync pulse is 73 clocks wide so do some line setup until timer = 12+73=85

	cpi		vlineh, 0x00			; Are we above line 255?
	breq	LINETST					; no, goto hc1 
	cp		vline, linetop			; yes: are we at the last line?
	brne	BLANKH					; no, wait for hsync end

VSYNC:
	ldi		vlineh, 0x00			; yes, reset vert counters
	ldi		vline, 0x00				;
	ldi     J, 0x03					;
	out     OCR1BH, J        		;
	out     OCR1BL, ocrbv			; (to end VSYNC pulse) 
	cpi		Cursclk, 0xff			; 
	breq	vs1						;
	inc		Cursclk					; inc cursor clock
vs1:
	rjmp	EOL						; line 0 is a VSYNC line, we're done

LINETST:	
	cpi		vline, 0x03				; no: are we in line 0-2 (VSYNC)
	brlo	vs1						; yes, leave CSYNC low and exit
	brne	lntst1					; 	
	ldi     J, 0x03					; for line 3, do this
	out     OCR1BH, J        		;
	out     OCR1BL, ocrba			; no end VSYNC needed
	rjmp	BLANK1					; 
LNTST1:
	cp		vline, line1			; no, are we below the first display line?
	brlo	BLANK					; yes, blank line, wait for hsync to end
	breq	FIRSTDISP				; are we on the first active line?
	cp		vline, line200			; are we above line 200?
	brlo	DISPLAY					; no, get ready for active display line
	nop								; nop used to equalize the timing loop
BLANKH:
	nop								; nop used to equalize the timing loop
BLANK:
	nop								; nop used to equalize the timing loop
BLANK1:
	ldi		chrln, 0xFF				; reset pointer
	nop								; nop used to equalize the timing loop
	rjmp	WAITSYNCEND				; 

FIRSTDISP:
	ldi		chrln, 0x00				; sync chrlin with first active line	
	nop								; nop used to equalize the timing loop	
DISPLAY:
	movw	svz, ZL					; save Z reg contents
	movw	svacc, Acc				; save Z reg contents
	; set the SRAM character pointer (X) to the start of the current line (chrln)
	ldi		XL, 0x28				; # chrs per line
	ldi		XH, 0x60				; offset to start of SRAM 
	mov		J, chrln				; Get current display line
	lsr		J						; divide by 8 
	lsr		J						; 
	lsr		J						; 
	mul		XL, J					; multiply by chrs per line
	ldi		J, 0x00					; 
	add		R0, XH					; add SRAM start offset to result
	adc		R1, J					; 
	movw	XL, R0					; mov it to X reg
	; set Z to point to base of font row
	ldi		ZL, 0x00				; ZL=0, ASCII code will be added for each character
	mov 	ZH, chrln				; ZH= current display line 
	andi	ZH, 0x07				; do ZH MOD 8
	ori		ZH, 0x10				; add $1000 for offset to start of font table
	or		ZH, fntsel				; adjust for primary/alt font

WAITSYNCEND:                        ;  else wait sync pulse end
	in      J, TCNT1L				;  
	cpi     J, 0x50	               	; is it @80 ticks
	brlo    WAITSYNCEND				; no, look again

HSYNCEND:        
	sbi		portc, syncpin			; raise sync (73 ticks wide hsync pulse)
	cpi		chrln, 0xFF				; are we on an active display line?
	brne	WAITVISIBLEPORTION		; yes, active display line
	rjmp	EOL						; no, blank line, we're done


WAITVISIBLEPORTION:        
;
; lots of time here for more house keeping.... (~75 clocks)
;

wvp:
	in      J, TCNT1L        		; wait for start of visible area
	in		K, TCNT1H				;
	cpi		K, 0x01
	brne	wvp		
	cp		J, leftside            	; (sets the left side of the display)
	brlo    wvp

BUILDSCREEN:
;
; each line consists of 40 macro calls, each 16 clocks long.  Here is the macro:
.macro DispChr
;
;  gets the font byte of the current character and outputs it to the shift register
;	
;  16 clocks

	ld 		ZL, X+				; 2   move curr chr in ZL
	lpm 	J, Z				; 3   get font byte for current chr on curr line
	mov		K, J				; 1   save to K
	andi	J, 0x3F				; 1   mask out output bits
	out		portb, J			; 1   save bits 0-5 to port B 
	ldi		J, 0x13				; 1   PORT C template
	swap	K					; 1   flip nibbles of font byte
	andi	K, 0x0C				; 1   mask out bits 2&3 (was 6&7)
	or		K, J				; 1   merge Port C template
	mov		J, K				; 1   copy it to J
	cbr		J, 0x02				; 1   mask out ldsrpin bit
	out		portc, J			; 1   sent to port C w/ldsr low
	out		portc, K			; 1   sent to port C w/ldsr high


.endmacro
;
; here are the 40 calls
;
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr
	DispChr							; display 1 chr

	movw	ZL, svz					; restore Z register
	movw	Acc, svacc				; restore Acc Register

EOL:								; end of line, we're done
	inc		chrln					; inc line counter
	inc		vline					; inc vertical line counter
	brne	eol1
	inc		vlineh					;

eol1:
	; RETI is in main program
