;
;  goal:  be able to pulse an IR led at 45khz, modulated
;  with codes from various remote controls.
;
;  period timing: 11uS LED on, 11uS LED off during xmit time (45.454kHz)
;                 22uS off during each blank period.
;
;  asumptions:  on time low value is pre-dec'd by 1 in table 
;               on time low =0 means done
;               off time low value is > 2
;               off time low value = 0 means last entry
;
;  1 - 1uS deviation unresolved
;

ptr     =    $00             ; number of bursts to perform
htim    =    $01             ; hold upper timing value
buffer  =    $6000           ;

        *=   $4000           ;

irsend  lda  $7f03           ;
        ora  #$10            ; ir xmitter
        sta  $7f03           ;
        lda  $7f01           ; turn xmitter off
        and  #$EF            ; 
        sta  $7f01           ;

        ldy  #$00            ; set up buffer pointer
        ldx  #$00            ; 
        sty  ptr             ;

cmd     lda  buffer,y        ; 4    get on timer (x must = 0 on entry here)
        sta  htim            ; 3    (23uS)  
        iny                  ; 2       
        lda  buffer,y        ; 4    get on timer
        beq  done            ; 2    check for end of data
        tax                  ; 2    get on time (pre decrement in table by 1)
        iny                  ; 2
        lda  $7f01           ; 4

loop1   ora  #$10            ; 2     turn led on
        sta  $7f01           ; 4     (22uS) 
        cpx  ptr             ; 3     nop
        nop                  ; 2
loop1a  and  #$EF            ; 2
        sta  $7f01           ; 4
        dex                  ; 2
        bne  loop1           ; 3  2  1 cycle short here

        ora  #$10            ; 2     turn led on    x=0
        sta  $7f01,x         ; 5     x=0 
        cpx  htim            ; 3     x=0
        beq  loop1b          ; 2
        and  #$EF            ; 2
        sta  $7f01           ; 4     turn led off 
        dec  htim            ; 5
        ora  #$10            ; 2     turn led on
        sta  $7f01           ; 4
        dex                  ; 2
        bra  loop1a          ; 3

loop1b  and  #$EF            ; 2     turn led off 
        sta  $7f01           ; 4     (12uS)

                             ; 1     for extra cycle used above
        lda  buffer,y        ; 4     get off timer 
        sta  htim            ; 3     (31uS)  (fixed below)
        iny                  ; 2       
        lda  buffer,y        ; 4    get on timer
        tax                  ; 2    get on time (pre decrement in table by 1)
        beq  done            ; 2
        iny                  ; 2
        beq  done            ; 2
        dex                  ; 2   compensate for calc time by sub 2
        dex                  ; 2   at least 3 ticks in counter to work!!!!!   
        nop                  ; 2 
        bra  loop2b          ; 3   reduced initial timing to comp for overages

loop2   nop                  ; 2
        nop                  ; 2
        nop                  ; 2
        lda  htim            ; 3 
        lda  htim            ; 3   
loop2a  lda  htim            ; 3  
loop2b  nop                  ; 2
        dex                  ; 2 
        bne  loop2           ; 3 2  2
        cpx  htim            ;   3  3
        beq  cmd             ;   2  3  +5 dev on jump to cmd, fixed by bra loop2b line
        dec  htim            ;   5  
        bra  loop2a          ;   3  

done    lda  $7f01           ; turn xmitter off
        and  #$EF            ; 
        sta  $7f01           ;
        lda  $7f03           ;
        and  #$EF            ; ir xmitter disable
        sta  $7f03           ;
        rts
