;
; originally by Ullrich von Bassewitz and Sidney Cadot
;
; clock_t clock (void);
; unsigned _clocks_per_sec (void);
;
;
; Modified for SBC-3 by Daryl Rictor
;

	.export		_clock, __clocks_per_sec
	.importzp	sreg

	.constructor	initclock
	.destructor	stopclock

	.include	"sbc3.inc"

.proc	_clock			; read the system vertical refresh counter
	lda	$0700FD
	sta	sreg+1		; save low byte
	lda	$0700FF
	sta	sreg
	lda	$0700FE
	tax
	lda	$0700FD
	cmp	sreg+1		; low byte changed during read, read again to prevent
	bne	_clock	      ; timer corruption
	stz	sreg+1
	rts
.endproc


.proc	__clocks_per_sec

	lda	$080030		; SBC-3 VDR
	and	#$08		; Video Mode 0=NTSC,1=PAL
	bne	@1
	lda	#60
	ldx	#0
	rts
	
@1:	lda	#50
	ldx	#0
	rts

.endproc

.proc	initclock

	lda	#$00		; zero the tick counter
	sta	$0700FD
	sta	$0700FE
	sta	$0700FF
	lda	$080030		; read VDR
	ora	#$20
	sta	$080030		; enable tick counter interrupts
	rts

.endproc

.proc	stopclock

	lda	$080030		; read VDR
	and	#$DF
	sta	$080030		; disable tick counter interrupts
	rts

.endproc
