; mainargs.s
;
; Ullrich von Bassewitz, 2003-03-07
; Based on code from Stefan A. Haubenthal <polluks@web.de>, 2003-11-08
; Greg King, 2003-05-18
; Stefan Haubenthal, 2005-01-07
; Oliver Schmidt, 2005-04-05
;
;
; Modified for SBC-3 by Daryl Rictor
;
; Scan a group of arguments that are in the input buffer.
; Build an array that points to the beginning of each argument.
; Send, to main(), that array and the count of the arguments.

; Command-lines look like these lines:
;
; EX200
; EX 200 
; EX 200 : arg1 " arg 2 is quoted "  arg3 "" arg5
;
; Leading and trailing spaces outside of quotes are ignored.

; TO-DO:
; Add a control-character quoting mechanism.

        .constructor	initmainargs, 18
        .import		__argc, __argv, __dos_type

        .include	"zeropage.inc"
        .include	"sbc3.inc"

; Maximum number of arguments allowed in the argument table.
; (An argument contains a comma, at least.)

MAXARGS = 10

; The SBC-3 input buffer holds 240 characters. I reserve 200 characters terminated by a zero for args.

BUF_LEN = 200

BUFFER := $070200

; Get possible command-line arguments. Goes into the special INIT segment,
; which may be reused after the startup code is run.

	.segment	"INIT"

initmainargs:

; Find the : 

	ldx	#$00
:	lda	BUFFER,x
	beq	done		; No "rem" -> no args
	inx
	cmp	#':'
	bne	:-

; create a copy of the command-line in a different buffer before the original is potentially
; destroyed.

	ldy	#$00
:	lda	BUFFER,x
	sta	buffer,y
	inx
	iny
	cpy	#BUF_LEN - 1	; Keep the terminating zero intact
	bcc	:-

; Start processing the arguments.

	ldx	#0
	ldy	#$01 * 2	; Start with argv[1]

next:	lda	buffer,x
	beq	done
	inx
	cmp	#' '		; Skip leading spaces
	beq	next

; Found start of next argument. We've incremented the pointer in X already, so
; it points to the second character of the argument. This is useful since we
; will check now for a quoted argument, in which case we will have to skip this
; first character.

	cmp	#'"'		; Is the argument quoted?
	beq	:+		; Jump if so
	dex			; Reset pointer to first argument character
	lda	#' '		; A space ends the argument
:	sta	tmp1		; Set end of argument marker

; Now store a pointer to the argument into the next slot.

	txa			; Get low byte
	clc
	adc	#<buffer
	sta	argv,y		; argv[y] = &arg
	iny
	lda	#$00
	adc	#>buffer
	sta	argv,y
	iny
	inc	__argc		; Found another arg

; Search for the end of the argument.

:	lda	buffer,x
	beq	done
	inx
	cmp	tmp1
	bne	:-

; We've found the end of the argument. X points one character behind it, and
; A contains the terminating character. To make the argument a valid C string,
; replace the terminating character by a zero.

	lda	#$00
	sta	buffer-1,x

; Check if the maximum number of command-line arguments is reached. If not,
; parse the next one.

	lda	__argc		; Get low byte of argument count
	cmp	#MAXARGS	; Maximum number of arguments reached?
	bcc	next		; Parse next one if not

; (The last vector in argv[] already is NULL.)

done:	lda	#<argv
	ldx	#>argv
	sta	__argv
	stx	__argv+1
	rts

; This array is zeroed before initmainargs is called.
; char* argv[MAXARGS+1] = {FNAM};

	.data

argv:	.res	MAXARGS * 2

	.bss

buffer:	.res	BUF_LEN
