;
; Ullrich von Bassewitz, 16.11.2002
;
; int open (const char* name, int flags, ...);	/* May take a mode argument */
;
; Be sure to keep the value priority of closeallfiles lower than that of
; closeallstreams (which is the high level C file I/O counterpart and must be
; called before closeallfiles).
;
; Modified for SBC-3 by Daryl Rictor
;

	.export		_open
	.destructor	closeallfiles, 17

	.import		addysp, popax
	.import		__oserror
	.import		_close
	.importzp	sp, tmp2, tmp3, ptr1

	.include	"errno.inc"
	.include	"fcntl.inc"
	.include	"filedes.inc"
	.include	"sbc3.inc"



;--------------------------------------------------------------------------
; closeallfiles: Close all open files.

.proc	closeallfiles

	ldx	#MAX_FDS-1
loop:	lda	fdtab,x
	beq	next		; Skip unused entries

; Close this file

	txa
	pha			; Save current value of X
	jsr	_close
	plx

; Next file

next:	dex
	bpl	loop
	rts

.endproc

;--------------------------------------------------------------------------
; _open

.proc	_open

; Throw away any additional parameters passed through the ellipsis

	dey			; Parm count < 4 shouldn't be needed to be...
	dey			; ...checked (it generates a c compiler warning)
	dey
	dey
	beq	parmok		; Branch if parameter count ok
	jsr	addysp		; Fix stack, throw away unused parameters

; Parameters ok. Pop the flags and save them into tmp3

parmok:	jsr	popax		; Get flags
	sta	tmp3

; Get a free file handle and remember it in tmp2

	jsr	freefd
	bcs	nofile
	stx	tmp2

; Check the flags. We cannot have both, read and write flags set, and we cannot
; open a file for writing without creating it.

	lda	tmp3
	and	#(O_RDWR | O_CREAT)
	cmp	#O_RDONLY	; Open for reading?
	beq	doread		; Yes: Branch
	cmp	#(O_WRONLY | O_CREAT)   ; Open for writing?
	bne	invflags	; No: Invalid open mode

; If O_TRUNC is set, scratch the file, but ignore any errors

	lda	tmp3
	and	#O_TRUNC
	bne	trunc

; Complete the the file name. Check for append mode here.

notrunc:
	lda	tmp3		; Get the mode again
	ldx	#$30		; append
	and	#O_APPEND	; Append mode?
	bne	append		; Branch if yes

trunc:	ldx	#$20		; write
append:	txa
	bne	common

; Read bit is set. Add an 'r' to the name

doread:	lda	#$10		; read only

; Common read/write code. Flags in A, handle in tmp2

common:	sta	tmp3		; mode
	lda	tmp2		; handle	
	and	#$0F
	ora     tmp3
	pha
	lda	#$C9		; open file
 	jsl	_sendidec
	pla
	jsl	_sendide	; handle & mode

; Get the filename from stack and send to IDE
	jsr	popax		; Get name
	sta	ptr1
	stx	ptr1+1		; Save pointer to name
	ldy	#$00
fnsnd:	lda	(ptr1),y
	beq	fopn	   
	jsl	_sendide
	iny
	bra	fnsnd
fopn:	jsl	_sendide	; send null term
ftst:	jsl	_idestat	; get status  
	cmp	#$00
	bpl	ftst		; wait for ready
	and	#$1F		; mask lower 5 bits
	bne	error		; system error	

; File is open. Mark it as open in the table
	ldx	tmp2
	lda	#$01		; read
	ldy	tmp3
	cpy	#$10
	beq	@1
	lda	#$02		; write     
@1:	sta	fdtab,x
; Done. Return the handle in a/x
	txa			; Handle
	ldx	#0
	rts

; Error entry: No more file handles

nofile:	lda	#$0E		; Too many open files

; Error entry. Error code is in A.

error:	sta	__oserror
errout:	lda	#$FF
	tax			; Return -1
	rts

; Error entry: Invalid flag parameter

invflags:
	lda	#EINVAL
	sta	__errno
	lda	#0
	sta	__errno+1
	bra	errout

.endproc
