;
; Ullrich von Bassewitz, 16.11.2002
;
; int read (int fd, void* buf, unsigned count);
;
;
; Modified for SBC-3 by Daryl Rictor
;
	.export		_read
	.constructor	initstdin

	.import		__oserror
	.import		rwcommon	
	.importzp		ptr1, ptr2, ptr3, tmp1, tmp2, tmp3

	.include		"fcntl.inc"
	.include		"filedes.inc"
	.include		"sbc3.inc"

;--------------------------------------------------------------------------
; initstdin: Open the stdin file descriptors for the keyboard
     
.segment  "INIT"

.proc	 initstdin

	lda	#SBC3_STDIN
	sta	fdtab+STDIN_FILENO
	rts		

.endproc

;--------------------------------------------------------------------------
; _read

.code

.proc  _read

; rwcommon set this up:
;  ptr1= count (unsigned int, # bytes to read)
;  ptr2= buffer pointer (bank 0 only)
;  ptr3= bytes read(unsigned int)
;  tmp2= file handle

	jsr	rwcommon		; Pop params, check handle
	bcs	errout		; Invalid handle, errno already set

; Check if the handle is valid and the file is open for writing

	tax
	lda	fdtab,x		; Get flags for this handle
	and	#$01			; File open for reading?
	beq	notopen

; Check the EOF flag. If it is set, don't read anything

	bit	fdtab,x		; Get flags for this handle
	bmi	eof
	bvs	kybrd			; STDIN

; Do file read
	; Check for zero count
	lda	ptr1
	ora	ptr1+1
	beq	chk1

; Read from device
	jsr	fget			; get chr from file
	bcs	eof			; end if file reached

chk1:	lda	ptr3
	ldx	ptr3+1
	rts
     
kybrd: 
	; Check for zero count
	lda	ptr1
	ora	ptr1+1
	beq	chk2

	; Read from kybrd
next2:
	jsl	_cin			; wait for keypress

	; Put char into buf
	ldy	#$00
	sta	(ptr1),y

	; Increment pointer
	inc	ptr2
	bne	:+
	inc	ptr2+1

	; Increment counter
:	inc	ptr3
	bne	chk2
	inc	ptr3+1

	; Check for counter less than count
chk2:	lda	ptr3
	cmp	ptr1
	bcc	next2
	ldx	ptr3+1
	cpx	ptr1+1
	bcc	next2

	; Return success, AX already set
	rts


; Return the number of chars read

eof:	cmp	#$0F			; EOF flag
	bne	error			; system error abort
	lda	ptr3
	ldx	ptr3+1
	rts

; Error entry, file is not open

notopen:
	lda     #$08		; File not open

; Error entry, status not ok

error:
	sta	__oserror
errout:
	lda	#$FF
	tax				; Return -1
	rts

fget:					; read char from file 
	lda	#$CA			; checkc for eof
	jsl	_sendidec		; read cmd
	lda	tmp2			; handle
	jsl	_sendide		;
	lda	ptr1
	jsl	_sendide		;
	lda	ptr1+1
	jsl	_sendide		;
	lda	#$00
	jsl	_sendide		;
	lda	#$00
	jsl	_sendide		;
	nop
loop:
	jsl	_rxide		;
	ldy	#$00
	sta	(ptr2),y
	rep	#$20
	inc	ptr2
	inc	ptr3
	dec	ptr1
	sep	#$20
	bne	loop
	nop
:	jsl	_idestat		;
	cmp	#$00
	bpl	:-			; wait for ready
	and	#$1F
	clc
	beq	:+
	sec				; mark error
:	rts						

.endproc
