;
; Ullrich von Bassewitz, 16.11.2002
;
; Modified for SBC-3 by Daryl Rictor
;

;
; int write (int fd, const void* buf, unsigned count);
;
	.export		_write
	.constructor	initstdout

	.import		__oserror
	.import		rwcommon
	.importzp	sp, ptr1, ptr2, ptr3

	.include	"fcntl.inc"
	.include	"filedes.inc"
	.include	"sbc3.inc"


;--------------------------------------------------------------------------
; initstdout: Open the stdout and stderr file descriptors for the screen.

.segment	"INIT"

.proc	initstdout

	lda	#SBC3_STDOUT
	sta	fdtab+STDOUT_FILENO
	sta	fdtab+STDERR_FILENO
	rts	

.endproc

;--------------------------------------------------------------------------
; _write
                      
.code

.proc	_write

	jsr     rwcommon        ; Pop params, check handle
	bcs     errout          ; Invalid handle, errno already set

	tax			; save handle
	lda	ptr1
	EOR	#$FF
	sta	ptr1
	lda	ptr1+1
	EOR	#$FF
	sta	ptr1+1		; Remember -count-1
	inc	ptr1
	bne	:+
	inc	ptr1+1		; add 1 to count


; Check if the handle is valid and the file is open for writing

:	lda	fdtab,x		; Get flags for this handle
	and	#$02		; File open for writing?
	beq	notopen

	bit	fdtab,x		; Get flags for this handle
	bvs	console		; STDOUT
		
; Output the next character from the buffer

@L0:	ldy	#0
	lda	(ptr2),y
	inc	ptr2
	bne	@L1
	inc	ptr2+1		; A = *buf++;
@L1:	jsr	fout
	bcs	error		; Bail out on errors

; Count characters written

	inc	ptr3
	bne	@L2
	inc	ptr3+1

; Decrement count

@L2:	inc	ptr1
	bne	@L0
	inc	ptr1+1
	bne	@L0

; Wrote all chars
; Return the number of chars written

	lda	ptr3
	ldx	ptr3+1
	rts

console:
@L0:	ldy	#0
	lda	(ptr2),y
	inc	ptr2
	bne	@L1
	inc	ptr2+1		; A = *buf++;
@L1:	jsl	 _cout

; Count characters written

	inc	ptr3
	bne	@L2
	inc	ptr3+1

; Decrement count

@L2:	inc	ptr1
	bne	@L0
	inc	ptr1+1
	bne	@L0

; Wrote all chars, close the output channel
; Return the number of chars written

	lda	ptr3
	ldx	ptr3+1
	rts
	
; Error entry, file is not open
notopen:
	lda	#$08		; File not open

; Error entry, status not ok

error:	sta	__oserror
errout:	lda	#$FF
	tax			; Return -1
	rts

fout:
	pha
	lda	#$CB		; write file
	jsl	_sendidec	; read cmd
	txa			; handle
	jsl	_sendide	;
	lda	#$01
	jsl	_sendide	;
	lda	#$00
	jsl	_sendide	;
	lda	#$00
	jsl	_sendide	;
	lda	#$00
	jsl	_sendide	;
	pla
	jsl	_sendide	;
:	jsl	_idestat	; status
	cmp	#$00
	bpl	:-		; wait for ready
	and	#$1F
	clc
	beq	:+
	sec
:	rts

.endproc




