;*********************************************************
;
;  65C02 Composite Video Timing Demonstration
;                    v1.0  
;              by Daryl Rictor   
;          http://65c02.tripod.com/
;
;  This program will demonstrate how to generate an 
;  NTSC compatible B&W non-interlaced composite video
;  signal using a 65C02 MPU clocked at 2.0MHz, with a
;  simple D-A converter and buffer.  See the schematic
;  labeled "c-video.gif" for more details.
;
;  A PAL version could also be demonstrated by changing 
;  the timing loops. 
;  
;  This program does NOT generate a character display.
;  It generates 24 vertical bars alternating from 
;  grey-white-black.  
;  
;  The video signal is generated in software and outputed 
;  to a 65C22 VIA on Ports PA0 & PA1 (pins 2 & 3).  Here,
;  the digital bit pattern is converted to the analog 
;  voltages required to generate the display.
;
;  Bit 0 is the SYNC bit, Bit 1 is the DATA bit
;
;  (Compiled successfully using TASS)
;
;*********************************************************

SYNC	=  $00		; 0.0v
BLACK	=  $01		; 0.3v
GREY	=  $02		; 0.7v
WHITE	=  $03		; 1.0v

VIDEO	=  $7F51		; bits 0 & 1 of VIA1 Port A

CNTR	=  $C0		; 1 byte line counter

	*= $1000		; start anywhere

Init	
	LDA  #$FF		; Set VIA1 PORT A as all outputs
	STA  $7F53		; VIA1 Port A = output port
	LDA  #BLACK		; Set value to 0.3v
	STA  VIDEO		; set initial level to black
;
; start accurate cycle counting
; each line is 64uS long (actual NTSC timing should be 63.5uS)
; each field is 262 lines long (16768uS) (actual interlaced NTSC count is 262.5 lines)
; 6 pre-equalization pulses are 2uS @ sync, 30uS @ black (lines 1-3, 2 per line)
; 6 Vertical Sync pulses are 30uS @ Sync, 2uS @ black (lines 4-6, 2 per line)
; 6 post-equalization pulses are 2uS @ sync, 30uS @ black (lines (7-9, 2 per line)
; H Sync is 4.5uS @ Sync, 6.5uS @ black (actual NTSC Hsync is 4.7uS and 10.9uS blanking)
; Video out is 53uS from lines (10-261)
; the last line is 53uS sent at 0.3v (Black) to allow for housekeeping of the software) 
;
;
VPRE	; pre equalization pulses (6) 

	STZ  VIDEO		; 4 do eq pulse
;-----------------------
;32uS
	STA  VIDEO		; 4 (4 clocks = 2uS)
	LDX  #$0B		; 2 - 30uS black level
VPRE1	DEX			; 2
	BNE  VPRE1		; 3,2   (loop 60=5x + 6, x@10 = 56
	STZ  VIDEO		; 4 do eq pulse
;-----------------------
;32us
	STA  VIDEO		; 4 (4 clocks = 2uS)
	LDX  #$0B		; 2 - 30uS black level
VPRE2	DEX			; 2
	BNE  VPRE2		; 3,2   (loop 60=5x + 6, x@10 = 56
	STZ  VIDEO		; 4 do eq pulse
;-----------------------
;32us
	STA  VIDEO		; 4 (4 clocks = 2uS)
	LDX  #$0B		; 2 - 30uS black level
VPRE3	DEX			; 2
	BNE  VPRE3		; 3,2   (loop 60=5x + 6, x@10 = 56
	STZ  VIDEO		; 4 do eq pulse
;-----------------------
;32us
	STA  VIDEO		; 4 (4 clocks = 2uS)
	LDX  #$0B		; 2 - 30uS black level
VPRE4	DEX			; 2
	BNE  VPRE4		; 3,2   (loop 60=5x + 6, x@10 = 56
	STZ  VIDEO		; 4 do eq pulse
;-----------------------
;32us
	STA  VIDEO		; 4 (4 clocks = 2uS)
	LDX  #$0B		; 2 - 30uS black level
VPRE5	DEX			; 2
	BNE  VPRE5		; 3,2   (loop 60=5x + 6, x@10 = 56
	STZ  VIDEO		; 4 do eq pulse
;-----------------------
;32us
	STA  VIDEO		; 4 (4 clocks = 2uS)
	LDX  #$0B		; 2 - 30uS black level
VPRE6	DEX			; 2
	BNE  VPRE6		; 3,2   (loop 60=5x + 6, x@10 = 56

VSYNC	; Verticle Sync pulse (6)

	STZ  VIDEO		; 4 do vsync pulse
;-----------------------
;32us
	LDX  #$0B		; 2 - 30uS sync level
VSYN1	DEX			; 2
	BNE  VSYN1		; 3,2   (loop 60=5x + 6, x@10 = 56
	STA  VIDEO		; 4 (4 clocks = 2uS)
	STZ  VIDEO		; 4 do vsync pulse
;-----------------------
;32us
	LDX  #$0B		; 2 - 30uS sync level
VSYN2	DEX			; 2
	BNE  VSYN2		; 3,2   (loop 60=5x + 6, x@10 = 56
	STA  VIDEO		; 4 (4 clocks = 2uS)
	STZ  VIDEO		; 4 do vsync pulse
;-----------------------
;32us
	LDX  #$0B		; 2 - 30uS sync level
VSYN3	DEX			; 2
	BNE  VSYN3		; 3,2   (loop 60=5x + 6, x@10 = 56
	STA  VIDEO		; 4 (4 clocks = 2uS)
	STZ  VIDEO		; 4 do vsync pulse
;-----------------------
;32us
	LDX  #$0B		; 2 - 30uS sync level
VSYN4	DEX			; 2
	BNE  VSYN4		; 3,2   (loop 60=5x + 6, x@10 = 56
	STA  VIDEO		; 4 (4 clocks = 2uS)
	STZ  VIDEO		; 4 do vsync pulse
;-----------------------
;32us
	LDX  #$0B		; 2 - 30uS sync level
VSYN5	DEX			; 2
	BNE  VSYN5		; 3,2  (loop 60=5x + 6, x@10 = 56
	STA  VIDEO		; 4 (4 clocks = 2uS)
	STZ  VIDEO		; 4 do vsync pulse
;-----------------------
;32us
	LDX  #$0B		; 2 - 30uS sync level
VSYN6	DEX			; 2
	BNE  VSYN6		; 3,2  (loop 60=5x + 6, x@10 = 56
	STA  VIDEO		; 4 (4 clocks = 2uS)

VPST	; post equalization pulses (6)?
	STZ  VIDEO		; 4 do eq pulse
;-----------------------
;32us
	STA  VIDEO		; 4 (4 clocks = 2uS)
	LDX  #$0B		; 2 - 30uS black level
VPST1	DEX			; 2
	BNE  VPST1		; 3,2   (loop 60=5x + 6, x@10 = 56
	STZ  VIDEO		; 4 do eq pulse
;-----------------------
;32us
	STA  VIDEO		; 4 (4 clocks = 2uS)
	LDX  #$0B		; 2 - 30uS black level
VPST2	DEX			; 2
	BNE  VPST2		; 3,2   (loop 60=5x + 6, x@10 = 56
	STZ  VIDEO		; 4 do eq pulse
;-----------------------
;32us
	STA  VIDEO		; 4 (4 clocks = 2uS)
	LDX  #$0B		; 2 - 30uS black level
VPST3	DEX			; 2
	BNE  VPST3		; 3,2   (loop 60=5x + 6, x@10 = 56
	STZ  VIDEO		; 4 do eq pulse
;-----------------------
;32us
	STA  VIDEO		; 4 (4 clocks = 2uS)
	LDX  #$0B		; 2 - 30uS black level
VPST4	DEX			; 2
	BNE  VPST4		; 3,2   (loop 60=5x + 6, x@10 = 56
	STZ  VIDEO		; 4 do eq pulse
;-----------------------
;32us
	STA  VIDEO		; 4 (4 clocks = 2uS)
	LDX  #$0B		; 2 - 30uS black level
VPST5	DEX			; 2
	BNE  VPST5		; 3,2   (loop 60=5x + 6, x@10 = 56
	STZ  VIDEO		; 4 do eq pulse
;-----------------------
;32us
	STA  VIDEO		; 4 (4 clocks = 2uS)
	LDX  #$09		; 2 - 30uS black level
VPST6	DEX			; 2
	BNE  VPST6		; 3,2   (loop 60=5x + 6, x@8 = 46
      LDA  #$FD		; 2   (252 lines, $FD=252+1)
	STA  CNTR		; 3 save line counter
	STA  CNTR		; 3 NOP
	NOP			; 2

HSYNC	; Horzontal Sync and blanking

	STZ  VIDEO		; 4 start HSYNC pulse
;-----------------------
;11us
	LDX  #00		; 2 setup
	LDA  #BLACK		; 2 Set value to 0.3v
	STA  VIDEO,x	; 5 9 clks = 4.5uS (ideal is 4.7)
				;  H Blanking = 11uS (10.9 ideal)
	LDA  #BLACK		; 2 Set value to 0.3v
	LDX  #GREY		; 2 Set value to 0.7v
	LDY  #WHITE		; 2 Set value to 1.0v
	DEC  CNTR		; 5 dec line counter
	BEQ  LAST		; 2,3 if 0, do last line as all black 
;-----------------------
;53us

DISLN	; display data 
;
; This code will display 24 vertical bars alternating grey, white, & black
;
	STX  VIDEO		; 4 0.7v Grey       
	STY  VIDEO		; 4 1.0v White
	STA  VIDEO		; 4 0.3v Black
	STX  VIDEO		; 4 0.7v Grey       
	STY  VIDEO		; 4 1.0v White
	STA  VIDEO		; 4 0.3v Black
	STX  VIDEO		; 4 0.7v Grey       
	STY  VIDEO		; 4 1.0v White
	STA  VIDEO		; 4 0.3v Black
	STX  VIDEO		; 4 0.7v Grey       
	STY  VIDEO		; 4 1.0v White
	STA  VIDEO		; 4 0.3v Black
	STX  VIDEO		; 4 0.7v Grey       
	STY  VIDEO		; 4 1.0v White
	STA  VIDEO		; 4 0.3v Black
	STX  VIDEO		; 4 0.7v Grey       
	STY  VIDEO		; 4 1.0v White
	STA  VIDEO		; 4 0.3v Black
	STX  VIDEO		; 4 0.7v Grey       
	STY  VIDEO		; 4 1.0v White
	STA  VIDEO		; 4 0.3v Black
	STX  VIDEO		; 4 0.7v Grey       
	STY  VIDEO		; 4 1.0v White
	STA  VIDEO		; 4 0.3v Black
      BIT  $00		; 3 3 clk NOP	
      JMP  HSYNC		; 3 do next line  
;-----------------------
;52.5uS (105 clks)
LAST ; Do last line as all black...
      LDX  #$10		; 2
LST1	DEX			; 2
	BNE  LST1		; 3,2   (5x + 6 = 105:  x@15 = 81
	LDA  $7F71 		; 4 --- Serial port status
	AND  #$08         ; 2 --- mask rcvr full bit
	BEQ  Nokey        ; 3
	LDA  $7F70		; --- get keystroke from buffer
      RTS			; --- exit program.
Nokey	LDA  #BLACK		; 2 Set value to 0.3v
	NOP			; 2 use 6 clocks
	NOP			; 2
	NOP			; 2
	JMP  VPRE		; 3 do the next field
;     STZ  VIDEO		; 4 (shown here for reference only)

