;/******************************************************************************/
;/  NTSC/PAL Video Display (40 character x 25 line monochrome text)					/
;/  8 bit parallel load with latch and busy handshake lines 					/
;/																				/
;/  Designed and written by Daryl Rictor (c)2003-2004							/ 
;/******************************************************************************/

;******************************************************************************
; include ATMega8 definitions & program specific definitions and macros
;******************************************************************************
.include  "m8def.inc"
.include  "defs.inc"
;******************************************************************************
; Code Begins at 0x000
; System IRQ jump table from 0x0000 - 0x0012
;******************************************************************************
	rjmp 	RESET 				; Reset Handler
	rjmp	EXT_INT0 			; IRQ0 Handler
	rjmp 	EXT_INT1 			; IRQ1 Handler
	rjmp 	TIM2_COMP 			; Timer2 Compare Handler
	rjmp 	TIM2_OVF 			; Timer2 Overflow Handler
	rjmp 	TIM1_CAPT 			; Timer1 Capture Handler
	rjmp 	TIM1_COMPA 			; Timer1 CompareA Handler
	rjmp 	TIM1_COMPB 			; Timer1 CompareB Handler
	rjmp 	TIM1_OVF 			; Timer1 Overflow Handler
	rjmp 	TIM0_OVF 			; Timer0 Overflow Handler
	rjmp 	SPI_STC 			; SPI Transfer Complete Handler
	rjmp 	USART_RXC 			; USART RX Complete Handler
	rjmp 	USART_UDRE 			; UDR Empty Handler
	rjmp 	USART_TXC	 		; USART TX Complete Handler
	rjmp 	ADCC 				; ADC Conversion Complete Handler
	rjmp 	EE_RDY	 			; EEPROM Ready Handler
	rjmp 	ANA_COMP 			; Analog Comparator Handler
	rjmp 	TWSI 				; Two-wire Serial Interface Handler
	rjmp 	SPM_RDY 			; Store Program Memory Ready Handler
	
;******************************************************************************
; unused IRQ vector's point to software reset code
;******************************************************************************

EXT_INT0:
EXT_INT1:
TIM2_COMP:
TIM2_OVF:
TIM1_CAPT:
;TIM1_COMPA:
;TIM1_COMPB:
TIM1_OVF:
TIM0_OVF:
SPI_STC:
USART_RXC:
USART_UDRE:
USART_TXC:
ADCC:
EE_RDY:
ANA_COMP:
TWSI:
SPM_RDY:

;******************************************************************************
; Program Starts here on Reset
;******************************************************************************
RESET:							; first line executed after reset 
	CLI							; disable interupts

;******************************************************************************
; Initialize the I/O Ports
;******************************************************************************
; port B pins (video data bus & host status input)
	ldi		J, 0x00				; data 0
	out		portb, J			; set port b outputs to 0
	LDI		J, 0x3F				; pins 0 - 5 of port B
	out		ddrb, J				; set DDRB pins 0 - 5 to outputs

; port C pins (control, handshake, and video data)
	SBI		portc, syncpin		; turn on sync bit (done after mode select below)
	SBI		portc, ldsrpin		; turn shift load bit on 
	cbi		portc, PC2			; 
	cbi		portc, PC3			; turn bits 2&3 off
	SBI		portc, ackdpin		; set ack pin to inactive
	LDI		J, 0x1F				; pins 0 - 4 of port C
	out		ddrc, J				; set DDRC pins 0 - 4 to outputs

; port D pins (host data input bus)
	LDI		J, 0x00				; all 8 pins of port D
	out		ddrd, J				; set DDRD pins 0-7 to inputs
	out		portd, J			; disc pullup's

; Load Shift Register with all 0' so display remains blank
	CBI		portc, ldsrpin		; set SR to LOAD mode
	SBI		portc, ldsrpin		; set SR to SHIFT mode

;******************************************************************************
; Select PAL/NTSC System
;******************************************************************************

	SBI		portc, ackdpin		; add pull up resistor to pin C4
	ldi		J, 0x0F				; set pin C4 as input
	out		ddrc, J				;
	nop							; delay (wait for ext pull down resistor...)
	nop							; ""
	nop							; ""
	nop							; delay (if PAL is selected...) 
	nop							; ""
	nop							; ""
	nop							; delay (to bring this line low)
	nop							; ""
	nop							; ""
	nop							; delay 
	nop							; ""
	nop							; ""
	sbis	pinc, ackdpin		; is Pin C4 low?
	rjmp	SETPAL				; Yes, set PAL mode

SETNTSC:						; no, set NTSC mode
	ldi		J, 0xD6				; VSYNC end value
	mov		ocrbv, J			;
	ldi		J, 0xe9				; Active end time
	mov		ocrba, J			;
	ldi		J, 0x06				; 0x0106 Last vertical line 262
	mov		linetop, J			;
	ldi		J, 0x28				; first vertical line of active disp
	mov		line1, J			;
	ldi		J, 0xF0				; last verical line of active disp
	mov		line200, J			;
	ldi		J, 0x00				; timer value to start of active disp line
	mov		leftside, J			;
	ldi		K, 0xf8				; 0x03f8 = 1016 clocks per line (63.5us)
	rjmp	Restore_c4			;
	
SETPAL:
	ldi		J, 0xDE				; VSYNC end value
	mov		ocrbv, J			;
	ldi		J, 0xF1				; Active end time
	mov		ocrba, J			;
	ldi		J, 0x38				; 0x0138 Last vertical line 312
	mov		linetop, J			;
	ldi		J, 0x37				; first vertical line of active disp
	mov		line1, J			;
	ldi		J, 0xFF				; last verical line of active disp
	mov		line200, J			;
	ldi		J, 0x02				; timer value to start of active disp line
	mov		leftside, J			;
	ldi		K, 0x00				;  0x0400 = 1024 clocks per line		

RESTORE_C4:
	ldi		J, 0x1F				; set Pin C4 to output
	out		ddrc, J				;
	SBI		portc, ackdpin		; raise Pin C4 to hi level

;******************************************************************************
; Initialize Timer 1 
;******************************************************************************
;   OCR1A will hold the value of 1016/1024 and be used to reset the counter and 
;        cause an IRQ that will start the HSYNC process
;   OCR1B will cause an IRQ that will be used to prepare the CPU for the
;        HSYNC and put it into sleep mode to ensure accurate timing.
;		 (also used to end VSYNC signal)
;   ICR1 will not be used
;   The counter will count each clock cycle (16MHz - no prescaling)

	ldi     J, 0x00
	out     TCCR1A, J
	ldi     J, 0x09				; set timer 1 to CTC mode 4
	out     TCCR1B, J		

	ldi     J, 0x03
	cpi		K, 0x00
	brne	initt1
	ldi		J, 0x04
initt1:
	out     OCR1AH, J        
	out     OCR1AL, K			; set Counter 1 to cycle from 0 to 1016 (1024 PAL) and IRQ

	ldi		J, 0x03				; 
	out     OCR1BH, J        	; 
	out     OCR1BL, ocrbv		; VSYNC IRQ

	ldi     J, 0x18				; allow OCR1A & OCR1B IRQ's
	out     TIMSK, J			

;******************************************************************************
; Init other CPU I/O registers
;******************************************************************************
	ldi     J, 0x80
	out     ACSR, J				; turn off comparator

	ldi     J, 0x80
	out		MCUCR, J			; enable sleep mode

	ldi		J, 0x04				; set stack pointer to top of SRAM 
	out		SPH, J				; (24 bytes max stack size)
	ldi		J, 0x5f				; uses 6 bytes (IRQA-2, IRQB-2, ProcChr sub-2
	out		SPL, J				;

;******************************************************************************
; Initialize the Display SRAM
;******************************************************************************
; fill display SRAM with the Startup Banner 
	ldi		ZL, 0x00
	ldi		ZH, 0x0C			; set to start of Banner in Prog Mem
	ldi		YL, 0x60
	ldi		YH, 0x00			; set to start of SRAM
	ldi		XL, 0xE8
	ldi		XH,	0x03			; set X to 1000
	ldi		J, 0x20				; 
frloop:
	lpm		J, Z+				; get boot image
	st 		Y+, J				; save to SRAM			
	sbiw	XL, 1				; dec X
	brne	frloop				; do until X=0

;******************************************************************************
; Initialize Program Variables
;******************************************************************************
	ldi		chrln, 0x00			; keeps track of font gen line (0-7)
	ldi		vlineh, 0x00		; vertical line counter 0-262
	ldi		vline, 0x00			;  ""
	ldi		YL, 0x60			;
	ldi		YH, 0x00			; init y pointer
	ldi		hpos, 0x00			; keeps track of horizontal chr pos (0-39)
	ldi		vpos, 0x00			; keeps track of vertical chr pos (0-24)
	ldi		Cursclk, 0x00		; init cursor blink timer
	ld		Curschr, Y			; init cursor character
	ldi		Cursctl, 0x5F		; init cursor '_' character
	ldi		J,	0x00			; 0
	mov		fnthl, J			; set high/low flag to low
	mov		fntsel, J			; set Font to primary

;******************************************************************************
;  Reset Timer 1 and clear Busy Status Flag
;******************************************************************************
	ldi     J, 0x00				; clear timer 1
	out     TCNT1H, J			; 
	out     TCNT1L, J			; 

	ldi		J, 0x1C				; clear timer 1 IRQ's
	out		TIFR, J				; clear timer 1 IRQ's

	CBI		portc, ackdpin		; clear the ACK line to the SBC 
	SBI		portc, ackdpin		; 
								; 
	SEI							; Ready to go - enable system IRQ's - GO!!!

BANNER:
	cpi		CursClk, 0xFF		; 
	brne	Banner				; 
	inc		Cursclk				; reset to 0 for blink mode
;	ldi		inpt, 0x0C			; ^L 
;	rcall	ProcChr				; Clear the screen
		
;******************************************************************************
; Main Loop 
;******************************************************************************
Main:
	cpi		Cursctl, 0x00		; is the cursor turned on?
	breq	main2				; no, skip control code
	cpi		Cursclk, 0xff		; is clk at 0xFF?
	breq	main1				; yes - solid cursor (no blink)
	cpi		Cursclk, 0x1C		; is it above 28 frames?
	brpl	main2				; if >25, show Cursor
	st		Y, Curschr			; if clr, show character
	rjmp	main3				; 
main1:
	st		Y, Cursctl			; if set show cursor
	rjmp	main3
main2:
	st		Y, Cursctl			; if set show cursor
	cpi		Cursclk, 0x37		; is it >56 
	brmi	main3				; no
	clr		Cursclk				; reset timer
main3:
	sbis	pinc, dflgpin		; test host data input & get it if ready
	rjmp	Main				; if no data, then update the cursor
	st		Y, Curschr			; restore chr under cursor
	in		inpt, pind			; save data
	rcall	ProcChr				; process the host data
	cbi		portc, ackdpin		; 
	sbi		portc, ackdpin		; clear D FF and tell host we're ready for next cmd
	rjmp	Main				; repeat

;******************************************************************************
; IRQ Service Routines
;******************************************************************************

TIM1_COMPB:						; IRQ service for OCR1B 
	in		savsr, SREG			; save the status register
	sbi		portc, syncpin		; raise the sync pin (its low during VSYNC, else no effect)
	sbi		portc, ackdpin		; raise dack pin (if it was low just before irq) keep host from waiting...
	sei							; enable IRQ before we sleep so OCR1A can work
	sleep						; wait for TIM1_COMPA IRQ
	out		SREG, savsr			; restore the status register
	reti						; all done, go back to Main Prgm

TIM1_COMPA:						; IRQ service for OCR1A 
	.include "vidgen.inc"		; file that holds the video generation code	
	reti						; done

;******************************************************************************
; include subroutines and Constant Data tables
;******************************************************************************
.include "ProcChr.inc"			; terminal emulation subroutines 0x500-0x5FF
.include "banner.inc"			; opening screen banner 0x600-0x7F3
.include "primfont.inc"			; disp PRIMARY font 0x800-0xBFF
.include "altfont.inc"			; disp ALT font 0xC00-0xFFF

;******************************************************************************
; End of Program
;******************************************************************************

